/***************************************************************************
 *   Copyright (C) 2007 by Jeroen van der Waal                             *
 *   jwaal@kabelfoon.nl                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>

#include "averageframerate.h"

/*!
    \class AverageFrameRate
    \brief Determines the framerate per second (fps) of the QDicomImageWidget
    \author Jeroen van der Waal <jwaal@kabelfoon.nl>

    Dicom files with a number of frames are played like a movie. Sometimes
    (due to slow hardware e.g.) the QDicomImageWidget can not perform for
    the required number of frames per second(fps). The class measures the fps
    and can be used to give feedback to the user.
*/

extern int SDR_VERBOSE;

///constructor
AverageFrameRate::AverageFrameRate()
  : m_qtime()
  , m_averageFrameRate(0.0f)
  , m_measurements()
  , m_totalMeasurements(0)
  , m_isMeasuring(false)
{
  //empty
}

/*!
    Call this method when the measuring is to start.
*/
void AverageFrameRate::startMeasuring()
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "AverageFrameRate::startMeasuring\n");
  m_averageFrameRate = 0.0f;
  m_totalMeasurements = 0;
  m_isMeasuring = true;
  m_qtime.start();
}

/*!
    When a frame is decoded and shown on the screen, call this method.
*/
void AverageFrameRate::nextFrameReady()
{
  if(!m_isMeasuring)
  {
    return;
  }
  
  int index = m_totalMeasurements % MAX_MEASUREMENTS;
  m_measurements[index] = m_qtime.elapsed();
  m_totalMeasurements++;
  
  int storedMeasurements = qMin((int)MAX_MEASUREMENTS, m_totalMeasurements);
  int totalTimeInMsec = 0;
  for(int i = 0; i < storedMeasurements; i++)
  {
    totalTimeInMsec += m_measurements[i];
  }
  if(totalTimeInMsec)
  {
    m_averageFrameRate = (float)(storedMeasurements * 1000) /
                         (float)(totalTimeInMsec);
  }
  m_qtime.restart();
}

/*!
    Returns the average frame rate as calculated.
    by this class.
    \return average framerate
*/
float AverageFrameRate::averageFrameRate() const
{
  return m_averageFrameRate;
}

/*!
    When ready with measuring, call the stopMeasuring() method.
*/
void AverageFrameRate::stopMeasuring()
{
  m_isMeasuring = false;
  m_averageFrameRate = 0.0f;
}
