/***************************************************************************
 *   Copyright (C) 2007 by Jeroen van der Waal                             *
 *   jwaal@kabelfoon.nl                                                    *
 *                                                                         *
 *   with minor distortions by Johannes A. Koeppen                         *
 *   koeppen@ieee.org                                                      *
 *                                                                         *
 ***************************************************************************/

//qt includes
#include <QPixmap>
#include <QVector>
#include <QFile>
#include <QFileInfo>
#include <QMessageBox>
#include <cstdio>

#include "globals.h"
#include "dcmIconListModel.h"

extern int SDR_VERBOSE;

/*!
   \class CIconData
   \brief Locally used class for managing thumbnail related data.
*/

CDcmIconListModel::CDcmIconListModel( QObject * parent )
  : QAbstractListModel(parent)
  , m_data(0)
  , m_showAllAfterLoadFile(true)
  , m_filename()
  , m_dirname()
{
  //empty
}

/*!
    Destruct object, destroys all data
*/
CDcmIconListModel::~CDcmIconListModel()
{
  clear();
}

/*!
    Load a DICOMDIR file into the widget. The file will be parsed
    and thumbnails will be retrieved and stored.
    \param dicomdirFileName
*/
bool CDcmIconListModel::loadDir( const QString &dirName )
{
  clear();
  QDir qdir(dirName);
  if(!qdir.exists())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stderr, "No directory opened.\n");
    return false;
  }
  
  m_filename = dirName;
  qdir.setFilter(QDir::Files | QDir::NoSymLinks);
  qdir.setNameFilters(dcm_suffixes);
  //qdir.setSorting(QDir::Time);
  qdir.setSorting(QDir::NoSort);
  QFileInfoList dir_list = qdir.entryInfoList();
  
  for (int i = 0; i < dir_list.size(); ++i)
  {
  
    QFileInfo fileInfo = dir_list.at(i);
    QString abs_path = fileInfo.absoluteFilePath();
    if (!isDcmFilename(abs_path))
      continue;
    if (fileInfo.isDir())
      continue;
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "CDcmIconListModel::loadDir calls new MinDcmImage\n");
    MinDcmImage *midi = new MinDcmImage(abs_path, true);
    if (midi->m_success)
    {
      m_data.push_back(midi);
    }
    else
    {
      if (SDR_VERBOSE > 0)
        fprintf(stderr, "Image not valid.\n");
      return false;
    }
  }
  selectAll();
  return true;
}


//const unsigned long CIF_AcrNemaCompatibility         = 0x0000001;

/// accept wrong palette attribute tags
//const unsigned long CIF_WrongPaletteAttributeTags    = 0x0000002;

/// element pixel data may be detached if it is no longer needed by dcmimage
//const unsigned long CIF_MayDetachPixelData           = 0x0000004;

/// use presentation state instead of 'built-in' LUTs & overlays
//const unsigned long CIF_UsePresentationState         = 0x0000008;

/// don't convert YCbCr (Full and Full 4:2:2) color images to RGB
//const unsigned long CIF_KeepYCbCrColorModel          = 0x0000010;

/// take responsibility for the given external DICOM dataset, i.e. delete it on destruction
//const unsigned long CIF_TakeOverExternalDataset      = 0x0000020;

/// ignore modality transformation (rescale slope/intercept or LUT) stored in the dataset
//const unsigned long CIF_IgnoreModalityTransformation = 0x0000040;

/// ignore third value of the modality LUT descriptor, determine bit depth automatically
//const unsigned long CIF_IgnoreModalityLutBitDepth    = 0x0000080;

void CDcmIconListModel::tst(QString qfn)
{
  QByteArray ba = qfn.toLatin1();
  const char *fname = ba.data();
  
  // register RLE decompression codec
  DcmRLEDecoderRegistration::registerCodecs(OFTrue, OFFalse);
  DJDecoderRegistration::registerCodecs(EDC_photometricInterpretation,
                                        EUC_default,
                                        EPC_default,
                                        OFTrue);
                                        
  DcmFileFormat *dfile = new DcmFileFormat();
  OFCondition cond = dfile->loadFile(fname, EXS_Unknown, EGL_withoutGL,
                                     DCM_MaxReadLength);
                                     
  if (cond.good())
  {
    E_TransferSyntax xfer = dfile->getDataset()->getOriginalXfer();
    unsigned long compatibilityMode = CIF_MayDetachPixelData
                                      | CIF_TakeOverExternalDataset;
    //| CIF_AcrNemaCompatibility;
    //DicomImage *di = new DicomImage(dfile, xfer, compatibilityMode,
    // okay for PALETTE COLOR
    //DicomImage *di = new DicomImage(dfile, xfer, CIF_UsePresentationState,
    //DicomImage *di = new DicomImage(dfile, xfer, CIF_WrongPaletteAttributeTags,
    DicomImage *di = new DicomImage(dfile, xfer, compatibilityMode,
                                    //DicomImage *di = new DicomImage(dfile, xfer, CIF_TakeOverExternalDataset,
                                    //DicomImage *di = new DicomImage(dfile, xfer, CIF_IgnoreModalityTransformation,
                                    //DicomImage *di = new DicomImage(dfile, xfer, CIF_IgnoreModalityLutBitDepth,
                                    0, 1);
    if (di == NULL)
      if (SDR_VERBOSE > 0)
        fprintf(stderr, "Out of memory\n");
        
    if (di->getStatus() == EIS_Normal)
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,"di->getStatus() == EIS_Normal\n");

      if (SDR_VERBOSE > 2)
      {
        FILE *ofile = NULL;
        ofile = fopen("sdr.pnm", "wb");
        bool result = di->writePPM(ofile, 8, 0);
        //bool result = di->writeRawPPM(ofile, 8, 0);
        if (! result)
          fprintf(stderr, "error while wrighting pnm file\n");
        if (ofile == NULL)
            fprintf(stderr, "(ofile == NULL\n");

        fclose(ofile);
      }
    }
    else
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,"di->getStatus() != EIS_Normal\n");
    }
  }
  DcmRLEDecoderRegistration::cleanup();
  // deregister JPEG decompression codecs
  DJDecoderRegistration::cleanup();
}

bool CDcmIconListModel::loadStringList(QStringList dcm_filename_list)
{
  clear();
  
  for (int i = 0; i < dcm_filename_list.size(); ++i)
  {
    QString abs_path = dcm_filename_list.at(i);
    tst(abs_path);
    
    MinDcmImage *midi = new MinDcmImage(abs_path, true);
    if (midi->m_success)
    {
      m_data.push_back(midi);
    }
    else
    {
      QByteArray ba = abs_path.toLatin1();
      const char *fname = ba.data();
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "loadStringList, cannot get image in file\n'%s'.\n",
                fname);
      return false;
    }
  }
  selectAll();
  return true;
}


void CDcmIconListModel::selectAll( )
{
  m_selected.clear();
  reset();
  m_selected = m_data;
}

///Property to show all the available thumbnails in the DICOMDIR file
bool CDcmIconListModel::showAfterLoadFile( ) const
{
  return m_showAllAfterLoadFile;
}

///Property to show all the available thumbnails in the DICOMDIR file
void CDcmIconListModel::setShowAfterLoadFile(bool p_show)
{
  m_showAllAfterLoadFile = p_show;
}


/*!
    Returns the number of items to show.
*/
int CDcmIconListModel::rowCount( const QModelIndex & ) const
{
  return m_selected.size();
}

/*!
    Return the data for index and role
*/
QVariant CDcmIconListModel::data( const QModelIndex & index, int role ) const
{
  if (!index.isValid())
    return QVariant();
    
  if (index.row() >= m_data.size())
    return QVariant();
    
  if (role == Qt::DisplayRole)
    return m_selected[index.row()]->label();
  if(role == Qt::DecorationRole)
    return m_selected[index.row()]->icon();
  if(role == ReferenceFileIDRole)
    return m_selected[index.row()]->file_ID();
  else
    return QVariant();
}

///clear all data
void CDcmIconListModel::clear()
{
  const MinDcmImage* data;
  foreach(data, m_data)
    delete data;
  m_data.clear();
  m_selected.clear();
}

