/**************************************************************************
 *  Author      : Johannes A. Koeppen, koeppen@ieee.org                   *
 *  Year(s)     : 2010 - 2015                                             *
 *  File        : globals.cpp                                               *
 *  Context     : Sdr2 (Not approved for clinical use)                    *
 *  License     : GPL 2 or later                                          *
 *  Description : global functions                                        *
 *                                                                        *
 **************************************************************************/

#include "globals.h"

#include <math.h>
#include <QString>
#include <QRegExp>
#include <QChar>
#include <QFileInfo>

extern int SDR_VERBOSE;

inline double roundd(double val)
{
  return floor(val + 0.5);
}

inline float roundf(float val)
{
  return floor(val + 0.5);
}

int pad_to_8_16_32_64(int in)
{
  int ret = 0;
  if (in < 9)
    ret = 8;
  else if (in > 8 && in < 17)
    ret = 16;
  else if (in > 16 && in < 33)
    ret = 32;
  else if (in > 32 && in < 65)
    ret = 64;
    
  return ret;
}


int qStringToDoubleArray(QString str, int size, double *double_array)
{

  int pos, double_counter;
  bool is;
  double d;
  QString msg, part;
  QRegExp rx("[-+]?[0-9]*\\.?[0-9]+([eE][-+]?[0-9]+)?");
  rx.setPatternSyntax(QRegExp::RegExp2);
  pos = 0;
  double_counter = 0;
  
  while ((pos = rx.indexIn(str, pos)) != -1)
  {
    if (size == double_counter)
      break;
      
    part = str.mid(pos, rx.matchedLength());
    //list.append(part);
    pos += rx.matchedLength();
    
    d = part.toDouble(&is);
    if (is)
    {
      double_array[double_counter] = d;
      double_counter++;
    }
    else
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "ERROR in qStringToDoubleArray\n");
    }
  }
  
  return double_counter;
}


int qStringToIntArray(QString str, int size, int *int_array)
{
  int pos, int_counter;
  bool is;
  int i;
  QString msg, part;
  QRegExp rx("[-+]?[0-9]*\\?([eE][-+]?[0-9]+)?");
  rx.setPatternSyntax(QRegExp::RegExp2);
  
  pos = 0;
  int_counter = 0;
  
  while ((pos = rx.indexIn(str, pos)) != -1)
  {
    if (size == int_counter)
      break;
      
    part = str.mid(pos, rx.matchedLength());
    pos += rx.matchedLength();
    
    i = part.toInt(&is);
    if (is)
    {
      int_array[int_counter] = i;
      int_counter++;
      
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "qStringToIntArray found int %i\n", i);
    }
  }
  return int_counter;
}


void printMat44(mat44 M)
{
  for (int row = 0; row < 4; row++)
  {
    fprintf(stdout, "row %i: <%.4f, %.4f, %.4f, %.4f>\n",
            row, M.m[row][0], M.m[row][1], M.m[row][2], M.m[row][3]);
  }
}

void printNiftDatatype(int dt)
{
  if (dt == 0)
    fprintf (stdout, "DT_UNKNOWN or DT_NONE\n");
  else if (dt == 1)
    fprintf (stdout, "DT_BINARY\n");
  else if (dt == 2)
    fprintf (stdout, "DT_UNSIGNED_CHAR or DT_UINT8\n");
  else if (dt == 4)
    fprintf (stdout, "DT_SIGNED_SHORT or DT_INT16\n");
  else if (dt == 8)
    fprintf (stdout, "DT_SIGNED_INT or DT_INT32\n");
  else if (dt == 16)
    fprintf (stdout, "DT_FLOAT or DT_FLOAT32\n");
  else if (dt == 32)
    fprintf (stdout, "DT_COMPLEX or DT_COMPLEX64\n");
  else if (dt == 64)
    fprintf (stdout, "DT_DOUBLE or DT_FLOAT64\n");
  else if (dt == 128)
    fprintf (stdout, "DT_RGB or DT_RGB24\n");
  else if (dt == 255)
    fprintf (stdout, "DT_ALL ???\n");
  else if (dt == 256)
    fprintf (stdout, "DT_INT8\n");
  else if (dt == 512)
    fprintf (stdout, "DT_UINT16\n");
  else if (dt == 768)
    fprintf (stdout, "DT_UINT32\n");
    
  else if (dt == 768)
    fprintf (stdout, "DT_UINT32\n");
  else if (dt == 1024)
    fprintf (stdout, "DT_INT64\n");
  else if (dt == 1280)
    fprintf (stdout, "DT_UINT64\n");
  else if (dt == 1536)
    fprintf (stdout, "DT_FLOAT128\n");
  else if (dt == 1792)
    fprintf (stdout, "DT_COMPLEX128\n");
  else if (dt == 2048)
    fprintf (stdout, "DT_COMPLEX256\n");
  else if (dt == 2304)
    fprintf (stdout, "DT_RGBA32\n");
  else
    fprintf (stdout, "unknown nifti code\n");
}


bool isDcmFilename(QString s)
{
  for (int j = 0; j < (dcm_suffixes.size()); j++)
  {
    QString suf = dcm_suffixes.at(j);
    if (s.endsWith(suf))
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "isDcmFilename suffix ok!\n");
      return true;
      //break;
    }
  }
  
  if (s.contains("."))
  {
    QFileInfo fifo(s);
    QString suff = fifo.suffix();
    bool numeric = true;
    //int si = suff.toInt(&numeric);
    
    if (numeric)
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "file name has numeric suffix, potentially a dicom file\n");
      return true;
    }
    else
    {
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "isDcmFilename: suffix is neither numeric nor usual dicom suffix, do'nt use!\n");
      return false;
    }
  }
  else
  {
    return true;
  }
  
  // not reached
  return false;
}


int
uniqueIntInIntArray(int check, const int *intArray, int num)
{
  int r = 0;
  for (int i = 0; i < num; i++)
  {
    int a = intArray[i];
    if (a == check)
    {
      r++;
    }
  }
  return r;
};

