/**************************************************************************
 *  Author      : Johannes A. Koeppen, koeppen@ieee.org                   *
 *  Year(s)     : 2010 - 2015                                             *
 *  File        : mainwindow.cpp                                          *
 *  Context     : Sdr2 (Not approved for clinical use)                    *
 *  License     : GPL 2 or later                                          *
 *  Description : main window, mainly Qt stuff,                           *
 *                basically three splitters and few buttons               *
 *                                                                        *
 **************************************************************************/

//#include <stdlib.h>

#include <QListView>
#include <QSplitter>
#include <QHeaderView>
#include <QFileSystemModel>
#include <QFileInfo>
#include <QMessageBox>
#include <QPushButton>
#include <QBoxLayout>
#include <QtGui/QLabel>
#include <QDebug>
#include <QSettings>

#include "globals.h"
#include "mainwindow.h"
#include "qdicomdiriconwidget.h"
#include "qdicomimageForm.h"

extern int SDR_VERBOSE;

MainWindow::MainWindow(QWidget *parent, Qt::WFlags flags)
  : QMainWindow(parent, flags)
{
  dicomPath = QDir::currentPath();
  clipboard = QApplication::clipboard();
  
  if (dicomPath.endsWith("debug", Qt::CaseInsensitive))
  {
    dicomPath = dicomPath.remove((dicomPath.length() - 6), 6);
  }
  else if (dicomPath.endsWith("release", Qt::CaseInsensitive))
  {
    dicomPath = dicomPath.remove((dicomPath.length() - 8), 8);
  }
  dicomPath.append("/DICOMDIR");
  
  QSplitter *centralSplitter = new QSplitter();
  
  QSplitter *leftSplitter = new QSplitter();
  leftSplitter->setOrientation(Qt::Vertical);
  leftSplitter->setMidLineWidth(4);
  leftSplitter->setMaximumWidth(280);
  
  QSplitter *rightSplitter = new QSplitter();
  rightSplitter->setOrientation(Qt::Vertical);
  rightSplitter->setMidLineWidth(4);
  
  QWidget *dirWidget = new QWidget();
  QVBoxLayout *dirLayout = new QVBoxLayout;
  
  QLabel *select_dir_label = new QLabel();
  select_dir_label->setText("Search in folder and 1st subfolder(s) for DICOM files:");
  dirLayout->addWidget(select_dir_label);
  
  QHBoxLayout *dirOptionsHorizontalLayout = new QHBoxLayout;
  dirOptionsHorizontalLayout->setSpacing(6);
  dirImportPushButton = new QPushButton("Select Folder");
  dirImportPushButton->setObjectName(QString::fromUtf8("dirImportPushButton"));
  dirImportPushButton->setEnabled(false);
  QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
  dirImportPushButton->setSizePolicy(sizePolicy);
  dirImportPushButton->setMaximumSize(QSize(120, 40));
  dirOptionsHorizontalLayout->addWidget(dirImportPushButton);
  
  currentDcmDir = QDir::homePath();
  
  readSettings();
  
  dir_model = new QFileSystemModel;
  dir_model->setFilter(QDir::Filters(QDir::AllDirs | QDir::NoDotAndDotDot));
  dir_model->setRootPath(QDir::currentPath());
  
  dir_tree = new QTreeView();
  dir_tree->setModel(dir_model);
  dir_tree->setColumnWidth ( 0, 280);
  dir_tree->setColumnHidden(1, true);
  dir_tree->setColumnHidden(2, true);
  dir_tree->setIndentation(24);
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MainWindow::MainWindow past dir_tree\n");
    
  //if (QFile::exists(dicomPath))
  QDir qd(dicomPath);
  if (qd.isReadable())
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "dicomPath is readable\n");
    QModelIndex dir_index = dir_model->index(dicomPath);
    dir_tree->setCurrentIndex(dir_index);
  }
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MainWindow::MainWindow past dir_tree->scrollTo\n");
    
  miniDcmXmlWidget = new MiniDcmXmlWidget();
  miniDcmXmlWidget->openXmlDefault();
  
  dirLayout->addWidget(dir_tree);
  dirLayout->addLayout(dirOptionsHorizontalLayout);
  
  dirWidget->setLayout(dirLayout);
  
  leftSplitter->addWidget(dirWidget);
  leftSplitter->addWidget(miniDcmXmlWidget);
  
  qdif = new Ui_qdicomimageForm();
  qdif->setupUi(rightSplitter);
  
  rightSplitter->addWidget(qdif->layoutWidget);
  rightSplitter->setContentsMargins(0, 0, 12, 6);
  
  centralSplitter->addWidget(leftSplitter);
  
  QWidget *dicomIconsParentLayoutWidget = new QWidget;
  QVBoxLayout *vBoxLayoutIconViewParent = new QVBoxLayout(dicomIconsParentLayoutWidget);
#ifndef Q_OS_MAC
  vBoxLayoutIconViewParent->setSpacing(6);
#endif
  qdicomDirIconWidget = new QDicomDirIconWidget(dicomIconsParentLayoutWidget);
  
  vBoxLayoutIconViewParent->addWidget(qdicomDirIconWidget);
  
  QHBoxLayout *hBoxLayoutSaveButtonParent = new QHBoxLayout();
  
#ifndef Q_OS_MAC
  hBoxLayoutSaveButtonParent->setSpacing(6);
#endif
  
  QSpacerItem *spacerSave = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
  
  hBoxLayoutSaveButtonParent->addItem(spacerSave);
  QHBoxLayout *hBoxLayoutSave = new QHBoxLayout();
#ifndef Q_OS_MAC
  hBoxLayoutSave->setSpacing(6);
#endif
#ifndef Q_OS_MAC
  hBoxLayoutSave->setContentsMargins(0, 0, 0, 0);
#endif
  
  labelSelectImage = new QLabel("Please select an image.");
  labelSelectImage->setEnabled(false);
  vBoxLayoutIconViewParent->addWidget(labelSelectImage);
  pushButtonSaveVolume = new QPushButton(dicomIconsParentLayoutWidget);
  pushButtonSaveVolume->setText("Save Series of Selected Image");
  pushButtonSaveVolume->setEnabled(false);
  hBoxLayoutSave->addWidget(pushButtonSaveVolume);
  hBoxLayoutSaveButtonParent->addLayout(hBoxLayoutSave);
  vBoxLayoutIconViewParent->addLayout(hBoxLayoutSaveButtonParent);
  
  centralSplitter->addWidget(dicomIconsParentLayoutWidget);
  centralSplitter->addWidget(rightSplitter);
  setCentralWidget(centralSplitter);
  
  //rightSplitter->setFrameShape(QFrame::VLine); //  HLine);
  //leftSplitter->setStyleSheet("margin-left: 10px");
  
  dockSaveVolume = new SaveVolume();
  dockSaveVolume->setMinimumWidth(360);
  dockSaveVolume->setMinimumHeight(200);
  addDockWidget(Qt::BottomDockWidgetArea, dockSaveVolume);
  dockSaveVolume->hide();
  
  setWindowIcon(QPixmap(":/images/oives_16x16.png"));
  setWindowTitle("SDR2");
  
  errorMessage = new QErrorMessage(this);
  
  //#ifndef QT_NO_STATUSBAR
  //    QStatusBar *statusBar(); // const;
  //    void setStatusBar(QStatusBar *statusbar);
  //#endif
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MainWindow::MainWindow, make connections\n");
    
  connect(dir_tree, SIGNAL(clicked(QModelIndex)),
          this, SLOT(checkSelectedDir(QModelIndex)));
  connect(dirImportPushButton, SIGNAL(clicked()), this, SLOT(importDir()));
  connect(miniDcmXmlWidget->miniDcmXmlTree, SIGNAL(signalDcmPreview()),
          this, SLOT(previewDcmSeries()));
  connect(qdicomDirIconWidget, SIGNAL(imageSelected(MinDcmImage*)),
          (qdif->qdicomImageWidget), SLOT(loadMinDcmImage(MinDcmImage*)));
  connect(qdicomDirIconWidget, SIGNAL(imageSelected(MinDcmImage*)),
          this, SLOT(pushButtonSaveVolumeEnable(MinDcmImage*)));
  connect(qdicomDirIconWidget, SIGNAL(imageClicked(MinDcmImage*)),
          this, SLOT(pushButtonSaveVolumeEnable(MinDcmImage*)));
  connect(pushButtonSaveVolume, SIGNAL(clicked()),
          this, SLOT(dockSaveVolumeShow()));
  connect(qdif->qdicomImageWidget, SIGNAL(dcmImageChanged()),
          this, SLOT(pushButtonSaveVolumeDisable()));
  connect(qdif->qdicomImageWidget, SIGNAL(setSeriesWindowing(double, double)),
          qdicomDirIconWidget, SLOT(resetSeriesWindowing(double, double)));
          
  checkDcmtkDictionary();
  
  //statusBar()->showMessage("UI ready", 5000);
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "eo MainWindow::MainWindow\n");
}


MainWindow::~MainWindow()
{
  //empty;
}

//void MainWindow::createStatusBar()
//{
//  statusBar()->showMessage("Ready", 1500);
//}

void MainWindow::checkDcmtkDictionary()
{
  if (SDR_VERBOSE > 0)
  {
    //dcmDataDict = new GlobalDcmDataDictionary(true, false);
#ifdef DCM_DICT_ENVIRONMENT_VARIABLE
    fprintf(stderr, "DCM_DICT_ENVIRONMENT_VARIABLE is defined.\n");
#else
    fprintf(stderr, "DCM_DICT_ENVIRONMENT_VARIABLE is not defined.\n");
#endif
    
    char* pDict;
    pDict = getenv ("DCMDICTPATH");
    if (pDict != NULL)
    {
      fprintf (stdout, "Environment variable 'DCMDICTPATH' is: %s.\n", pDict);
    }
    else
    {
      fprintf(stdout, "Environment variable 'DCMDICTPATH' is not set.\n");
    }
  }
  
  if (!dcmDataDict.isDictionaryLoaded())
  {
    if (SDR_VERBOSE > 0)
    {
      fprintf(stderr, "Warning: no data dictionary loaded, check environment variable: %s\n",
              DCM_DICT_ENVIRONMENT_VARIABLE);
      fprintf (stdout, "dcmDataDictionary is not loaded, "
               "gone load local './dicom.dic'\n");
    }
    /*
    dcmDataDict.clear();
    DcmDataDictionary& d = dcmDataDict.wrlock();
    bool dic = d.loadDictionary("./dicom.dic");
    if (dic)
    {
      fprintf(stdout, "LOCAL dic is loaded.\n");
    }
    else
    {
      fprintf(stdout, "LOCAL dic is not loaded.\n");
    }
    dcmDataDict.unlock();
    */
    
    QMessageBox msgBox;
    msgBox.setWindowTitle("Sdr2 Critical issue");
    msgBox.setIcon(QMessageBox::Critical);
    msgBox.setText("Warning: No data dictionary loaded!\n\n"
                   "Either DCMTK is compiled without embedded dictionary,\n"
                   "or the environment variable 'DCMDICTPATH' \ndoesn't point to a valid 'dicom.dic'.\n\n"
                   "It is likely that DICOM files will be unreadable.");
    msgBox.setInformativeText("Do you want to exit Sdr2?");
    msgBox.setStandardButtons(QMessageBox::Ignore | QMessageBox::Abort);
    msgBox.setButtonText(QMessageBox::Abort, "Exit ");
    msgBox.setDefaultButton(QMessageBox::Ignore);
    int ret = msgBox.exec();
    
    switch(ret)
    {
      case QMessageBox::Ignore:
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "case QMessageBox::Ignore:.\n");
          
        break;
      }
      case QMessageBox::Abort:
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "case QMessageBox::Abort.\n");
        exit(0);
        break;
      }
      default:
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "default.\n");
        break;
      }
    }
  }  // eo if (!dcmDataDict.isDictionaryLoaded())
}


int MainWindow::countFiles(const QString &path, int limit, bool countDirs)
{
  QFileInfo finfo(path);
  if(!finfo.exists() || !finfo.isDir())
    return 0;
  int res = 0;
  QStringList slist = finfo.absoluteDir().entryList(QDir::Dirs | QDir::NoSymLinks |
                      QDir::NoDotAndDotDot);
  foreach(QString path, slist)
  {
    QFileInfo childInfo(path);
    if(childInfo.isDir())
    {
      if(countDirs)
        res++;
      res += countFiles(path, countDirs);
    }
    else
    {
      res++;
    }
    if ((limit != 0) && (res > limit))
      return limit;
  }
  return res;
}


void MainWindow::readSettings()
{
  QSettings settings("Neuranse", "SDR2");
  
  QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
  
  QSize size = settings.value("size", QSize(400, 400)).toSize();
  resize(size);
  move(pos);
  
  QString path = "";
  path = settings.value("dicomPath").toString();
  if (! path.isEmpty())
  {
    this->dicomPath = path;
  }
}


void MainWindow::writeSettings()
{
  QSettings settings("Neuranse", "SDR2");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  
  if (QFile::exists(dicomPath))
  {
    settings.setValue("dicomPath", dicomPath);
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MainWindow::writeSettings, dicomPath not saved.\n");
  }
}


void MainWindow::previewDcmSeries()
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo MainWindow::previewDcmSeries\n");
    
  labelSelectImage->setEnabled(true);
  qdif->qdicomImageWidget->clearDcmImage();
  
  qdicomDirIconWidget->loadStringList(miniDcmXmlWidget
                                      ->miniDcmXmlTree->imageList);
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "eo MainWindow::previewDcmSeries\n");
}

// study, serie, each file is an acquisition
// memo:
//    As others point out, first you need the row and column direction
//        cosines from (0020,0037) Image Orientation (Patient).
//    getMajorAxisFromPatientRelativeDirectionCosine(double x,
//                                                   double y,
//                                                   double z)
//    Get a label describing the major axis from a unit vector (direction cosine) as found in ImageOrientationPatient.
//    Some degree of deviation from one of the standard orthogonal axes is allowed before deciding no major axis applies and returning null.
//
//    returns:    the string describing the orientation of the vector, or null if oblique

void MainWindow::checkSelectedDir(QModelIndex current_dir_index)
{
  QString spath;
  bool dcmInDir = false;
  bool subdirsInDir = false;
  bool dcmInSubdirs = false;
  
  spath = dir_model->filePath(current_dir_index);
  
  currentDcmDir = QDir(spath);
  currentDcmDir.setFilter(QDir::Files | QDir::NoDotAndDotDot | QDir::NoSymLinks);
  QFileInfoList info_list = currentDcmDir.entryInfoList();
  
  for (int i = 0; i < info_list.size(); ++i)
  {
    QFileInfo fi;
    fi = info_list.at(i);
    QString s = fi.absoluteFilePath();
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "MainWindow::checkSelectedDir, calls isDcmFilename\n");
    if (isDcmFilename(s))
    {
      MinDcmImage *midi = new MinDcmImage(s, false);
      if (midi->m_success)
      {
        dcmInDir = true;
      }
      delete midi;
    }
    
    if (dcmInDir)
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "MainWindow::checkSelectedDir, dcm found in dir\n");
      break;
    }
  } // eo for (int i = 0; i < info_list.size(); ++i)
  
  if (! dcmInDir)
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "MainWindow::checkSelectedDir, checking in subdirs now.\n");
      
    currentDcmDir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
    QFileInfoList info_list = currentDcmDir.entryInfoList();
    for (int i = 0; i < info_list.size(); ++i)
    {
      QFileInfo fi;
      fi = info_list.at(i);
      if (fi.isDir())
      {
        subdirsInDir = true;
        QString path = fi.absoluteDir().absolutePath();
        if (one_proven_dcm_file_in_subdirs(path))
        {
          dcmInSubdirs = true;
          break;
        }
      }
    }
  }
  
  if ((! dcmInDir) && (! dcmInSubdirs))
  {
    dirImportPushButton->setEnabled(false);
    if (subdirsInDir)
    {
      showNoDicomMessage(spath, 2);
    }
    else
    {
      showNoDicomMessage(spath, 1);
    }
  }
  else
  {
    dirImportPushButton->setEnabled(true);
  }
}


bool MainWindow::one_proven_dcm_file_in_subdirs(QString parent_dir_str)
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo one_proven_dcm_file_in_subdirs\n");
  int count = 0;
  
  QDir pdir(parent_dir_str);
  
  if(!pdir.exists())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stderr, "No directory opened.\n");
    return false;
  }
  pdir.setFilter(QDir::Dirs | QDir::NoSymLinks | QDir::NoDotAndDotDot);
  //pdir.setNameFilters(dcm_suffixes);
  QFileInfoList pdir_list = pdir.entryInfoList();
  
  for (int i = 0; i < pdir_list.size(); ++i)
  {
    QFileInfo fileInfo = pdir_list.at(i);
    QString sub_dir_name = fileInfo.absoluteFilePath();
    QDir sdir(sub_dir_name);
    sdir.setFilter(QDir::Files);
    
    QFileInfoList sdir_list = sdir.entryInfoList();
    
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "one_proven_dcm_file_in_subdirs\n");
    for (int i = 0; i < sdir_list.size(); ++i)
    {
      QFileInfo fileInfo = sdir_list.at(i);
      QString file_name = fileInfo.absoluteFilePath();
      if (isDcmFilename(file_name))
      {
        MinDcmImage *midi = new MinDcmImage(file_name, false);
        if (midi->m_success)
        {
          if (SDR_VERBOSE > 2)
            fprintf(stdout,
                    "MainWindow::one_proven_dcm_file_in_subdirs, midi->m_success\n");
          count++;
        }
        delete midi;
        
      }
      if (count > 0)
      {
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "One proven_dcm_file_in_subdirs found.\n");
        return true;
      }
    }
  }
  return false;
}


bool MainWindow::dcm_in_subdirs(QString parent_dir_str)
{
  int count = 0;
  
  QDir pdir(parent_dir_str);
  
  if(!pdir.exists())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stderr, "No directory opened.\n");
    return false;
  }
  pdir.setFilter(QDir::Dirs | QDir::NoSymLinks | QDir::NoDotAndDotDot);
  //pdir.setSorting(QDir::Size );
  //pdir.setNameFilters(dcm_suffixes);
  QFileInfoList pdir_list = pdir.entryInfoList();
  for (int i = 0; i < pdir_list.size(); ++i)
  {
    QFileInfo fileInfo = pdir_list.at(i);
    QString sub_dir_name = fileInfo.absoluteFilePath();
    QDir sdir(sub_dir_name);
    sdir.setFilter(QDir::Files);
    sdir.setNameFilters(dcm_suffixes);
    
    QFileInfoList sdir_list = sdir.entryInfoList();
    
    for (int i = 0; i < sdir_list.size(); ++i)
    {
      QFileInfo fileInfo = sdir_list.at(i);
      QString file_name = fileInfo.fileName();
      if (isDcmFilename(file_name))
        count++;
    }
    
    if (count > 0)
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "dcm_in_subdirs true.\n");
      return true;
    }
    else
    {
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "dcm_in_subdirs = 0.\n");
    }
  }
  return false;
}


void MainWindow::importDir()
{
  QByteArray ba = currentDcmDir.absolutePath().toLatin1();
  char *dname = ba.data();
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo MainWindow::importDir: '%s'\n",
            dname);
            
#ifndef QT_NO_CURSOR
  QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
  
  int num_dcm_files;
  currentDcmDir.setFilter(QDir::Files);
  QFileInfoList info_list = currentDcmDir.entryInfoList();
  num_dcm_files = info_list.size();
  
  for (int i = 0; i < info_list.size(); ++i)
  {
    QFileInfo fi = info_list.at(i);
    QString s = fi.fileName();
    if (!isDcmFilename(s))
    {
      info_list.removeOne(s);
      num_dcm_files--;
    }
  }
  
  if (num_dcm_files > 0)
  {
    miniDcmXmlWidget->parseDcmDir(currentDcmDir.absolutePath());
#ifndef QT_NO_CURSOR
    QApplication::setOverrideCursor(Qt::ArrowCursor);
#endif
    return;
  }
  
  if (dcm_in_subdirs(currentDcmDir.absolutePath()))
    miniDcmXmlWidget->parseDcmDir(currentDcmDir.absolutePath());
    
#ifndef QT_NO_CURSOR
  QApplication::setOverrideCursor(Qt::ArrowCursor);
#endif
  
}


void MainWindow::dockSaveVolumeHide()
{
  dockSaveVolume->hide();
}
void MainWindow::dockSaveVolumeShow()
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo MainWindow::dockSaveVolumeShow\n");
  dockSaveVolume->setOrigDcmImageVector(&qdicomDirIconWidget->m_model->m_data);
  dockSaveVolume->sorted = false;
  dockSaveVolume->setFloating(true);
  dockSaveVolume->show();
}


void MainWindow::pushButtonSaveVolumeEnable(MinDcmImage *selectedDcmImage)
{
  pushButtonSaveVolume->setEnabled(true);
  labelSelectImage->setEnabled(true);
  dockSaveVolume->setReferenceDcmImage(selectedDcmImage);
}


void MainWindow::pushButtonSaveVolumeDisable()
{
  pushButtonSaveVolume->setEnabled(false);
  dockSaveVolumeHide();
}

void MainWindow::showNoDicomMessage(QString folder, int msgText)
{
  errorMessage->setWindowTitle("Sdr2 Issue");
  
  QString msg("");
  if (msgText == 1)
  {
    msg = QString("No readable Dicom files found in folder \n '%1'.")
          .arg(folder);
  }
  else if (msgText == 2)
  {
    msg = QString("No readable Dicom files found in folder \n '%1'"
                  "\nand its first subfolder(s).")
          .arg(folder);
  }
  
  errorMessage->showMessage(msg);
}


void
MainWindow::closeEvent(QCloseEvent *event)
{
  writeSettings();
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MainWindow::closeEvent\n");
    
  event->accept();
}

