/**************************************************************************
 *  Based on Jeroen van der Waal's code (2007)                            *
 *  jwaal@kabelfoon.nl                                                    *
 *                                                                        *
 *  Modifications: Johannes A. Koeppen, koeppen@ieee.org                  *
 *  Year(s)      : 2010 - 2015                                            *
 *  File         : minDcmImage.cpp                                        *
 *  Context      : Sdr2 (Not approved for clinical use)                   *
 *  License      : GPL 2 or later                                         *
 *  Description  : minimal DICOM image                                    *
 *                                                                        *
 **************************************************************************/


#include <iostream>

#include <QDebug>
#include <QVector>
#include <QMessageBox>

#include "globals.h"
#include "minDcmImage.h"

#ifdef HAVE_GUSI_H
#include <GUSI.h>
#endif

extern int SDR_VERBOSE;

#include <dcmtk/dcmdata/dctk.h>          /* for various dcmdata headers */

#ifdef DEBUG
#include <dcmtk/dcmdata/dcdebug.h>       /* for SetDebugLevel */
#endif

//#include <dcmtk/dcmdata/dcdeftag.h>    /* for dicom.dic */
#include <dcmtk/dcmdata/dcuid.h>         /* for dcmtk version name */
#include <dcmtk/dcmdata/dcrledrg.h>      /* for DcmRLEDecoderRegistration */

//#include <dcmtk/dcmimgle/dcmimage.h>    /* for DicomImage */
#include <dcmtk/dcmimgle/digsdfn.h>       /* for DiGSDFunction */
#include <dcmtk/dcmimgle/diciefn.h>       /* for DiCIELABFunction */

//#include <dcmtk/ofstd/ofconapp.h>       /* for OFConsoleApplication */
#include <dcmtk/dcmimage/diregist.h>      /* include to support color images */
//#include "dcmtk/ofstd/ofstd.h>          /* for OFStandard */

#ifdef SUPPORT_JPEG_LS
#include <dcmtk/dcmjpeg/djdecode.h>       /* for dcmjpeg decoders */
#include <dcmtk/dcmjpeg/dipijpeg.h>       /* for dcmimage JPEG plugin */
#endif

#ifdef WITH_LIBTIFF
#include <dcmtk/dcmimage/dipitiff.h>      /* for dcmimage TIFF plugin */
#endif

#ifdef WITH_LIBPNG
#include <dcmtk/dcmimage/dipipng.h>       /* for dcmimage PNG plugin */
#endif


#ifdef WITH_ZLIB
#include <zlib.h>                         /* for zlibVersion() */
#endif



//! Locally used class to manage a color table for gray level images
/*!
Creates and destroys a color table for gray level QImages having
QImage::Format_Indexed8.
The class is designed to improve the performance. Only one color table
can be shared among all gray level images. The table needs only to be
setup once.
*/

class QGrayColorTable
{
public:
  typedef enum {GRAY_LEVELS = 256};
  typedef QVector<QRgb> TColorTable;
  
  /**
  * Initializes the table holding gray levels for 8bit images.
  */
  QGrayColorTable()
    : m_colorTable(GRAY_LEVELS)
  {
    //color table for monochroom images.
    for(unsigned int i = 0; i < GRAY_LEVELS; i++)
    {
      m_colorTable[i] = qRgb(i, i, i);
    }
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "Color table with gray levels created.\n");
  }
  
  ~QGrayColorTable()
  {
    m_colorTable.clear();
  }
  
  /**
  * access the table
  * @return the color table holding 256 different shades of gray
  */
  const TColorTable& operator()() const
  {
    return m_colorTable;
  }
  
private:
  TColorTable m_colorTable;
};


/*!
\brief locally used class to manage (de)registration of dcmtk codecs.

Dcmtk requires registration and deregistration of codecs. This class
is to be created before using the QDicomImage class and is to be
destroyed after the last use of qdicomimage.
*/
class DecoderRegistration
{
public:
  /**
  * Register codecs
  */
  DecoderRegistration()
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "DecoderRegistration()\n");
    DJDecoderRegistration::registerCodecs(EDC_always,
                                          EUC_default, EPC_default, OFTrue);
    DcmRLEDecoderRegistration::registerCodecs(OFFalse, OFFalse);
  }
  
  
  /**
  * Deregister codecs
  */
  ~DecoderRegistration()
  {
    DJDecoderRegistration::cleanup();
    //DcmRLEDecoderRegistration::cleanup();
  }
};


MinDcmImage::MinDcmImage(const QString& filename, bool getImage)
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo MinDcmImage::MinDcmImage.\n");
    
  char str[1024];

  p_frame = 0;
  QByteArray ba = filename.toLatin1();
  const char *fname = ba.data();
  
  m_dicomFileFormat = new DcmFileFormat();
  const unsigned long fstart = 0;
  const unsigned long fcount = 0;
  m_success = true;
  
  //one color table shared among all images
  static const QGrayColorTable m_grayColorTable;

  OFCondition cond = m_dicomFileFormat->loadFile(fname,
                     EXS_Unknown,
                     EGL_withoutGL,
                     DCM_MaxReadLength,
                     ERM_autoDetect);
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MinDcmImage, condition is %s\n",
            cond.text());
            
  if (loadDicomImg(fname))
  {
    m_image_absolute_filepath = filename.toAscii();
    
    if (SDR_VERBOSE > 2)
      fprintf(stdout, " --- bo read header --- \n");
      
    DcmObject *dset = m_dicomFileFormat;
    dset = m_dicomFileFormat->getDataset();
    
    DcmTagKey searchKey;
    DcmDataset *dataset = m_dicomFileFormat->getDataset();
    searchKey.set(0x0002, 0x0010);
    if (! getElement(dataset, searchKey,
                     &m_transfer_syntax_uid))
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "m_transfer_syntax_uid not found \n");
    }
    
    searchKey.set(0x0008, 0x0018);
    if (! getElement(dataset, searchKey,
                     &m_sop_instance_uid))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "sop_instance_uid not found.\n");
        
    }
    
    searchKey.set(0x0010, 0x0010);
    if (! getElement(dataset, searchKey,
                     &m_patient_name))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "patient_name not found.\n");
      m_patient_name = QString("Patient name unknown");
    }
    else
    {
      if (m_patient_name.trimmed().length() < 2)
      {
        m_patient_name = QString("Patient name unknown");
      }
      else
      {
        m_patient_name = m_patient_name.replace(QString("^"), QString(" "));
        m_patient_name = m_patient_name.simplified();
      }
    }
    
    searchKey.set(0x0010, 0x0030);
    if (! getElement(dataset, searchKey,
                     &m_pat_birth_date))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "birth_date not found.\n");
        
      m_pat_birth_date = QString("unknown");
    }
    else
    {
      m_pat_birth_date = dcmDateTodmyString(m_pat_birth_date);
    }
    
    searchKey.set(0x0008, 0x0060);
    if (! getElement(dataset, searchKey, &m_modality))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "modality not found.\n");
    }
    else
    {
      strcpy(str, m_modality.toAscii().data());
    }
    searchKey.set(0x0008, 0x0021);
    if (! getElement(dataset, searchKey,
                     &m_series_date))
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "series_date not found\n");
    }
    else
    {
      m_series_date = dcmDateTodmyString(m_series_date);
    }
    searchKey.set(0x0008, 0x0031);
    if (! getElement(dataset, searchKey,
                     &m_series_time))
    {
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "series_time not found\n");
    }
    else
    {
      m_series_time = dcmTimeTohhmmssString(m_series_time);
    }
    
    searchKey.set(0x0008, 0x0070);
    getElement(dataset, searchKey,
               &m_device_manufacturer);
               
    searchKey.set(0x0008, 0x1090);
    getElement(dataset, searchKey,
               &m_manufacturers_model_name);
               
    searchKey.set(0x0008, 0x1030);
    getElement(dataset, searchKey,
               &m_study_description);
               
    searchKey.set(0x0008, 0x103E);
    getElement(dataset, searchKey,
               &m_series_description);
               
    searchKey.set(0x0018, 0x0050);
    if (! getElement(dataset, searchKey,
                     &m_slice_thickness))
    {
      m_slice_thickness = "0.0";
    }
    
    // m_mr_echo_time helps to split combined MR series
    searchKey.set(0x0018, 0x0081);
    QString etqs;
    if (getElement(dataset, searchKey, &etqs))
    {
      bool b = false;
      m_mr_echo_time = etqs.toFloat(&b);
      
      if (! b)
        m_mr_echo_time = 0.0;
    }
    else
    {
      m_mr_echo_time = 0.0;
    }
    
    searchKey.set(0x0018, 0x1120);
    if (! getElement(dataset, searchKey,
                     &m_detector_tilt))
    {
      m_detector_tilt = "0.0";
    }
    //strcpy(str, m_detector_tilt.toAscii().data());
    //fprintf(stderr, "MinDcmImage m_detector_tilt: %s\n", str);
    
    searchKey.set(0x0018, 0x5100);
    if (! getElement(dataset, searchKey,
                     &m_patient_position))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,  "patient_position not found\n");
        
      m_patient_position = QString("unknown");
    }
    
    searchKey.set(0x0020, 0x000D);
    if (! getElement(dataset, searchKey,
                     &m_study_instance_uid))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,  "study_instance not found\n");
    }
    
    searchKey.set(0x0020, 0x000E);
    if (! getElement(dataset, searchKey,
                     &m_series_instance_uid))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "series_instance_uid not found \n");
    }
    
    searchKey.set(0x0020, 0x0011);
    if (! getElement(dataset, searchKey,
                     &m_series_number))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "series_number\n");
      //m_success = false;
    }
    
    //searchKey.set(0x0020, 0x0012);
    //if (! getElement(dataset, searchKey,
    //  &m_acquisition_number)) {
    //  m_success = false;
    //}
    
    searchKey.set(0x0020, 0x0013);
    if (! getElement(dataset, searchKey,
                     &m_instance_number))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "instance_number not found\n");
    }
    
    searchKey.set(0x0020, 0x0032);
    if (! getElement(dataset, searchKey,
                     &m_image_position_patient_str))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,  "image_position_patient not found\n");
      m_image_position_patient_str = QString("unknown");
    }
    
    searchKey.set(0x0020, 0x0037);
    if (! getElement(dataset, searchKey,
                     &m_image_orientation_patient_str))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,  "image_orientation_patient not found\n");
      m_image_orientation_patient_str = QString("unknown");
    }
    
    searchKey.set(0x0020, 0x0052);
    if (! getElement(dataset, searchKey,
                     &m_frame_of_reference))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,  "frame_of_reference not found\n");
    }
    
    searchKey.set(0x0020, 0x4000);
    if (! getElement(dataset, searchKey,
                     &m_images_comments))
    {
    }
    
    /*
    MONOCHROME1 = Pixel data represent a single monochrome image plane.
    The minimum sample value is intended to be displayed as white after any VOI gray scale transformations have been performed. See PS 3.4. This value may be used only when Samples per Pixel (0028,0002) has a value of 1.
    
    MONOCHROME2 = Pixel data represent a single monochrome image plane.
    The minimum sample value is intended to be displayed as black after any VOI gray scale transformations have been performed. See PS 3.4. This value may be used only when Samples per Pixel (0028,0002) has a value of 1.
    
    PALETTE COLOR = Pixel data describe a color image with a single sample per pixel (single image plane). The pixel value is used as an index into each of the Red, Blue, and Green Palette Color Lookup Tables (0028,1101-1103&1201-1203). This value may be used only when Samples per Pixel (0028,0002) has a value of 1. When the Photometric Interpretation is Palette Color; Red, Blue, and Green Palette Color Lookup Tables shall be present.
    
    RGB = Pixel data represent a color image described by red, green, and blue image planes. The minimum sample value for each color plane represents minimum intensity of the color. This value may be used only when Samples per Pixel (0028,0002) has a value of 3.
    
    HSV = Retired.
    
    ARGB = Retired.
    
    CMYK = Retired.
    
    YBR_FULL = Pixel data represent a color image described by one luminance (Y) and two chrominance planes (CB and CR). This photometric interpretation may be used only when Samples per Pixel (0028,0002) has a value of 3. Black is represented by Y equal to zero. The absence of color is represented by both CB and CR values equal to half full scale.
    
    Note: In the case where the Bits Allocated (0028,0100) has value of 8 half full scale is 128.
    
    In the case where Bits Allocated (0028,0100) has a value of 8 then the following equations convert between RGB and YCBCR Photometric Interpretation.
    
    Y = + .2990R + .5870G + .1140B
    
    CB = - .1687R - .3313G + .5000B + 128
    
    CR = + .5000R - .4187G - .0813B + 128
    
    Note: The above is based on CCIR Recommendation 601-2 dated 1990.
    
    YBR_FULL_422 = The same as YBR_FULL except that the CB and CR values are sampled horizontally at half the Y rate and as a result there are half as many CB and CR values as Y values.
    
    */
    
    //QString m_photometric_interpretation; //0028, 0004
    searchKey.set(0x0028, 0x0004);
    if (! getElement(dataset, searchKey,
                     &m_photometric_interpretation))
    {
      m_photometric_interpretation = QString("MONOCHROME2");
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "photometric_interpretation not found.\n");
    }
    
    searchKey.set(0x0028, 0x0030);
    if (! getElement(dataset, searchKey,
                     &m_pixel_spacing))
    {
      m_pixel_spacing = QString("0.0\\0.0");
    }
    
    searchKey.set(0x0028, 0x0100);
    if (getElement(dataset, searchKey,
                   &m_bits_allocated))
    {
      //strcpy(str, m_bits_allocated.toAscii().data());
      //printf("m_bits_allocated (from header): %s\n", str);
    }
    
    searchKey.set(0x0028, 0x0101);
    if (getElement(dataset, searchKey,
                   &m_bits_stored))
    {
    }
    
    searchKey.set(0x0028, 0x0106);
    m_smallest_image_pixel_value_ss = 0;
    if (! getSSElement(dataset, searchKey,
                       &m_smallest_image_pixel_value_ss))
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "m_smallest_image_pixel_value_string  //0028, 0106 not found\n");
    }
    else
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "m_smallest_image_pixel_value_ss (from header): %i\n",
                m_smallest_image_pixel_value_ss);
    }
    
    searchKey.set(0x0028, 0x0107);
    m_largest_image_pixel_value_ss = 0;
    if (! getSSElement(dataset, searchKey,
                       &m_largest_image_pixel_value_ss))
    {
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "m_largest_image_pixel_value_string  //0028, 0107 not found\n");
    }
    else
    {
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "m_largest_image_pixel_value_ss (from header): %i\n",
                m_largest_image_pixel_value_ss);
    }
    
    searchKey.set(0x0028, 0x1050);
    if (! getElement(dataset, searchKey,
                     &m_win_center_string))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stderr,  "win_center 0028 1050 not found\n");
      m_win_center_string = "0";
    }
    else
    {
      m_win_center_string = firstIntString(m_win_center_string);
      //strcpy(str, m_win_center_string.toAscii().data());
      //printf("m_win_center_string (from header): %s\n", str);
    }
    
    searchKey.set(0x0028, 0x1051);
    if (! getElement(dataset, searchKey,
                     &m_win_width_string))
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout,  "win_width 0028 1051 not found\n");
      m_win_width_string = "0";
    }
    else
    {
      m_win_width_string = firstIntString(m_win_width_string);
    }
    
    searchKey.set(0x0028, 0x1052);
    if (getElement(dataset, searchKey,
                   &m_rescale_intercept_string))
    {
    }
    else
    {
      m_rescale_intercept_string = "0";
    }
    
    searchKey.set(0x0028, 0x1053);
    if (getElement(dataset, searchKey,
                   &m_rescale_slope_string))
    {
    }
    else
    {
      m_rescale_slope_string = "1"; //0";
    }
    
    searchKey.set(0x0028, 0x1056);
    if (getElement(dataset, searchKey,
                   &m_voi_lut_function))
    {
    }
    
    m_patient_label  = QString("%1, *%2\n")
                       .arg(m_patient_name)
                       .arg(m_pat_birth_date);
    ba = m_patient_label.toLatin1();
    const char *plabel = ba.data();
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "patient label: %s\n", plabel);
      
    m_study_label  = QString("%1 %2")
                     .arg(m_modality)
                     .arg(m_series_date);
                     
    if (! m_series_description.trimmed().isEmpty())
    {
      if ((m_series_description.trimmed() != m_images_comments.trimmed())
          && (! m_images_comments.trimmed().isEmpty()))
      {
        m_series_label  = QString("%1, %2")
                          .arg(m_series_description.trimmed())
                          .arg(m_images_comments);
      }
      else
      {
        m_series_label  = QString("%1")
                          .arg(m_series_description.trimmed());
      }
    }
    else if (! m_study_description.trimmed().isEmpty())
    {
      m_series_label  = QString("%1")
                        .arg(m_study_description);
      if ((m_study_description.trimmed() != m_images_comments.trimmed())
          && (! m_images_comments.trimmed().isEmpty()))
      {
        m_series_label.append(", ");
        m_series_label.append(m_images_comments);
      }
    }
    else  if (! m_images_comments.trimmed().isEmpty())
    {
      m_series_label  = QString("%1")
                        .arg(m_images_comments);
    }
    else
    {
      m_series_label  = QString("%1")
                        .arg(m_modality);
    }
    
    m_image_label  = QString("%1, * %2\n%3, # %4")
                     .arg(m_patient_name)
                     .arg(m_pat_birth_date)
                     .arg(m_series_label)
                     .arg(m_instance_number);
                     
    if (getImage)
    {
      // trouble with 0002, 0010 Transfer Syntax UID:    1.2.840.10008.1.2.4.70
      m_status = m_dicomImage->getStatus();
      if (m_status != EIS_Normal)
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "minDcmImage, issue while loading dicom image\n");
          
      }
      if(EIS_Normal == m_status)
      {
        //fprintf(stderr, "MinDcmImage m_dicomImage->getStatus is EIS_Normal\n");
        m_success = true;
        m_dcmImageDepth = m_dicomImage->getDepth();
        m_dcmImageFrameCount = m_dicomImage->getFrameCount();
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "minDcmImage has %i frames\n",
                  m_dcmImageFrameCount);
                  
        double c = 0.0;
        double w = 0.0;
        if (m_dicomImage->setWindow(0))
        {
          m_dicomImage->getWindow(c, w);
          m_dcmImageWinCenterAndWidth[0] = c;
          m_dcmImageWinCenterAndWidth[1] = w;
          if (SDR_VERBOSE > 0)
            fprintf(stdout, "MinDcmImage: window and center from dcmtk dicom image; %4.4f and width %4.4f\n",
                    m_dcmImageWinCenterAndWidth[0],
                    m_dcmImageWinCenterAndWidth[1]);
        }
        
        if ((c == 0.0) || (w == 0.0))
        {
          bool bc, bw;
          int headerCenter = m_win_center_string.toInt(&bc);
          int headerWidth = m_win_width_string.toInt(&bw);
          // if toInt Returns 0 if the conversion fails
          //Sint16 m_smallest_image_pixel_value_ss;  //0028, 0106
          //Sint16 m_largest_image_pixel_value_ss;  //0028, 0107
          m_dcmImageWinCenterAndWidth[0] = headerCenter;
          m_dcmImageWinCenterAndWidth[1] = headerWidth;
          if (bc && bw)
          {
            if (SDR_VERBOSE > 2)
              fprintf(stdout, "MinDcmImage: window and center from dcm header %4.4f and width %4.4f\n",
                      m_dcmImageWinCenterAndWidth[0],
                      m_dcmImageWinCenterAndWidth[1]);
          }
          
          if ((!bc) || (!bw) || (headerCenter == 0) || (headerWidth == 0))
          {
          
            if (SDR_VERBOSE > 1)
              fprintf( stdout,
                       "MinDcmImage: window and center not found in header, default to %4.4f and width %4.4f\n",
                       m_dcmImageWinCenterAndWidth[0],
                       m_dcmImageWinCenterAndWidth[1]);
                       
            double min, max;
            if (m_dicomImage->getMinMaxValues(min, max, 0))
            {
              if (SDR_VERBOSE > 1)
                fprintf( stdout, "MinDcmImage: used min and max found in dicomImage: %4.4f and %4.4f\n",
                         min, max);
            }
            else
            {
              if (m_dicomImage->getMinMaxValues(min, max, 1))
              {
                if (SDR_VERBOSE > 1)
                  fprintf(stdout,
                          "MinDcmImage: used min and max not found in dicomImage, try possible min and max %.2f, %.2f\n",
                          min, max);
              }
              else
              {
                if(! m_dicomImage->isMonochrome())
                {
                  min = 0;
                  max = 255;
                }
                else
                {
                  int numberOfPixel = m_dicomImage->getWidth()
                                      * m_dicomImage->getHeight();
                  int bytesPerPixel = m_dicomImage->getOutputDataSize()
                                      / numberOfPixel;
                  int paddedDepth = pad_to_8_16_32_64(m_dicomImage->getDepth());
                  int samplesPerPixel = bytesPerPixel / (paddedDepth / 8);
                  int maxBytesPerSample = bytesPerPixel / samplesPerPixel;
                  
                  //calc max from bytes
                  if (SDR_VERBOSE > 1)
                    fprintf(stdout,
                            "MinDcmImage: possible min and max not found in dicomImage, guess min max from date type with %i byte per sample.\n",
                            maxBytesPerSample);
                  min = 0;
                  if (maxBytesPerSample == 1)
                    max = 255;
                  else if (maxBytesPerSample == 2)
                    max = 65535;
                  else
                    max = 4294967295;
                }
              }
            }
            m_dcmImageWinCenterAndWidth[1] = (int) (max - min);
            m_dcmImageWinCenterAndWidth[0] = (int) ((max - min) / 2);
            if (SDR_VERBOSE > 2)
              fprintf(stdout, "MinDcmImage: use m_dcmImageWinCenterAndWidth center %.2f and width %.2f\n",
                      m_dcmImageWinCenterAndWidth[0], m_dcmImageWinCenterAndWidth[1]);
          }  // eo if ((!bc) || (!bw) || (headerCenter == 0) || (headerWidth == 0))
          m_dicomImage->setWindow(
            m_dcmImageWinCenterAndWidth[0],
            m_dcmImageWinCenterAndWidth[1]);
        }
        
        if(m_dicomImage->isMonochrome())
        {
          m_qimage = QImage(m_dicomImage->getWidth(),
                            m_dicomImage->getHeight(), QImage::Format_Indexed8);
          m_qimage.setColorTable(m_grayColorTable());
          //m_qimage.convertToFormat(QImage::Format_ARGB32);
        }
        else
        {
          if (SDR_VERBOSE > 1)
            fprintf(stdout, "MinDcmImage is not Monochrom\n");
          m_qimage = QImage(m_dicomImage->getWidth(),
                            m_dicomImage->getHeight(), QImage::Format_ARGB32);
        }
        m_qimage = outputData(0, 0);
        
        if (m_qimage.isNull())
        {
          if (SDR_VERBOSE > 0)
            fprintf(stderr, "MinDcmImage, m_qimage.isNull\n");
        }
        
        if (m_pixmap.isNull())
        {
          fprintf(stderr, "MinDcmImage, m_pixmap.isNull\n");
        }
      }
      else //(EIS_Normal != m_status)
      {
        //problem opening dicom file.
        if (SDR_VERBOSE > 0)
          fprintf(stderr, "MinDcmImage status is not normal (%s).\n",
                  m_dicomImage->getString(m_status));
                  
        m_success = false;
      }
      DcmRLEDecoderRegistration::cleanup();
      
      // deregister JPEG decompression codecs
      DJDecoderRegistration::cleanup();
    }
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stderr, "Problem with MinDcmImage.\n");
      
    m_success = false;
  }
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "eo MinDcmImage\n");
}


MinDcmImage::~MinDcmImage()
{
  if(m_dicomImage)
  {
    //delete m_dicomImage;
  }
  //if(m_pixmap)
  //{
  //  delete m_pixmap;
  //}
  //if (m_dcm12bitData)
  //{
  //  free(m_dcm12bitData);
  //}
  if(m_dicomFileFormat)
  {
    delete m_dicomFileFormat;
  }
}

bool MinDcmImage::loadDicomImg(const char *full_file_name)
{

  bool xferKnown = true;
  
  // make sure data dictionary is loaded
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo MinDcmImage::loadDicomImg\n");
  // register RLE decompression codec
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MinDcmImage::loadDicomImg DcmRLEDecoderRegistration\n");
    
  DcmRLEDecoderRegistration::registerCodecs(OFFalse, OFTrue);
  
#ifdef SUPPORT_JPEG_LS
  // register JPEG decompression codecs
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MinDcmImage::loadDicomImg DJDecoderRegistration register.\n");
    
  DJDecoderRegistration::registerCodecs(EDC_always, EUC_default,
                                        EPC_default, OFTrue);
#endif
                                        
                                        
  OFCondition cond = m_dicomFileFormat->loadFile(full_file_name, //opt_ifname,
                     EXS_Unknown, EGL_withoutGL,
                     DCM_MaxReadLength, ERM_autoDetect);
  if (cond.bad())
  {
  
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "DcmFileFormat is in bad condition.\n");
      
    return false;
  }
  else
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "DcmFileFormat is in good condition.\n");
  }
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MinDcmImage::loadDicomImg, set E_TransferSyntax\n");
    
    
    
  E_TransferSyntax xfer = m_dicomFileFormat->getDataset()->getOriginalXfer();
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MinDcmImage::loadDicomImg, E_TransferSyntax seems to be ok.\n");
    
  if (xfer == EXS_Unknown)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MinDcmImage::loadDicomImg, E_TransferSyntax == EXS_Unknown\n");
      
    xferKnown = false;
  }
  else if (xfer == EXS_JPEGProcess1TransferSyntax)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MinDcmImage::loadDicomImg, E_TransferSyntax 'EXS_JPEGProcess1TransferSyntax'"
              " may be not supported.\n");
    /*
    DcmDataset *dataset = m_dicomFileFormat->getDataset();
    // decompress data set if compressed
    dataset->chooseRepresentation(EXS_LittleEndianExplicit, NULL);
    
    // check if everything went well
    if (! dataset->canWriteXfer(EXS_LittleEndianExplicit))
    {
      fprintf(stdout, "MinDcmImage::loadDicomImg, EXS_LittleEndianExplicit is NOT okay.\n");
      QMessageBox msgBox;
      msgBox.setWindowTitle("Sdr2 Issue");
      msgBox.setIcon(QMessageBox::Warning);
      msgBox.setText("Your DCMTK implementation does not seem\n"
                     "to support JPEG compressed DICOM files.");
      msgBox.setStandardButtons(QMessageBox::Ok);
      msgBox.setDefaultButton(QMessageBox::Ok);
      //int ret =
      msgBox.exec();
    
      return false;
    }
    else
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "MinDcmImage::loadDicomImg, EXS_LittleEndianExplicit is okay.\n");
    
      //m_dicomFileFormat->chooseRepresentation(EXS_LittleEndianExplicit, NULL);
      //xferKnown = false;
    }
    */
    xferKnown = false;
  }
  else
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "MinDcmImage::loadDicomImg, E_TransferSyntax == %i\n",
              xfer);
  }
  
  if (! xferKnown)
  {
    QMessageBox msgBox;
    msgBox.setWindowTitle("Sdr2 Critical Issue");
    msgBox.setIcon(QMessageBox::Warning);
    //msgBox.
    msgBox.setText("Warning: Tranfer syntax is unknown or unstable!\n"
                   "Sdr2 may crash if you try to load this file.");
    msgBox.setInformativeText("It is recommended to abort loading.");
    msgBox.setStandardButtons(QMessageBox::Ignore | QMessageBox::Abort);
    //msgBox.setButtonText(QMessageBox::Abort, "Exit ");
    msgBox.setDefaultButton(QMessageBox::Abort);
    int ret = msgBox.exec();
    
    switch(ret)
    {
      case QMessageBox::Ignore:
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "case QMessageBox::Ignore:.\n");
          
        break;
      }
      case QMessageBox::Abort:
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "case QMessageBox::Abort.\n");
        return false;
        break;
      }
      default:
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "default.\n");
        return false;
        break;
      }
    }
  }
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MinDcmImage::loadDicomImg, new DicomImage\n");
    
  m_dicomImage = new DicomImage(m_dicomFileFormat, xfer,
                                (CIF_MayDetachPixelData | CIF_TakeOverExternalDataset),
                                0, 1);
  //CIF_IgnoreModalityTransformation, 0, 1);
  //CIF_TakeOverExternalDataset, 0, 1);
  //CIF_KeepYCbCrColorModel, 0, 1);
  //CIF_WrongPaletteAttributeTags, 0, 1);
  //CIF_UsePresentationState, 0, 1);
  //CIF_AcrNemaCompatibility, 0, 1);
  //CIF_IgnoreModalityLutBitDepth, 0, 1);
  //CIF_MayDetachPixelData, 0, 1);
  // CIF_UsePartialAccessToPixelData, 0, 1);
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MinDcmImage::loadDicomImg, back from new DicomImage\n");
    
  if (m_dicomImage == NULL)
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "Out of memory (m_dicomImage == NULL), gone return false\n");
      
    return false;
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "m_dicomImage != NULL\n");
  }
  
  int eis = m_dicomImage->getStatus();
  
  if (SDR_VERBOSE > 0)
  {
    if (eis == EIS_Normal)
    {
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "DicomImage is in good condition\n");
    }
    else if (eis == EIS_NoDataDictionary)
    {
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "data dictionary not found !\n");
    }
    else if (eis == EIS_InvalidDocument)
    {
      fprintf(stdout, "invalid dataset/file\n");
    }
    else if (eis == EIS_MissingAttribute)
    {
      fprintf(stdout, "mandatory attribute missing\n");
    }
    else if (eis == EIS_InvalidValue)
    {
      fprintf(stdout, "invalid value for an important attribute\n");
    }
    
    else if (eis == EIS_NotSupportedValue)
    {
      fprintf(stdout, "specified value for an attribute not supported\n");
    }
    else if (eis == EIS_MemoryFailure)
    {
      fprintf(stdout, "memory exhausted etc.\n");
    }
    else if (eis == EIS_MissingAttribute)
    {
      fprintf(stdout, "invalid image, internal error\n");
    }
    else if (eis == EIS_OtherError)
    {
      fprintf(stdout, "EIS_InvalidImage\n");
    }
  }
  
  if (eis != EIS_Normal)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "loadDicomImage failed (m_dicomImage->getStatus() != EIS_Normal)\n");
      
    return false;
  }
  // options are set and file is loaded
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "gone deregister RLE decompression codec\n");
    
  // deregister RLE decompression codec
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MinDcmImage::loadDicomImg, DcmRLEDecoderRegistration::cleanup()\n");
    
  DcmRLEDecoderRegistration::cleanup();
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MinDcmImage::loadDicomImg, DcmRLEDecoderRegistration::cleanup done\n");
    
#ifdef SUPPORT_JPEG_LS
  // deregister JPEG decompression codecs
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MinDcmImage::loadDicomImg, DJDecoderRegistration::cleanup()\n");
  DJDecoderRegistration::cleanup();
#endif
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "*************** eo loadDicomImg ******************\n");
    
  return true;
}



const QImage& MinDcmImage::outputData(const unsigned long p_frame, const int p_planar)
{
  if((p_frame == m_previousConvertedFrame) &&
      (m_renewImage == false) && (!m_pixmap.isNull()))
  {
    return m_qimage;
  }
  m_renewImage = false;
  
  if(EIS_Normal == m_status)
  {
    uchar* pixelData = (uchar *)(m_dicomImage->getOutputData(IMG_DEPTH8,
                                 p_frame, p_planar));
    if (m_dicomImage->isMonochrome())
    {
      //Monochroom images
      //transform representation from dcmtk(gray) to qimage (gray)
      for(unsigned y = 0; y < m_dicomImage->getHeight(); y++)
      {
        uchar* scanLine = m_qimage.scanLine(y);
        for(unsigned x = m_dicomImage->getWidth(); x != 0; --x)
        {
          *(scanLine++) = *(pixelData++);
        }
      }
    }
    else
    {
      //color images
      //transform representation from dcmtk(RGB) to qimage (RGBA)
      uchar* qImageBuffer = m_qimage.bits();
      for(unsigned long pixels = m_qimage.width() * m_qimage.height();
          pixels != 0; --pixels)
      {
        *qImageBuffer++ = *pixelData++ ; //R
        *qImageBuffer++ = *pixelData++;  //G
        *qImageBuffer++ = *pixelData++;  //B
        *qImageBuffer++ = 0xFF; //Alpha
      }
    }
  }
  m_pixmap = QPixmap::fromImage(m_qimage, Qt::AutoColor);
  m_pixmap = m_pixmap.scaled((QSize(128, 128)), Qt::KeepAspectRatio,
                             Qt::FastTransformation);
  return m_qimage; // is ok to return, qimage supports shallow copy
}


bool MinDcmImage::getElement(DcmDataset *dataset,
                             DcmTagKey searchKey,
                             QString *element)
{
  const char *strPtr = NULL;
  OFCondition ec = EC_Normal;
  ec = dataset->findAndGetString(searchKey, strPtr);
  if ((ec != EC_Normal) || (strPtr == NULL))
  {
    OFString key = searchKey.toString();
    return false;
  }
  else
  {
    *element = strPtr;
    return true;
  }
}


bool MinDcmImage::getSSElement(DcmDataset *dataset,
                               DcmTagKey searchKey, Sint16 *ss)
{
  Sint16 ss_int = 0; //NULL;
  Sint16 *ssPtr = &ss_int;
  
  OFCondition ec = EC_Normal;
  
  ec = dataset->findAndGetSint16(searchKey, *ssPtr);
  if ((ec != EC_Normal) || (ssPtr == NULL))
  {
    return false;
  }
  else
  {
    *ss = *ssPtr; // &ss_int;
    return true;
  }
}


unsigned long MinDcmImage::frameCount() const
{
  //return d->m_dicomImage->getFrameCount();
  return m_dicomImage->getFrameCount();
}

/*!
\fn QDicomImage::frameTimeMsec
In case of a run/loop (movie) this method tries to determine the
frames per second.
determine the frame time from various dicom tags
*/
int MinDcmImage::frameTimeMsec() const
{
  //no image loaded, return immediately
  if(m_dicomFileFormat == NULL)
  {
    return 0;
  }
  if(frameCount() < 2)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "No frame time calculated. Image has only "
              "zero or one frame.");
  }
  
  DcmDataset* dataset = m_dicomFileFormat->getDataset();
  OFCondition cond;
  
  int frameTime = 0;
  long int recommendedDisplayFrameRate = 0;
  cond = dataset->findAndGetLongInt(
           DCM_RecommendedDisplayFrameRate, recommendedDisplayFrameRate);
  if(cond.good())
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "FrameTimeMsec-RecommendedDisplayFrameRate %ld",
              recommendedDisplayFrameRate);
              
    if(recommendedDisplayFrameRate)
    {
      frameTime = (1000 / (int)recommendedDisplayFrameRate);
    }
  }
  
  Float64 dicomFrameTime = 0.0;
  cond = dataset->findAndGetFloat64(
           DCM_FrameTime, dicomFrameTime);
  if(cond.good())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "FrameTimeMsec-FrameTime %f",
              (float)dicomFrameTime);
    if(!frameTime)
    {
      frameTime = int(dicomFrameTime);
    }
  }
  
  long int cineRate = 0;
  cond = dataset->findAndGetLongInt(
           DCM_CineRate, cineRate);
  if(cond.good())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "FrameTimeMsec-CineRate %ld", cineRate);
      
    if(!frameTime && cineRate)
    {
      frameTime = 1000 / cineRate;
    }
  }
  
  //default is 30 fps (1000 millisecond within 30 frames)
  if(frameTime)
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "Frametime set to %d", frameTime);
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "No frame time determined, setting default for "
              "30 fps.");
    frameTime = 1000 / 30;
  }
  return frameTime;
}


/*!
\fn QDicomImage::size()
*/
QSize MinDcmImage::size()
{
  QSize s(m_dicomImage->getWidth(),
          m_dicomImage->getHeight());
  return s;
}


void MinDcmImage::setWindowCenter( const double center )
{
  m_renewImage = true;
  m_dicomImage->setWindow(center, windowWidth());
}


double MinDcmImage::windowCenter( ) const
{
  double center;
  double width;
  double min, max;
  bool b = m_dicomImage->getWindow(center, width);
  if (b)
    return center;
  else
  {
    b = m_dicomImage->getMinMaxValues(min, max, 0);
    if (! b)
      b = m_dicomImage->getMinMaxValues(min, max, 1);
      
    center = (max - min) / 2.0;
    return center;
  }
  fprintf(stdout, "MinDcmImage::windowCenter failed\n");
  return 0.0;
}


void MinDcmImage::setWindowWidth( const double width )
{
  m_renewImage = true;
  m_dicomImage->setWindow(windowCenter(), width);
}


double MinDcmImage::windowWidth( ) const
{
  double center;
  double width;
  double min, max;
  bool b =  m_dicomImage->getWindow(center, width);
  if (b)
  {
    return width;
  }
  else
  {
    b = m_dicomImage->getMinMaxValues(min, max, 0);
    if (! b)
      b = m_dicomImage->getMinMaxValues(min, max, 1);
      
    width = max - min;
    return width;
  }
  if (SDR_VERBOSE > 0)
    fprintf(stdout, "MinDcmImage::windowWidth failed\n");
  return 0.0;
}


bool MinDcmImage::setMinMaxWindow( )
{
  m_renewImage = true;
  return (m_dicomImage->setMinMaxWindow() > 0);
}


bool MinDcmImage::minMaxValues( double & min, double & max ) const
{
  return (m_dicomImage->getMinMaxValues(min, max) > 0);
}


QString MinDcmImage::dcmDateTodmyString(QString dcmDate)
{
  QString d, m, y;
  y = dcmDate.left(4);
  m = dcmDate.mid(4, 2);
  d = dcmDate.right(2);
  
  if (m == "01") m = ("Jan.");
  else if (m == "02") m = ("Feb.");
  else if (m == "03") m = ("Mar.");
  else if (m == "04") m = ("Apr.");
  else if (m == "05") m = ("May");
  else if (m == "06") m = ("June");
  else if (m == "07") m = ("July");
  else if (m == "08") m = ("Aug.");
  else if (m == "09") m = ("Sep.");
  else if (m == "10") m = ("Oct.");
  else if (m == "11") m = ("Nov.");
  else if (m == "12") m = ("Dec.");
  else m = ("unknown");
  
  return QString("%1.%2 %3")
         .arg(d) .arg(m) .arg(y);
}


QString MinDcmImage::dcmTimeTohhmmssString(QString long_dcm_time)
{
  QString hh, mm, ss;
  hh = long_dcm_time.left(2);
  mm = long_dcm_time.mid(2, 2);
  ss = long_dcm_time.mid(4, 2);
  return QString("%1:%2:%3")
         .arg(hh) .arg(mm) .arg(ss);
}


QString MinDcmImage::firstIntString(QString str)
{
  QString ret;
  for (int i = 0; i < str.length(); i++)
  {
    QChar c = str.at(i);
    if (!c.isDigit())
      str.remove(i, 1);
    else
      break;
  }
  
  for (int i = 0; i < str.length(); i++)
  {
    QChar c = str.at(i);
    if (c.isDigit())
      ret.append(c);
    else
      break;
  }
  if (ret.isEmpty() || ret.isNull())
    ret = "0";
    
  return (ret);
}

