/**************************************************************************
 *  Author      : Johannes A. Koeppen, koeppen@ieee.org                   *
 *  Year(s)     : 2010 - 2015                                             *
 *  File        : minDcmXmlTree.cpp                                         *
 *  Context     : Sdr2 (Not approved for clinical use)                    *
 *  License     : GPL 2 or later                                          *
 *  Description : minimal file tree                                       *
 *                                                                        *
 **************************************************************************/

#include <QtGui>

#include "miniDcmXmlTree.h"

extern int SDR_VERBOSE;


MiniDcmXmlTree::MiniDcmXmlTree(QWidget *parent)
  : QTreeWidget(parent)
{
  itemSelected = false;
  QStringList labels;
  labels << tr("Dicom Item") << tr("Last Image Path");
  
  header()->resizeSection(0, 300); //260);
  setHeaderLabels(labels);
  
  folderIcon.addPixmap(style()->standardPixmap(QStyle::SP_DirClosedIcon),
                       QIcon::Normal, QIcon::Off);
  folderIcon.addPixmap(style()->standardPixmap(QStyle::SP_DirOpenIcon),
                       QIcon::Normal, QIcon::On);
  seriesIcon.addPixmap(style()->standardPixmap(QStyle::SP_FileIcon));
  
  xmlDefaultFileName = QString("DICOM.xMiniDcm");
  
  connect(this, SIGNAL(itemClicked(QTreeWidgetItem*, int)),
          this, SLOT(getSelectedItem(QTreeWidgetItem*)));
}


void MiniDcmXmlTree::getSelectedItem(QTreeWidgetItem *item)
{
  QDomElement element = domElementForItem.value(item);
  QString msg;
  
  if (element.isNull())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "selected element is Null\n");
    return;
  }
  else
  {
    msg = QString("%1")
          .arg(element.tagName());
          
    selectedItem = item;
    selectedDomElement = element;
    itemSelected = true;
    
    emit signalSelectionChanged(msg);
  }
}


void MiniDcmXmlTree::preparePreview()
{
  imageList.clear();
  
  if (selectedDomElement.tagName() != "series")
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "selectedDomElement.tagName() != 'series', gone return\n");
    return;
  }
  
#ifndef QT_NO_CURSOR
  QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
  
  for(QDomNode n = selectedDomElement.firstChild(); !n.isNull(); n = n.nextSibling())
  {
    QDomElement chk = n.toElement();
    if (chk.tagName() == "instance_absolute_filepath")
    {
      QString path = chk.text();
      imageList.append(path);
    }
  }
  
  emit signalSelectionChanged("disable");
  emit signalDcmPreview();
  
#ifndef QT_NO_CURSOR
  QApplication::setOverrideCursor(Qt::ArrowCursor);
#endif
  
}


bool MiniDcmXmlTree::deleteItem()
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo MiniDcmXmlTree::deleteItem()\n");
  if (! itemSelected)
    return false;
    
  QDomNode parent = selectedDomElement.parentNode();
  parent.removeChild(selectedDomElement);
  
  // ?delete selectedItem;
  return true;
}


bool MiniDcmXmlTree::loadDcmImage(const QString &fileName)
{
  QFileInfo fileInfo = fileName;
  if(!fileInfo.exists())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MiniDcmXmlTree::loadDcmImage: File does not exist\n");
    return false;
  }
  
  QString abs_path = fileInfo.absoluteFilePath();
  
  if (fileInfo.isDir())
  {
    QByteArray ba = abs_path.toLatin1();
    char *dname = ba.data();
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MiniDcmXmlTree::loadDcmImage: '%s' is dir, return false\n",
              dname);
  }
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MiniDcmXmlTree::loadImage calls new MinDcmImage\n");
    
  midi = new MinDcmImage(abs_path, false);
  if (midi->m_success)
  {
    return true;
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MiniDcmXmlTree::loadDcmImage: midi->m_success == false\n");
    return false;
  }
}


void MiniDcmXmlTree::ignoreBadFileOrExit(QFile *badFile)
{
  // memo: If MiniDcmXmlTree->xmlDefaultFile is corrupted
  //       then MiniDcmXmlTree->xmlDefaultFile will be deleted
  //       and a new empty standard MiniDcmXmlTree->xmlDefaultFile will be created.
  
  QString qfn = badFile->fileName();
  QByteArray ba = qfn.toLatin1();
  char *cfn = ba.data();
  
  QString msg = QString("Critical parse error in file:\n'%1'.\n\n"
                        "You need to ignore or delete this file,\n"
                        "or exit SDR2.\n\n")
                .arg(qfn);
  fprintf(stdout, "MiniDcmXmlTree::ignoreBadFileOrExit\n");
  
  
#ifndef QT_NO_CURSOR
  QApplication::setOverrideCursor(Qt::ArrowCursor);
#endif
  
  QMessageBox mb( "Sdr2 Critical issue",
                  msg,
                  QMessageBox::Critical,
                  QMessageBox::Ignore | QMessageBox::Default,
                  QMessageBox::Discard, // | QMessageBox::Default,
                  QMessageBox::Abort);
  mb.setButtonText( QMessageBox::Discard, "Delete" );
  mb.setButtonText( QMessageBox::Abort, "Exit" );
  switch( mb.exec() )
  {
    case QMessageBox::Ignore:
    {
      if (qfn.endsWith(xmlDefaultFileName))
      {
        int status;
        status = remove(cfn);
        if( status == 0 )
        {
          if (SDR_VERBOSE > 2)
            fprintf(stdout, "%s file deleted successfully.\n", cfn);
        }
        else
        {
          if (SDR_VERBOSE > 0)
            fprintf(stdout, "Unable to delete the file\n");
          perror("Error");
        }
      }
      break;
    }
    case QMessageBox::Discard:
    {
      int status;
      status = remove(cfn);
      if( status == 0 )
      {
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "%s file deleted successfully.\n", cfn);
      }
      else
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "Unable to delete the file\n");
        perror("Error");
      }
      break;
    }
    case QMessageBox::Abort:
      exit(0);
      break;
  }
}


bool MiniDcmXmlTree::readXmlDevice(QIODevice *device)
{
  QString errorStr;
  int errorLine;
  int errorColumn;
  
  if (!domDocument.setContent(device, true, &errorStr, &errorLine,
                              &errorColumn))
  {
    this->clear();
    return false;
  }
  
  QDomElement root = domDocument.documentElement();
  if (root.tagName() != "xMiniDcm")
  {
    QMessageBox::information(window(), tr("MiniDcmXmlTree"),
                             tr("The file is not an xMiniDcm file."));
    return false;
  }
  else if (root.hasAttribute("version")
           && root.attribute("version") != "0.1")
  {
    QMessageBox::information(window(), tr("MiniDcmXmlTree"),
                             tr("The file is not an XMiniDcm version 0.1 "
                                "file."));
    return false;
  }
  
  disconnect(this, SIGNAL(itemChanged(QTreeWidgetItem*, int)),
             this, SLOT(updateDomElement(QTreeWidgetItem*, int)));
             
  if (SDR_VERBOSE > 0)
    fprintf(stdout, "MiniDcmXmlTree::readXmlDevice gone clear widget.\n");
  this->clear();
  
  QDomElement child = root.firstChildElement("patient");
  
  while (!child.isNull())
  {
    parseDcmXmlDevice(child);
    child = child.nextSiblingElement("patient");
  }
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MiniDcmXmlTree::readXmlDevice gone reconnect slot.\n");
    
  connect(this, SIGNAL(itemChanged(QTreeWidgetItem*, int)),
          this, SLOT(updateDomElement(QTreeWidgetItem*, int)));
          
  return true;
}


bool MiniDcmXmlTree::writeXmlDevice(QIODevice *device)
{
  const int IndentSize = 2;
  
  QTextStream out(device);
  domDocument.save(out, IndentSize);
  return true;
}

void MiniDcmXmlTree::updateDomElement(QTreeWidgetItem *item, int column)
{
  QString errorStr, msg;
  
  QDomElement element = domElementForItem.value(item);
  if (element.isNull())
  {
    return;
  }
  
  if (column == 0)
  {
    //printf("update column 0\n");
    QDomElement oldTitleElement = element.firstChildElement("title");
    QDomElement newTitleElement = domDocument.createElement("title");
    QDomText newTitleText = domDocument.createTextNode(item->text(0));
    newTitleElement.appendChild(newTitleText);
    element.replaceChild(newTitleElement, oldTitleElement);
  }
  
  else if (column == 1)    // bo column is 1
  {
    if (element.tagName() == "series")
    {
      //printf("updateDomElement series column 1\n");
      QDomElement oldTitleElement = element.firstChildElement("series_absolute_filepath");
      QDomElement newTitleElement = domDocument.createElement("series_absolute_filepath");
      QDomText newTitleText = domDocument.createTextNode(item->text(1));
      newTitleElement.appendChild(newTitleText);
      element.replaceChild(newTitleElement, oldTitleElement);
    } // eo (element.tagName() == "series")
  } // eo column is 1
}


void MiniDcmXmlTree::parseDcmXmlDevice(const QDomElement &element,
                                       QTreeWidgetItem *parentItem)
{
  QString msg;
  QTreeWidgetItem *item = createItem(element, parentItem);
  QTreeWidgetItem *seriesItem = NULL;
  QString title = element.firstChildElement("title").text();
  if (title.isEmpty())
    title = QObject::tr("Patient");
    
  item->setFlags(item->flags() | Qt::ItemIsEditable);
  item->setIcon(0, folderIcon);
  item->setText(0, title);
  
  bool folded = (element.attribute("folded") != "no");
  setItemExpanded(item, !folded);
  
  QDomElement child = element.firstChildElement();
  while (!child.isNull())
  {
    if (child.tagName() == "patient")
    {
      parseDcmXmlDevice(child, item);
    }
    else if (child.tagName() == "study")
    {
      QTreeWidgetItem *childItem = createItem(child, item);
      QString title = child.firstChildElement("title").text();
      if (title.isEmpty())
        title = QObject::tr("Study");
        
      childItem->setFlags(item->flags() | Qt::ItemIsEditable);
      childItem->setIcon(0, folderIcon);
      childItem->setText(0, title);
      folded = (child.attribute("folded") != "no");
      setItemExpanded(childItem, !folded);
      QString studuid = child.firstChildElement("study_instance_uid").text();
      if (!studuid.isEmpty())
        childItem->setText(2, studuid);
        
      QDomElement studyElement = child;
      for(QDomNode n = studyElement.firstChild(); !n.isNull(); n = n.nextSibling())
      {
        QString series_name =  n.firstChildElement("title").text();
        QDomNode series_node  = n.firstChildElement("title").parentNode();
        QDomElement seriesElement = series_node.toElement();
        if (!series_name.isEmpty())
        {
          seriesItem = createItem(seriesElement, childItem);
          seriesItem->setFlags(item->flags() | Qt::ItemIsEditable);
          seriesItem->setIcon(0, seriesIcon);
          seriesItem->setText(0, series_name);
        }
        QString series_absolute_filepath_name =  n.firstChildElement("series_absolute_filepath").text();
        if (!series_absolute_filepath_name.isEmpty())
        {
          seriesItem->setText(1, series_absolute_filepath_name);
          //folded = (element.attribute("folded") != "no");
        }
        QString series_instance_uid_name =  n.firstChildElement("series_instance_uid").text();
        if (!series_instance_uid_name.isEmpty())
        {
          seriesItem->setText(2, series_instance_uid_name);
          //folded = (element.attribute("folded") != "no");
        }
      }
    }
    else if (child.tagName() == "separator")
    {
      QTreeWidgetItem *childItem = createItem(child, item);
      childItem->setFlags(item->flags() & ~(Qt::ItemIsSelectable | Qt::ItemIsEditable));
      childItem->setText(0, QString(30, 0xB7));
    }
    child = child.nextSiblingElement();
  }
}


// return true if user wants to add folder to collection.
bool MiniDcmXmlTree::reallyWantToAddFolder(QString folder, int column)
{
  QFileInfo finfo(folder);
  if(!finfo.exists())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MiniDcmXmlTree::reallyWantToAddFolder, (!finfo.exists())\n");
    return false;
  }
  
  QByteArray ba = folder.toLatin1();
  char *fo = ba.data();
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MiniDcmXmlTree::reallyWantToAddFolder find '%s'\n",
            fo);
  QList<QTreeWidgetItem*> items = this->findItems(folder,
                                  Qt::MatchRecursive, column);
  if (items.count() == 0)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MiniDcmXmlTree::reallyWantToAddFolder item not found\n");
    return true;
  }
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MiniDcmXmlTree::reallyWantToAddFolder item found!\n");
    
#ifndef QT_NO_CURSOR
  QApplication::setOverrideCursor(Qt::ArrowCursor);
#endif
  QMessageBox msgBox;
  QString txt = QString("Folder '%1' is already in collection.")
                .arg(folder);
  QString info = QString("Do you really want to add duplicate to collection? ");
  msgBox.setText(txt);
  msgBox.setInformativeText(info);
  msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
  msgBox.setDefaultButton(QMessageBox::No);
  int ret = msgBox.exec();
  switch (ret)
  {
    case QMessageBox::Yes:
      return true;
      break;
    case QMessageBox::No:
      return false;
      break;
    case QMessageBox::Cancel:
      // Cancel was clicked
      return false;
      break;
    default:
      return false;
      break;
  }
  
#ifndef QT_NO_CURSOR
  QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
  
  return false;
}


bool MiniDcmXmlTree::insertDcmImage()
{
  QString msg;
  bool ret = false;
  QDomElement root = domDocument.documentElement();
  QDomElement patient = root.firstChildElement("patient");
  found = NothingFound;
  
  while (!patient.isNull())
  {
    found = findDcmXmlElements(patient, midi);
    if (found != NothingFound)
      break;
    patient = patient.nextSiblingElement("patient");
  }
  
  if ( found != ImageFound)
  {
    switch(found)
    {
      case NothingFound:
      {
        QDomElement new_patient = domDocument.createElement("patient");
        new_patient.setAttribute("folded", "no");
        QDomElement patient_title = domDocument.createElement("title");
        QDomText newTitleText = domDocument.createTextNode(midi->m_patient_label);
        patient_title.appendChild(newTitleText);
        new_patient.appendChild(patient_title);
        root.insertBefore(new_patient, patient);
        ret = true;
        //break;
      }
      case PatientFound:
      {
        QDomElement patient = root.firstChildElement("patient");
        while (!patient.isNull())
        {
          QDomElement patient_title =  patient.firstChildElement("title");
          QString patient_label =  patient_title.text();
          if (patient_label == midi->m_patient_label)
          {
            QDomElement new_study = domDocument.createElement("study");
            new_study.setAttribute("folded", "no");
            QDomElement study_title = domDocument.createElement("title");
            QDomText newStudyTitleText = domDocument.createTextNode(midi->m_study_label);
            study_title.appendChild(newStudyTitleText);
            new_study.appendChild(study_title);
            
            QDomElement study_uid = domDocument.createElement("study_instance_uid");
            QDomText newStudyUidText = domDocument.createTextNode(midi->m_study_instance_uid);
            study_uid.appendChild(newStudyUidText);
            new_study.appendChild(study_uid);
            
            patient.insertAfter(new_study, patient_title);
            ret = true;
          }
          patient = patient.nextSiblingElement("patient");
        }
        //break;
      }
      case StudyFound:
      {
        if (SDR_VERBOSE > 1)
          fprintf(stdout, "insertDcmImage Study Found\n");
          
        QDomElement patient = root.firstChildElement("patient");
        while (!patient.isNull())
        {
          QDomElement patient_title =  patient.firstChildElement("title");
          QString patient_label =  patient_title.text();
          if (patient_label == midi->m_patient_label)
          {
            for(QDomNode patientChild = patient.firstChild(); !patientChild.isNull();
                patientChild = patientChild.nextSibling())
            {
              QDomElement patientChildElement = patientChild.toElement();
              if (patientChildElement.tagName() == "study")
              {
                QDomElement study = patientChildElement;
                for(QDomNode studyChild = study.firstChild(); !studyChild.isNull();
                    studyChild = studyChild.nextSibling())
                {
                  QDomElement studyChildElement = studyChild.toElement();
                  if (studyChildElement.tagName() == "study_instance_uid")
                  {
                    QString study_instance_uid_text =  studyChildElement.text();
                    if (study_instance_uid_text == midi->m_study_instance_uid)
                    {
                      QDomElement new_series = domDocument.createElement("series");
                      QDomElement series_title = domDocument.createElement("title");
                      QDomText newSeriesTitleText = domDocument.createTextNode(midi->m_series_label);
                      series_title.appendChild(newSeriesTitleText);
                      new_series.appendChild(series_title);
                      
                      QDir image_directory(midi->m_image_absolute_filepath);
                      image_directory.cdUp();
                      QString series_directory_name = image_directory.absolutePath();
                      QDomElement series_absolute_filepath = domDocument.createElement("series_absolute_filepath");
                      QDomText newSeriesAbsoluteFilepathText = domDocument.createTextNode(series_directory_name);
                      series_absolute_filepath.appendChild(newSeriesAbsoluteFilepathText);
                      new_series.appendChild(series_absolute_filepath);
                      
                      QDomElement series_uid = domDocument.createElement("series_instance_uid");
                      QDomText newSeriesUidText = domDocument.createTextNode(midi->m_series_instance_uid);
                      series_uid.appendChild(newSeriesUidText);
                      new_series.appendChild(series_uid);
                      study.insertAfter(new_series, studyChildElement);
                      ret = true;
                    }
                  }
                } // eo for(QDomNode studyChild
              } //eo if (patientChildElement
            }  //eo for(QDomNode patientChild
          } //eo if (patient_label
          patient = patient.nextSiblingElement("patient");
        }  //eo while (!patient.isNull())
        //break;
      } // eo case StudyFound
      case SeriesFound:
      {
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "MiniDcmXmlTree::insertDcmImage, case SeriesFound\n");
        // to do: ? want duplicate ??
        QDomElement patient = root.firstChildElement("patient");
        while (!patient.isNull())
        {
          QDomElement patient_title =  patient.firstChildElement("title");
          QString patient_label =  patient_title.text();
          if (patient_label == midi->m_patient_label)
          {
            for(QDomNode patientChild = patient.firstChild(); !patientChild.isNull();
                patientChild = patientChild.nextSibling())
            {
              QDomElement patientChildElement = patientChild.toElement();
              if (patientChildElement.tagName() == "study")
              {
                QDomElement anyStudy = patientChildElement;
                for(QDomNode anyStudyChild = anyStudy.firstChild(); !anyStudyChild.isNull();
                    anyStudyChild = anyStudyChild.nextSibling())
                {
                  QDomElement anyStudyChildElement = anyStudyChild.toElement();
                  if (anyStudyChildElement.tagName() == "study_instance_uid")
                  {
                    QString study_instance_uid_text =  anyStudyChildElement.text();
                    if (study_instance_uid_text == midi->m_study_instance_uid)
                    {
                      QDomElement parentStudy =  anyStudy;
                      for(QDomNode parentStudyChild = parentStudy.firstChild(); !parentStudyChild.isNull();
                          parentStudyChild = parentStudyChild.nextSibling())
                      {
                        QDomElement parentStudyChildElement =  parentStudyChild.toElement();
                        if (parentStudyChildElement.tagName() == "series")
                        {
                          QDomElement series = parentStudyChildElement;
                          for(QDomNode seriesChild = series.firstChild(); !seriesChild.isNull();
                              seriesChild = seriesChild.nextSibling())
                          {
                            QDomElement seriesChildElement = seriesChild.toElement();
                            if (seriesChildElement.tagName() == "series_instance_uid")
                            {
                              QString series_instance_uid_text =  seriesChildElement.text();
                              if (series_instance_uid_text == midi->m_series_instance_uid)
                              {
                                QDomElement instance_absolute_filepath
                                  = domDocument.createElement("instance_absolute_filepath");
                                QDomText newInstanceAbsoluteFilepathText
                                  = domDocument.createTextNode(midi->m_image_absolute_filepath);
                                instance_absolute_filepath.appendChild(newInstanceAbsoluteFilepathText);
                                series.insertAfter( instance_absolute_filepath, seriesChildElement);
                                QDomElement newInstanceUid
                                  =   domDocument.createElement("sop_instance_uid");
                                QDomText newInstanceUidText
                                  = domDocument.createTextNode(midi->m_sop_instance_uid);
                                newInstanceUid.appendChild(newInstanceUidText);
                                series.insertAfter(newInstanceUid, seriesChildElement);
                                ret = true;
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                } // eo for(QDomNode studyChild
              } //eo if (patientChildElement
            }  //eo for(QDomNode patientChild
          } //eo if (patient_label
          patient = patient.nextSiblingElement("patient");
        }  //eo while (!patient.isNull())
        break;
      } // eo case SeriesFound
      case ImageFound:
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "MiniDcmXmlTree::insertDcmImage, case ImageFound\n");
        break;
        
      default:
        break;
    }
  }
  return ret;
}


Found MiniDcmXmlTree::findDcmXmlElements(const QDomElement &element,
    MinDcmImage *midi)
{
  found = NothingFound;
  
  QString patient_title_str = midi->m_patient_label;
  QString study_instance_uid_str = midi->m_study_instance_uid;
  QString series_instance_uid_str = midi->m_series_instance_uid;
  QString image_absolute_filepath_str = midi->m_image_absolute_filepath;
  
  QDomElement child = element.firstChildElement();
  
  if ((child.tagName() == "title")
      && (element.tagName() == "patient")
      && (child.text() == patient_title_str))
  {
    found = PatientFound;
  }
  
  
  while (!child.isNull())
  {
    if (child.tagName() == "study")
    {
      QString title = child.firstChildElement("title").text();
      if (title.isEmpty())
        title = QObject::tr("Study");
        
      QString studuid = child.firstChildElement("study_instance_uid").text();
      if (!studuid.isEmpty())
      {
        if ((studuid == study_instance_uid_str)
            && (found == PatientFound))
        {
          found = StudyFound;
        }
      }
      QDomElement studyElement = child;
      for(QDomNode n = studyElement.firstChild(); !n.isNull(); n = n.nextSibling())
      {
        QString series_name =  n.firstChildElement("title").text();
        QString series_absolute_filepath_name =  n.firstChildElement("series_absolute_filepath").text();
        if (!series_absolute_filepath_name.isEmpty())
        {
        }
        QString series_instance_uid_name =  n.firstChildElement("series_instance_uid").text();
        if (!series_instance_uid_name.isEmpty())
        {
          if ((series_instance_uid_name == series_instance_uid_str)
              && (found == StudyFound))
          {
            found = SeriesFound;
          }
        }
        
        for(QDomNode m = n.firstChild(); !m.isNull(); m = m.nextSibling())
        {
          QDomElement instanceElement = m.toElement();
          if (instanceElement.tagName() == "instance_absolute_filepath")
          {
            QString img_path_name =  instanceElement.text();
            if ((img_path_name == image_absolute_filepath_str)
                && (found == SeriesFound))
            {
              found = ImageFound;
            }
          }
        }
        
      }
    }
    //else if (child.tagName() == "separator")
    //{
    //QTreeWidgetItem *childItem = createItem(child, item);
    //childItem->setFlags(item->flags() & ~(Qt::ItemIsSelectable | Qt::ItemIsEditable));
    //childItem->setText(0, QString(30, 0xB7));
    //}
    child = child.nextSiblingElement();
  }
  return found;
}


QTreeWidgetItem *MiniDcmXmlTree::createItem(const QDomElement &element,
    QTreeWidgetItem *parentItem)
{
  QTreeWidgetItem *item;
  if (parentItem)
  {
    item = new QTreeWidgetItem(parentItem);
  }
  else
  {
    item = new QTreeWidgetItem(this);
  }
  domElementForItem.insert(item, element);
  return item;
}

