/**************************************************************************
 *  Author      : Johannes A. Koeppen, koeppen@ieee.org                   *
 *  Year(s)     : 2010 - 2015                                             *
 *  File        : minDcmXmlWidget.cpp                                     *
 *  Context     : Sdr2 (Not approved for clinical use)                    *
 *  License     : GPL 2 or later                                          *
 *  Description : minimal DICOM Image widget to read and write XML        *
 *                                                                        *
 **************************************************************************/


#include <QtGui>
#include <QtGui/QVBoxLayout>
#include <QGridLayout>
#include <QtGui/QWidget>

#include "globals.h"
#include "miniDcmXmlWidget.h"
#include "miniDcmXmlTree.h"

extern int SDR_VERBOSE;

MiniDcmXmlWidget::MiniDcmXmlWidget()
  : QWidget()
{
  miniDcmXmlTree = new MiniDcmXmlTree;
  
  QDir startDicomDir =  QDir::homePath(); // QDir::currentPath();
  searchDicomDir = startDicomDir.absolutePath();
  
  countDcmSuccess = 0;
  
  QVBoxLayout *verticalLayout = new QVBoxLayout();
  verticalLayout->addWidget(miniDcmXmlTree);
  
  QGridLayout *buttonGridLayout = new QGridLayout;
  
  pushButtonDelete = new QPushButton("Delete Item");
  pushButtonSaveXml = new QPushButton("Save Collection");
  pushButtonOpenXml = new QPushButton("Open Collection");
  pushButtonPreview = new QPushButton("Select Series");
  
  pushButtonPreview->setEnabled(false);
  pushButtonDelete->setEnabled(false);
  
  buttonGridLayout->addWidget(pushButtonDelete, 0, 0);
  buttonGridLayout->addWidget(pushButtonPreview, 0, 1);
  buttonGridLayout->addWidget(pushButtonOpenXml, 1, 0);
  buttonGridLayout->addWidget(pushButtonSaveXml, 1, 1);
  
  verticalLayout->addLayout(buttonGridLayout);
  this->setLayout(verticalLayout);
  
  connect(pushButtonSaveXml, SIGNAL(clicked()),
          this, SLOT(saveXmlDcm()));
  connect(pushButtonDelete, SIGNAL(clicked()),
          this, SLOT(deleteXmlItem()));
  connect(miniDcmXmlTree, SIGNAL(signalSelectionChanged(QString)),
          this, SLOT(updateButtons(QString)));
          
  connect(pushButtonOpenXml, SIGNAL(clicked()),
          this, SLOT(saveAndOpenXmlDcm()));
  connect(pushButtonPreview, SIGNAL(clicked()),
          miniDcmXmlTree, SLOT(preparePreview()));
  connect(miniDcmXmlTree, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)),
          this, SLOT(doubleClickedHandler(QTreeWidgetItem*, int)));
}


void MiniDcmXmlWidget::openXmlDefault()
{
  bool haveValidXml = false;
  
  QDir startXmlDir = QDir::currentPath();
  QString xmlFileString = startXmlDir.absolutePath();
  xmlFileString.append("/");
  xmlFileString.append(miniDcmXmlTree->xmlDefaultFileName);
  
  QByteArray ba = xmlFileString.toLatin1();
  const char *fname = ba.data();
  
  QFile file(xmlFileString);
  if (!file.exists())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MiniDcmXmlWidget::openXmlDefaultdirs file '%s' does not exist.",
              fname);
  }
  
  if (file.open(QFile::ReadOnly | QFile::Text))
  {
    if (miniDcmXmlTree->readXmlDevice(&file))
    {
      domFileName = xmlFileString ;
      haveValidXml = true;
      //statusBar()->showMessage(tr("XML file loaded"), 2000);
    }
    else
    {
      file.close();
      miniDcmXmlTree->ignoreBadFileOrExit(&file);
      haveValidXml = false;
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "MiniDcmXmlWidget::openXmlDefaultdirs bad XML found. Delete it!\n");
    }
    file.close();
  }
  
  if (! haveValidXml)
  {
    if (!file.open(QFile::WriteOnly | QFile::Text))
    {
      QMessageBox::warning(this, tr("XMiniDcm"),
                           tr("Cannot write file %1:\n%2.")
                           .arg(xmlFileString)
                           .arg(file.errorString()));
      openXmlDcm();
    }
    else
    {
      QTextStream out(&file);
      out << "<?xml version='1.0' encoding='utf-8'?>\n";
      out << "<!DOCTYPE xMiniDcm>\n";
      out << "<il:xMiniDcm xmlns:il=\"http://tempuri.org/WDWEBSVC.xsd\" version=\"0.1\">\n\n";
      out << "</il:xMiniDcm>\n";
      file.close();
      
      miniDcmXmlTree->readXmlDevice(&file);
      domFileName = xmlFileString ;
    }
  }
}


void MiniDcmXmlWidget::saveAndOpenXmlDcm()
{
  QString msg;
  //char str[1024];
  
  QFile current_file(domFileName);
  if (!current_file.open(QFile::WriteOnly | QFile::Text))
  {
    QMessageBox::warning(this, tr("XMiniDcm"),
                         tr("Cannot write file %1:\n%2.")
                         .arg(domFileName)
                         .arg(current_file.errorString()));
    return;
  }
  if (! miniDcmXmlTree->writeXmlDevice(&current_file))
  {
    //statusBar()->showMessage(tr("Current file saved."), 2000);
  }
  current_file.close();
  openXmlDcm();
}

void MiniDcmXmlWidget::updateButtons(QString selected_tag)
{
  if (selected_tag == "patient")
  {
    pushButtonDelete->setEnabled(true);
    pushButtonPreview->setEnabled(false);
  }
  else if (selected_tag == "study")
  {
    pushButtonDelete->setEnabled(true);
    pushButtonPreview->setEnabled(false);
  }
  else if (selected_tag == "series")
  {
    if (SDR_VERBOSE > 0)
      pushButtonDelete->setEnabled(true);
    pushButtonPreview->setEnabled(true);
  }
  else
  {
    pushButtonDelete->setEnabled(false);
    pushButtonPreview->setEnabled(false);
  }
}


void MiniDcmXmlWidget::deleteXmlItem()
{
  if (miniDcmXmlTree->deleteItem())
  {
    reloadXmlDcm(domFileName);
    pushButtonDelete->setEnabled(false);
    pushButtonPreview->setEnabled(false);
  }
}

void MiniDcmXmlWidget::doubleClickedHandler(QTreeWidgetItem *item, int column)
{

  QDomElement element = miniDcmXmlTree->domElementForItem.value(item);
  if (element.isNull())
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "update element is Null\n");
    return;
  }
  
  if (element.tagName() == "series")
  {
    miniDcmXmlTree->preparePreview();
  }
  updateButtons(element.tagName());
}



void MiniDcmXmlWidget::openXmlDcm()
{
  QString xmlDcmFileName =
    QFileDialog::getOpenFileName(this, tr("Open XMiniDcm File"),
                                 QDir::currentPath(),
                                 tr("XMiniDcm Files (*.xMiniDcm *.xml)"));
  if (xmlDcmFileName.isEmpty())
    return;
    
  QFile file(xmlDcmFileName);
  if (!file.open(QFile::ReadOnly | QFile::Text))
  {
    QMessageBox::warning(this, tr("XMiniDcm"),
                         tr("Cannot read file %1:\n%2.")
                         .arg(xmlDcmFileName)
                         .arg(file.errorString()));
    return;
  }
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MiniDcmXmlWidget::openXmlDcm calls miniDcmXmlTree->readXmlDevice\n");
    
  if (miniDcmXmlTree->readXmlDevice(&file))
  {
    domFileName = xmlDcmFileName;
    //statusBar()->showMessage(tr("XML file loaded"), 2000);
  }
  else
  {
    file.close();
    miniDcmXmlTree->ignoreBadFileOrExit(&file);
    openXmlDefault();
    return;
  }
  pushButtonPreview->setEnabled(false);
  pushButtonDelete->setEnabled(false);
  file.close();
}


void MiniDcmXmlWidget::parseDcmDir(QString dirName)
{
  pushButtonPreview->setEnabled(false);
  pushButtonDelete->setEnabled(false);
  countDcmSuccess = 0;
  
  QDir qdir(dirName);
  
  if(!qdir.exists())
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "No directory opened.\n");
    return;
  }
  
  insertDcmDir(dirName) ;
  
  qdir.setFilter(QDir::Dirs | QDir::NoSymLinks | QDir::NoDotAndDotDot);
  qdir.setSorting(QDir::Size );
  
  QFileInfoList dir_list = qdir.entryInfoList();
  for (int i = 0; i < dir_list.size(); ++i)
  {
    QFileInfo fileInfo = dir_list.at(i);
    QString abs_path = fileInfo.absoluteFilePath();
    insertDcmDir(abs_path) ;
  }
  
  if (countDcmSuccess == 0)
  {
#ifndef QT_NO_CURSOR
    QApplication::setOverrideCursor(Qt::ArrowCursor);
#endif
    
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "MiniDcmXmlWidget::parseDcmDir, loading of folder aborted.\n");
    //QMessageBox::warning(this, tr("XMiniDcm"),
    //                     tr("No usable Dicom files found in neither\n%1\nnor in it's first subdirectories.")
    //                     .arg(searchDicomDir));
    //.arg(file.errorString()));
  }
  else
  {
    reloadXmlDcm(domFileName);
  }
}


void MiniDcmXmlWidget::insertDcmDir(QString dirName)
{
  QDir selectedDir(dirName);
  searchDicomDir = selectedDir.absolutePath();
  
  if(!selectedDir.exists())
  {
    if (SDR_VERBOSE > 1)
      fprintf(stderr, "No directory opened.\n");
    return;
  }
  
  if (! miniDcmXmlTree->reallyWantToAddFolder(dirName, 1))
    return;
    
  QFileInfoList dir_list = selectedDir.entryInfoList();
  
  char str[1024];
  strcpy(str, dirName.toAscii().data());
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "MiniDcmXmlWidget::insertDcmDir %s\n", str);
    
  for (int i = 0; i < dir_list.size(); ++i)
  {
    QFileInfo fileInfo = dir_list.at(i);
    QString abs_path = fileInfo.absoluteFilePath();
    if (isDcmFilename(abs_path))
    {
      QFileInfo fileInfo = abs_path;
      if (fileInfo.isDir())
      {
        QDir chkDir(abs_path);
        int count = chkDir.count();
        QByteArray ba = abs_path.toLatin1();
        char *dname = ba.data();
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "MiniDcmXmlWidget::insertDcmFile: folder'%s' has %i entries.\n",
                  dname, count);
        if (count > 400)
        {
#ifndef QT_NO_CURSOR
          QApplication::setOverrideCursor(Qt::ArrowCursor);
#endif
          QMessageBox msgBox;
          QString txt = QString("Folder '%1' has %2 entries ")
                        .arg(abs_path) .arg(count);
          QString info = QString("Do you really want to check and load %1 entries? ")
                         .arg(count);
          msgBox.setText(txt);
          msgBox.setInformativeText(info);
          msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
          msgBox.setDefaultButton(QMessageBox::No);
          int ret = msgBox.exec();
          switch (ret)
          {
            case QMessageBox::Yes:
              break;
            case QMessageBox::No:
              return;
              break;
            case QMessageBox::Cancel:
              return;
              break;
            default:
              return;
              break;
          }
        }
      }
      
#ifndef QT_NO_CURSOR
      QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
      
      insertDcmFile(abs_path, false);
    }
  }
}


void MiniDcmXmlWidget::insertDcmFile(QString dcmFileName, bool reload)
{
  QFileInfo fileInfo = dcmFileName;
  if(!fileInfo.exists())
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "MiniDcmXmlWidget::insertDcmFile: File or folder does not exist\n");
    return;
  }
  
  if (miniDcmXmlTree->loadDcmImage(dcmFileName))
  {
    countDcmSuccess++;
    if ((miniDcmXmlTree->insertDcmImage())
        && (reload))
    {
      reloadXmlDcm(domFileName);
      //statusBar()->showMessage(tr("DCM file reloaded"), 2000);
    }
  }
}


void MiniDcmXmlWidget::saveXmlDcm()
{
  QString xmlDcmFileName =
    QFileDialog::getSaveFileName(this, tr("Save XMiniDcm File"),
                                 QDir::currentPath(),
                                 tr("XMiniDcm Files (*.xMiniDcm *.xml)"));
  if (xmlDcmFileName.isEmpty())
    return;
    
  QFile file(xmlDcmFileName);
  if (!file.open(QFile::WriteOnly | QFile::Text))
  {
    QMessageBox::warning(this, tr("XMiniDcm"),
                         tr("Cannot write file %1:\n%2.")
                         .arg(xmlDcmFileName)
                         .arg(file.errorString()));
    return;
  }
  
  miniDcmXmlTree->writeXmlDevice(&file);
  //if (miniDcmXmlTree->writeXmlDevice(&file)) {
  //statusBar()->showMessage(tr("File saved"), 2000);
  //}
}

void MiniDcmXmlWidget::reloadXmlDcm(QString fileName)
{
  if (fileName.isEmpty())
    return;
    
  QFile file(fileName);
  if (!file.open(QFile::WriteOnly | QFile::Text))
  {
    QMessageBox::warning(this, tr("XMiniDcm"),
                         tr("Cannot write file %1:\n%2.")
                         .arg(fileName)
                         .arg(file.errorString()));
    return;
  }
  
  miniDcmXmlTree->writeXmlDevice(&file);
  //if (miniDcmXmlTree->writeXmlDevice(&file)) {
  //statusBar()->showMessage(tr("File saved"), 2000);
  //}
  file.close();
  
  if (!file.open(QFile::ReadOnly | QFile::Text))
  {
    QMessageBox::warning(this, tr("XMiniDcm"),
                         tr("Cannot read file %1:\n%2.")
                         .arg(fileName)
                         .arg(file.errorString()));
    return;
  }
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "MiniDcmXmlWidget::reloadXmlDcm calls miniDcmXmlTree->readXmlDevice\n");
  if (! miniDcmXmlTree->readXmlDevice(&file))
  {
    file.close();
    miniDcmXmlTree->ignoreBadFileOrExit(&file);
    openXmlDefault();
  }
  //if (miniDcmXmlTree->readXmlDevice(&file)) {
  //  statusBar()->showMessage(tr("File reloaded"), 2000);
  // }
}


void MiniDcmXmlWidget::about()
{
  QMessageBox::about(this, tr("About Sdr2 (Simple DICOM Reader)"),
                     tr("<b>Sdr2</b> is a simple tool to "
                        "select DICOM series and to convert these series to Nifti or  "
                        "Analyze files."));
}



/*
void MiniDcmXmlWidget::createActions()
{
    openAct = new QAction(tr("&Open..."), this);
    openAct->setShortcuts(QKeySequence::Open);
    connect(openAct, SIGNAL(triggered()), this, SLOT(openXmlDcm()));

    saveAsAct = new QAction(tr("&Save As..."), this);
    saveAsAct->setShortcuts(QKeySequence::SaveAs);
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveXmlDcm()));

    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

    aboutAct = new QAction(tr("&About"), this);
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), this);
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}
*/


/*
void MiniDcmXmlWidget::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(openAct);
    fileMenu->addAction(saveAsAct);
    fileMenu->addAction(exitAct);

    menuBar()->addSeparator();

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
}
*/
