/***************************************************************************
 *   Copyright (C) 2007 by Jeroen van der Waal                             *
 *   jwaal@kabelfoon.nl                                                    *
 *                                                                         *
 *   with minor distortions by Johannes A. Koeppen                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QMessageBox>

#include "qdicomdiriconwidget.h"
#include "dcmIconListModel.h"

//qt related includes
#include <QHBoxLayout>
#include <QListView>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QMimeData>
#include <QUrl>

extern int SDR_VERBOSE;

/*!
    \class QDicomDirIconWidget

    \brief A widget to show the thumbnails within a DICOMDIR file.

    Within a DICOMDIR it is possible to store thumbnails of images
    like xray, MR etc. This way it is possible to have an overview
    without loading the real images. The real images can have very large
    file sizes. Thumbnails in a DICOMDIR are optional.
    When the property is set, files can be dropped on the widget.
    Signals the Dicom referenced file id whenever a thumbnail is
    double clicked.

    If property ShowAfterLoadFile all thumbnails found are shown Via
    slot loadFile() the widget can be chained with the
    QDicomDirListWidget to load the same file.
    Slot selectedReferenceFileID() can be connected to the QDicomDirListWidget
    to update the thumbnails according to the selection within the list.
*/

/*!
    Constructs a QDicomDirWidget. Constructs the model and
    view for the widget.
*/
QDicomDirIconWidget::QDicomDirIconWidget(QWidget* parent)
  : QWidget(parent)
  , m_model(0)
  , m_listview(0)
{
  m_model = new CDcmIconListModel();
  m_listview = new QListView(this);
  QHBoxLayout* layout = new QHBoxLayout(this);
  layout->addWidget(m_listview);
  
  m_listview->setViewMode(QListView::IconMode);
  m_listview->setModel(m_model);
  m_listview->show();
  m_listview->setFlow(QListView::LeftToRight);
  m_listview->setWrapping(true);
  m_listview->setResizeMode(QListView::Adjust);
  //setIconSize(QSize(128,128));
  setIconSize(QSize(64, 64));
  m_listview->setSpacing(3);
  
  setAcceptDrops(TRUE);
  
  //enable selecting items in view
  connect(m_model, SIGNAL(currentIndex(const QModelIndex&)),
          m_listview, SLOT(setCurrentIndex(const QModelIndex&)));
  //handle double clicks on thumbnailsAverageFrameRate
  connect(m_listview, SIGNAL(clicked(const QModelIndex&)),
          this, SLOT(clickedHandler(const QModelIndex&)));
  connect(m_listview, SIGNAL(doubleClicked(const QModelIndex&)),
          this, SLOT(doubleClickedHandler(const QModelIndex&)));
          
}


/*!
    Property to set the size of the thumbnails.
*/
QSize QDicomDirIconWidget::iconSize() const
{
  return m_listview->iconSize();
}

/*!
    Property to set the size of the thumbnails.
*/
void QDicomDirIconWidget::setIconSize(const QSize& p_size)
{
  m_listview->setIconSize(p_size);
}

/*!
    Loads a DICOMDIR file into the widget. Opens the DICOMDIR file
    and searches for thumbnails.
*/
void QDicomDirIconWidget::loadDir( const QString &p_dirname )
{
  QString msg;
  char str[1024];
  strcpy(str, p_dirname.toLatin1().data());
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "QDicomDirIconWidget::loadDir %s\n", str) ;
    
  if (! m_model->loadDir(p_dirname))
  {
    m_listview->reset();
    msg = QString("Sdr2 cannot read files in\n '%1'.\nPlease delete item from collection.")
          .arg(p_dirname);
    QMessageBox::warning(this, "Sdr2 Issue", msg);
    //m_model->clear();
  }
}


void QDicomDirIconWidget::loadStringList(QStringList dcm_filename_list)
{
  QString msg;
  
  if (! m_model->loadStringList(dcm_filename_list))
  {
    m_listview->reset();
    msg = QString("Sdr2 cannot read files.\nPlease delete item from collection.");
    QMessageBox::warning(this, "Sdr2 Issue", msg);
  }
}


///support for drag and drop
void QDicomDirIconWidget::dragEnterEvent( QDragEnterEvent* event )
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "QDragEnterEvent received.");
    
  if (event->mimeData()->hasUrls())
  {
    event->acceptProposedAction();
  }
}


///support for drag and drop
void QDicomDirIconWidget::dropEvent( QDropEvent* event )
{
  QList<QUrl> urllist = event->mimeData()->urls();
  //strange! the reported length of the list for one item
  //is 2.
  if(urllist.size() <= 2)
  {
    char * dirname = urllist[0].toLocalFile().toAscii().data();
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "Dir '%s' is dropped on qdicomwidget.\n", dirname);
    loadDir(dirname);
  }
  else
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "Only dropping of single file is supported.");
  }
}

/*!
    Handle double clicked on thumbnail. Signals the reference file id
    and the full path of the dicom image file. Can be used to update
    other widgets.
*/
void QDicomDirIconWidget::doubleClickedHandler(const QModelIndex& index)
{

  QString referenceFileID = m_model->data(index,
                                          CDcmIconListModel::ReferenceFileIDRole).toString();
                                          
  midi = m_model->m_data.at(index.row());
  
  imageSelected(midi);
}


void QDicomDirIconWidget::clickedHandler(const QModelIndex& index)
{
  QString referenceFileID = m_model->data(index,
                                          CDcmIconListModel::ReferenceFileIDRole).toString();
                                          
  midi = m_model->m_data.at(index.row());
  imageClicked(midi);
}


void QDicomDirIconWidget::resetSeriesWindowing(double width, double center)
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "QDicomDirIconWidget should resetSeriesWindowing to width %f and center %f\n",
            width, center);
            
  for (int i = 0; i < m_model->m_data.count(); i++)
  {
    MinDcmImage *midi = m_model->m_data.at(i);
    midi->setWindowWidth(width);
    midi->setWindowCenter(center);
    m_model->m_data.at(i)->m_pixmap =
      QPixmap::fromImage(midi->outputData()).scaled((QSize(128, 128)), Qt::KeepAspectRatio,
          Qt::FastTransformation);
  }
  //stupid update
  QModelIndex idx = m_listview->currentIndex();
  m_model->selectAll();
  m_listview->setCurrentIndex(idx);
}

