/***************************************************************************
 *   Copyright (C) 2007 by Jeroen van der Waal                             *
 *   jwaal@kabelfoon.nl                                                    *
 *                                                                         *
 *   with minor distortions by Johannes A. Koeppen                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

extern int SDR_VERBOSE;

#include <iostream>
#include <streambuf>
#include <string>

#include <QLabel>
#include <QScrollArea>
#include <QScrollBar>
#include <QWheelEvent>
#include <QMimeData>
#include <QUrl>

#include <QByteArray>

#include <dcmtk/ofstd/ofstream.h>
#include <dcmtk/ofstd/ofcond.h>
#include <dcmtk/dcmdata/dctypes.h>

#include "qdicomimagewidget.h"
#include "averageframerate.h"


/*!
    \class QDicomImageWidget
    \brief A widget to show Dicom images.

    This widget can show dicom still images like xray or can play run/loop (video like) recordings.
    QDicomImageWidget is able to pan/zoom images, step through recordings frame by frame, plays
    recording on differents framerates, has contrast and brightness controll and most functionality
    can be controlled with the mouse.

    To load a file into the widget, use slot loadFile(). Dropping of files on the widget is also possible.
    A loaded image is shown immediately in the widget. Some images can have several frames (loop / run) just
    like a movie. Cycling through the images can be means of turning the mouse wheel. Via slots
    nextFrame() play(), stop() playReverse() etc movies can be played. The speed of playing can be changed
    via slot SetSpeed()

    Zoom functionality. When a file is loaded and property AutoZoomAtLoadFile is set, the internal zoomfactor
    is calculated to make the image fit in the available area. The zoomfactor can be changed by holding
    down the ctrl key and turning the mouse wheel. Via a slot the zoomfactor can be set (a slider e.g.)
    via setZoomFactor(); When property AutoZoomAtResize is set the widget tries to fit the image in the
    available space when the widget size changes.

    Contrast / brightness support. Via setWindowWidth() and setWindowCenter() the image brightness and
    contrast can be changed. Please refer to the Dicom standard how these two functions operate.
*/

/// Constructs the widget
QDicomImageWidget::QDicomImageWidget(QWidget *parent)
  : QScrollArea(parent)
  , m_qdicomImage(0)
  , m_label(0)
  , m_timer(this)
  , m_currentFrame(0)
  , m_playmode(NONE)
  , m_speed(100)
  , m_zoomfactor(50) //100)
  , m_zoomfactorChanged(false)
  , m_autozoomAtLoadFile(true)
  , m_autozoomAtResize(true)
  , m_lastMousePosition()
  , m_averageFrameRate(0)
  , m_useMinMaxWindow(true)
  , m_defaultWindowCenter(0.0)
  , m_defaultWindowWidth(0.0)
{

  m_averageFrameRate = new AverageFrameRate();  //< performance measurements
  m_label = new QLabel();
  setBackgroundRole(QPalette::Dark);
  setAutoFillBackground(true);
  setAlignment(Qt::AlignCenter);
  m_label->resize(viewport()->size());
  m_label->setAlignment(Qt::AlignCenter);
  setWidget(m_label);
  
  //'Property' of the widget, default set to true.
  setAcceptDrops(TRUE);
  
  dcmHeaderToQTextEdit = new QTextEdit();
  dcmHeaderToQTextEdit->setWindowModality(Qt::ApplicationModal);
  dcmHeaderToQTextEdit->createStandardContextMenu();
  dcmHeaderToQTextEdit->hide();
  
  //timer handler for run/loop playing
  connect(&m_timer, SIGNAL(timeout()), this, SLOT(handleTimer()));
}

/// Destructs the widget
QDicomImageWidget::~QDicomImageWidget()
{
  if(m_averageFrameRate)
  {
    delete m_averageFrameRate;
  }
  
  if(m_qdicomImage)
  {
    //delete m_qdicomImage;
  }
}

void QDicomImageWidget::clearDcmImage()
{
  //if playing, stop
  stop();
  
  //init values before showing another dicom file
  m_currentFrame = 0;
  m_speed = 100;
  m_zoomfactor = 50; //100;
  m_zoomfactorChanged = false;
  
  //create a new qdicomimage for the new dicom file
  //if already a file was opened, close this one
  //if(m_qdicomImage != NULL)
  {
    //delete m_qdicomImage;
    //printf("QDicomImageWidget::clear sets m_qdicomImage = NULL\n");
    m_qdicomImage = NULL;
    pixmap = NULL;
    m_label->setPixmap(pixmap);
  }
  //m_label->setText("");
  //showFrame();
  //this->repaint();
  emit dcmImageChanged();
  updateGeometry();
}


/*!
   Loads dicom file and shows the file in the current widget space. When
   property autoZoomAtLoadFile is set to true, the image will be fit in the widget.
   \param filename
 */
void QDicomImageWidget::loadFile(const QString& filename)
{
  clearDcmImage();
  //if playing, stop
  //stop();
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "QDicomImageWidget::loadFile calls new MinDcmImage\n");
  m_qdicomImage = new MinDcmImage(filename, true);
  
  if((m_qdicomImage == NULL) || (!m_qdicomImage->m_success))
  {
    //no succes
    m_label->setText("File not loaded");
    
    //signals to appropriate values.
    multiframe(false);
    currentFrame(0);
    zoomValueChanged(100);
    return;
  }
  m_label->setText("");
  
  
  if(m_useMinMaxWindow)
  {
    fprintf(stdout, "if(m_useMinMaxWindow)\n");
    m_qdicomImage->setMinMaxWindow();
    m_defaultWindowWidth = m_qdicomImage->windowWidth();
    m_defaultWindowCenter = m_qdicomImage->windowCenter();
  }
  
  //make image fit and show the image
  if(m_autozoomAtLoadFile)
  {
    QSize s(width(), height());
    autoZoom(s);
  }
  else
  {
    showFrame();
  }
  
  //Is the image a multi frame?
  multiframe(m_qdicomImage->frameCount() > 1); //signal
}


void QDicomImageWidget::showDcmHeader()
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo QDicomImageWidget::loadMinDcmImage()\n");
    
  if(m_qdicomImage == NULL)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "QDicomImageWidget::loadMinDcmImage(), m_qdicomImage == NULL\n");
    return;
  }
  
  std::ostream os(&buffer);
  
  dcmHeaderToQTextEdit->clear();
  
  QFont *textFont = new QFont;
  textFont->setFamily("monospace [Consolas]");
  textFont->setFixedPitch(true);
  textFont->setStyleHint(QFont::TypeWriter);
  dcmHeaderToQTextEdit->setFont(*textFont);
  //dcmHeaderToQTextEdit->setFontPointSize(11);
  //dcmHeaderToQTextEdit->t   QTextFormat::BlockFormat
  //dcmHeaderToQTextEdit->set >font().setFixedPitch(true);
  //dcmHeaderToQTextEdit->setFontFamily("Monospace");
  dcmHeaderToQTextEdit->setDocumentTitle("Sdr2 Volume Information");
  dcmHeaderToQTextEdit->setGeometry(200, 100, 800, 600);
  //QDebugStream qout(os, dcmHeaderToQTextEdit);
  //QDebugStream qout(sstream, dcmHeaderToQTextEdit);
  
  DcmDataset *dataset = m_qdicomImage->m_dicomFileFormat->getDataset();
  dataset->print(os, DCMTypes::PF_shortenLongTagValues);
  
  //dataset->print(std::cout, DCMTypes::PF_shortenLongTagValues);
  QString qstr = QString::fromStdString(buffer.str());
  dcmHeaderToQTextEdit->setText(qstr);
  dcmHeaderToQTextEdit->show();
}


void QDicomImageWidget::loadMinDcmImage(MinDcmImage *midi)
{
  clearDcmImage();
  
  //try to load the file
  m_qdicomImage = midi;
  //if((m_qdicomImage == NULL) || (!m_qdicomImage->m_success()))
  if(m_qdicomImage == NULL)
  {
    //no succes
    m_label->setText("Image not loaded");
    
    //signals to appropriate values.
    multiframe(false);
    currentFrame(0);
    zoomValueChanged(100);
    return;
  }
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "show header ...\n");
  DcmDataset *dataset = m_qdicomImage->m_dicomFileFormat->getDataset();
  
  showDcmHeader();
  
  m_label->setText("");
  
  
  if(m_useMinMaxWindow)
  {
    m_defaultWindowWidth = m_qdicomImage->windowWidth();
    //fprintf(stdout, "\n")
    m_defaultWindowCenter = m_qdicomImage->windowCenter();
  }
  
  //make image fit and show the image
  if(m_autozoomAtLoadFile)
  {
    QSize s(width(), height());
    autoZoom(s);
  }
  else
  {
    showFrame();
  }
  
  //Is the image a multi frame?
  multiframe(m_qdicomImage->frameCount() > 1); //signal
  
}


/* Display image already loaded */
/*
void QDicomImageWidget::loadDcmImage(MinDcmImage *dcmImg)
{
    //if playing, stop
  stop();

  //init values before showing another dicom file
  m_currentFrame = 0;
  m_speed = 100;
  m_zoomfactor = 100;
  m_zoomfactorChanged = false;

  m_qdicomImage = dcmImg;

  if(m_useMinMaxWindow)
  {
    m_qdicomImage->setMinMaxWindow();
    m_defaultWindowWidth = m_qdicomImage->windowWidth();
    m_defaultWindowCenter = m_qdicomImage->windowCenter();
  }

  //make image fit and show the image
  if(m_autozoomAtLoadFile)
  {
    QSize s(width(), height());
    autoZoom(s);
  }
  else
  {
    showFrame();
  }

  //Is the image a multi frame?
  multiframe(m_qdicomImage->frameCount() > 1); //signal
}
*/


/*!
    Advance to the next frame in case of a 'multi frame dicom file'. The widget
    content is immediately updated. The call is ignored in case of the last frame,
    or in case the the dicom file contains only a single frame.
*/
void QDicomImageWidget::nextFrame()
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  stop();
  gotoNextFrame();
}

/*!
    Advance to the previous frame in case of a 'multi frame dicom file'. The widget
    content is immediately updated. The call is ignored in case of the first frame,
    or in case the the dicom file contains only a single frame.
*/
void QDicomImageWidget::previousFrame()
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  stop();
  gotoPreviousFrame();
}

/*!
    \fn QDicomImageWidget::showFrame()
 */
void QDicomImageWidget::showFrame()
{
  //signal for every frame change.
  static int previousFrame = -1;
  //printf("QDicomImageWidget::showFrame\n");
  if(m_currentFrame != previousFrame)
  {
    previousFrame = m_currentFrame;
    currentFrame(m_currentFrame);  //signal
  }
  
  //calculate resolution according to zoom factor
  QSize s = m_qdicomImage->size() * ((double)m_zoomfactor / 100.0);
  
  //convert and scale image.
  //QPixmap
  pixmap = QPixmap::fromImage(m_qdicomImage->outputData(m_currentFrame, 0)).scaled(s);
  m_label->resize(qMax(viewport()->size().width(), pixmap.size().width()),
                  qMax(viewport()->size().height(), pixmap.size().height()));
  m_label->setPixmap(pixmap);
  m_averageFrameRate->nextFrameReady();
  updateGeometry();
  
  //signals for average framerate
  averageFrameRate( (int) m_averageFrameRate->averageFrameRate());
  averageFrameRate(m_averageFrameRate->averageFrameRate());
}

/*!
    Plays a multi frame dicom file using the frame time specified in the dicom file.
    Ignored in case of a single frame dicom file. When cine mode is 'loop',
    the frames will be played in a endless loop. If the cine mode is 'no loop' the play
    ends at the last frame.
 */
void QDicomImageWidget::play()
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  //a run/loop is only playable if more than 1 frame is available.
  startTimer(FORWARD);
}

/*!
   Same as play() but now the frames are shown in reverse order.
*/
void QDicomImageWidget::playReverse()
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  //a run/loop is only playable if more than 1 frame is available.
  startTimer(REVERSE);
}

/*!
    Stops playing the subsequent frames of the current dicom file.
*/
void QDicomImageWidget::stop()
{
  // stop the timer, cine will end
  m_averageFrameRate->stopMeasuring();
  averageFrameRate((int) 0);
  averageFrameRate(0.0f);
  
  m_timer.stop();
  m_playmode = NONE;
}


/*!
    \fn QDicomImageWidget::gotoNextFrame()
 */
void QDicomImageWidget::gotoNextFrame()
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  m_currentFrame++;
  if(m_currentFrame >= (int)m_qdicomImage->frameCount())
  {
    m_currentFrame = 0;
  }
  showFrame();
}


void QDicomImageWidget::gotoPreviousFrame()
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  if(m_currentFrame == 0)
  {
    m_currentFrame = (m_qdicomImage->frameCount() - 1);
  }
  else
  {
    m_currentFrame--;
  }
  showFrame();
}


void QDicomImageWidget::handleTimer()
{
  Q_ASSERT(m_qdicomImage != NULL);
  if(m_playmode == FORWARD)
  {
    gotoNextFrame();
  }
  else if(m_playmode == REVERSE)
  {
    gotoPreviousFrame();
  }
}


void QDicomImageWidget::startTimer(playmode p_playmode)
{
  Q_ASSERT(m_qdicomImage != NULL);
  
  //Needed, it can be running already from another call to startTimer.
  m_timer.stop();
  
  //calculate time between frames
  m_averageFrameRate->startMeasuring();
  unsigned long frametime = ((100 * m_qdicomImage->frameTimeMsec()) / m_speed);
  if(m_qdicomImage->frameCount() > 1 && p_playmode != NONE)
  {
    m_playmode = p_playmode;
    m_timer.start(frametime);
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "Play/Reverse Play not started, no frames to play or "
              "in the wrong mode.");
  }
}


/*!
    Change the speed of play/reverse play. Range from 30%-200%.
    \param p_speed integer in the range of 30-200
*/
void QDicomImageWidget::setSpeed(int p_speed)
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  //check range
  if(p_speed >= 10 && p_speed <= 200)
  {
    m_speed = p_speed;
    if(m_playmode != NONE)
    {
      startTimer(m_playmode);
    }
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "setSpeed did not change the speed. The speed value is out of"
              "range (10-200)");
  }
}


/*!
    Set zoom factor. Range 30% - 200%. After changing the zoom factor
    the contents of the widget is immediately updated. 100% is the speed
    that is indicated within the dicom file itself. Depending on the
    hardware, the fps is reached or not. See averageFrameRate()
    \param p_zoomfactor a integer in the range of 30 - 200
*/
void QDicomImageWidget::setZoomFactor(int p_zoomfactor)
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  //if(p_zoomfactor >= 30 && p_zoomfactor <= 200)
  if(p_zoomfactor >= 30 && p_zoomfactor <= 400)
  {
    m_zoomfactor = p_zoomfactor;
    m_zoomfactorChanged = true;
    zoomValueChanged(m_zoomfactor); //signal
    if(m_playmode == NONE)
    {
      showFrame();
    }
  }
  else
  {
    //qDebug("setZoom did not change the zoom factor. The value is out of"
    //       "range (30-400)");
  }
}

/*!
   Calculates the zoom factor needed to just fit
   an image in the available area. Zoomfactor is somewhere
   between 30% and 400 //200%
   \param size the available area to fit the image in
 */
void QDicomImageWidget::autoZoom(const QSize& size)
{
  if(m_qdicomImage == NULL)
  {
    //printf("QDicomImageWidget::autoZoom should return\n");
    return;
  }
  //printf("QDicomImageWidget::autoZoom didn't return\n");
  QSize img = m_qdicomImage->size();
  int factorx = (size.width() * 200) / img.width(); //*100
  int factory = (size.height() * 200) / img.height();
  
  int zoomfactor = qMin(factorx, factory);
  zoomfactor -= 2; //some extra space to prevent scroll sliders to appear
  if(zoomfactor < 30)
  {
    zoomfactor = 30;
  }
  if (zoomfactor > 400) //200
  {
    zoomfactor = 400; //200
  }
  //qDebug("AutoZoom factor calculated at: %d%%", zoomfactor);
  setZoomFactor(zoomfactor);
}

/*!
    Autozoom means that the image is resized such that it just fits
    the available widget space. This bool indicates that this options
    is enabled/disabled for when images are loaded for the first time.
    \return true if this option is enabled.
*/
bool QDicomImageWidget::autoZoomAtLoadFile() const
{
  return m_autozoomAtLoadFile;
}

/*!
    See autoZoomAtLoadFile()
    \param autozoom set this to true to activate this option.
*/
void QDicomImageWidget::setAutoZoomAtLoadFile(bool autozoom)
{
  m_autozoomAtLoadFile = autozoom;
}


/**
    Same as autoZoomAtLoadFile() but now for every resize.
    the image will fit for all sizes, as long the zoom factor
    is within range 30%-400% //200%.
    \return true if this option is enabled.
*/
bool QDicomImageWidget::autoZoomAtResize() const
{
  return m_autozoomAtResize;
}

/*!
    Same as autoZoomAtLoadFile() but now for every resize.
    the image will fit for all sizes, as long the zoom factor
    is within range 30%-200%.
    \return true if this option is enabled.
*/
void QDicomImageWidget::setAutoZoomAtResize(bool autozoom)
{
  m_autozoomAtResize = autozoom;
}

/*!
   Change the current setting for the window width (contrast) by percentage.
   the range is -100% to +100%. The current window value is 0%. This slot can
   be used for example with a slider with values ranging from -50 to +50.
   \param percentage a value in the range of -100, 100
*/
void QDicomImageWidget::setCurrentImageWindowWidth(int percentage)
{
  //valid range for the percentage is -100 till 100
  if(percentage > 100)
  {
    percentage = 100;
  }
  else if(percentage < -100)
  {
    percentage = -100;
  }
  
  //percentage *= -1;
  
  if(m_defaultWindowWidth == double(0.0))
  {
    //qDebug("defaultWindowWidth not set, can not change window width");
    return;
  }
  
  double window = qMax((double(100 - percentage) / double(100.0)) * m_defaultWindowWidth, 1.0);
  //double window = qMax((double(percentage)/double(100.0))*m_defaultWindowWidth, 1.0);
  m_qdicomImage->setWindowWidth(window);
  //imageWidthOrCenterReleased();
  showFrame();
}

/*!
    Change the current setting for the window center (brightness) by percentage.
    The range is -100% to +100%. The current window center value is taken to be the
    0% setting. See also setWindowWidth()
    \param percentage an integer value in the range -100, 100
*/
void QDicomImageWidget::setCurrentImageWindowCenter(int percentage)
{
  //valid range for the percentage is -100 till 100
  if(percentage > 100)
  {
    percentage = 100;
  }
  else if(percentage < -100)
  {
    percentage = -100;
  }
  
  //percentage *= -1;
  
  double minValue, maxValue;
  m_qdicomImage->minMaxValues(minValue, maxValue);
  
  if (m_defaultWindowCenter == 0.0)
  {
    // better something than nothing ??
    m_defaultWindowCenter = (minValue + maxValue) / 2;
    if (m_defaultWindowCenter == 0.0)
    {
      //qDebug("setWindowCenter not set, cannot change window center");
      return;
    }
  }
  
  //double minValue, maxValue;
  //m_qdicomImage->minMaxValues(minValue, maxValue);
  
  double center = m_defaultWindowCenter -
                  (double(percentage) / double(200.0)) * (maxValue - minValue);
  m_qdicomImage->setWindowCenter(center);
  //imageWidthOrCenterReleased();
  showFrame();
  //imageWidthOrCenterReleased();
}

//void QDicomImageWidget::setSeriesWindowing(double width, double center)
//{
//
//}

void QDicomImageWidget::imageWidthOrCenterReleased()
{
  double width;
  double center;
  
  if(!m_qdicomImage)
    return;
    
  width = m_qdicomImage->windowWidth();
  center = m_qdicomImage->windowCenter();
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "QDicomImageWidget::imageWidthOrCenterReleased,\n width = %f, center = %f\n",
            width, center);
  emit setSeriesWindowing(width, center);
}


// -- Event handlers --

/*!
    \fn QDicomImageWidget::resizeEvent ( QResizeEvent * )
 */
void QDicomImageWidget::resizeEvent ( QResizeEvent *ev )
{
  //When no image is openened, do nothing.
  if(!m_qdicomImage)
    return;
    
  if(m_autozoomAtResize)
  {
    autoZoom(ev->size());
  }
}

/*!
    The mousewheel events are used for next/previous frame and
    for zoom in and out (wheel + ctrl key)
*/
void QDicomImageWidget::wheelEvent (QWheelEvent * e)
{
  //qDebug("QWheelEvent received.");
  
  int angle = e->delta();
  if (e->modifiers() == Qt::NoModifier)
  {
    //Scrollwheel without pressing any keyboard key
    //is next, previos frame.
    if(angle > 0)
    {
      nextFrame();
    }
    else
    {
      previousFrame();
    }
    //let know that the event is handles
    e->accept();
  }
  else if(e->modifiers() == Qt::ControlModifier)
  {
    //wheel + ctrl button on keyboard down
    //is using zoom functionality
    if(angle > 0)
    {
      //setZoomFactor(qMin(200u, m_zoomfactor+5));
      setZoomFactor(qMin(200u, m_zoomfactor + 10));
    }
    else
    {
      //setZoomFactor(qMin(200u, m_zoomfactor-5));
      setZoomFactor(qMin(200u, m_zoomfactor - 10));
    }
    e->accept();
  }
}

/**
  * Support for dropping files on the widget. If an file is
  * dropped above the widget it is opened in the widget.
  * Easy for testing and other purposes.
  * \param event QDragEnterEvent
  */
void QDicomImageWidget::dragEnterEvent(QDragEnterEvent* event)
{
  //qDebug("QDragEnterEvent received.");
  if(event->mimeData()->hasUrls())
  {
    event->acceptProposedAction();
  }
}

void QDicomImageWidget::dropEvent(QDropEvent* event)
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "QDicomImageWidget::dropEvent\n");
  QList<QUrl> urllist = event->mimeData()->urls();
  //strange! the reported length of the list for one item
  //is 2.
  if(urllist.size() <= 2)
  {
    char * filename = urllist[0].toLocalFile().toAscii().data();
    //qDebug("\"File \'%s\' is dropped on qdicomwidget.\"", filename);
    loadFile(filename);
  }
  else
  {
    //qDebug("Only dropping of single file is supported.");
  }
}

/**
  * Support for dragging an image with the rightmouse button down
  * only when scrollbars are visible, the image can be moved around
  * using this option.
  */
void QDicomImageWidget::mousePressEvent (QMouseEvent* e)
{
  e->accept();
  m_lastMousePosition = e->pos();
  //qDebug("Started mouse dragging x %d, y %d", m_lastMousePosition.x(),
  //      m_lastMousePosition.y());
}

void QDicomImageWidget::mouseMoveEvent (QMouseEvent* e)
{
  e->accept();
  //qDebug("dragging");
  QPoint delta = m_lastMousePosition - e->pos();
  //vertical scrollbar
  QScrollBar* ver = verticalScrollBar();
  ver->setValue(ver->value() + delta.y());
  
  QScrollBar* hor = horizontalScrollBar();
  hor->setValue(hor->value() + delta.x());
  
  //qDebug("Move x %d, y %d", delta.x(), delta.y());
  m_lastMousePosition = e->pos();
}


//signals

/*!
    \fn void QDicomImageWidget::zoomValueChanged(int value)

    In case the zoom value has changed, this signal can be used to obtain the
    new value. The zoom factor can be changed when autozoom is active or when
    the zoom factor is changed via slot setZoomFactor(int)
    @param value the new zoom factor
*/

/*!
    \fn void QDicomImageWidget::currentFrame(int value)

    In case that another frame is shown, this signal provides the
    current frame.
    \param value the current frame that is displayed.

*/

/*!
    \fn void QDicomImageWidget::frameRate(float value)

    The frame rate as specified by the current dicom image.
    \param value the framerate in float format
*/

/*!
    \fn void QDicomImageWidget::averageFrameRate(double average)

     The average frame rate in case a run/loop is playing.
     can be used for performance measurement.
     \param average the calculated average in float format
*/

/*!
    \fn  void QDicomImageWidget::averageFrameRate(int average)

    The average frame rate
    \param average the calculated average in integer format
*/

/*!
    \fn void QDicomImageWidget::multiframe( bool enable)
    If a Dicom file has two or more frames (video like) this
    signal can be used to enable the buttons for play, stop
    etc.
    \param enable true if buttons should be enabled.
*/
