/***************************************************************************
 *   Copyright (C) 2007 by Jeroen van der Waal                             *
 *   jwaal@kabelfoon.nl                                                    *
 *                                                                         *
 *   with minor distortions by Johannes A. Koeppen                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef QDICOMWIDGET_H
#define QDICOMWIDGET_H

#include <QTimer>
#include <QScrollArea>
#include <QPixmap>
#include <QtGui/QTextEdit>
#include <QDebug>

#include "minDcmImage.h"

//forward declarations
class QLabel;
class QPixmap;
class MinDcmImage;
class AverageFrameRate;

class QDicomImageWidget : public QScrollArea
{
  Q_OBJECT
  Q_PROPERTY(bool AutoZoomAtLoadFile READ autoZoomAtLoadFile WRITE setAutoZoomAtLoadFile)
  Q_PROPERTY(bool AutoZoomAtResize READ autoZoomAtResize WRITE setAutoZoomAtResize)
  
public:
  QDicomImageWidget(QWidget *parent = 0);
  virtual ~QDicomImageWidget();
  bool autoZoomAtLoadFile() const;
  void setAutoZoomAtLoadFile(bool enable);
  bool autoZoomAtResize() const;
  void setAutoZoomAtResize(bool enable);
  virtual void dragEnterEvent(QDragEnterEvent* e);
  virtual void dropEvent(QDropEvent* e);
  
public slots:
  void clearDcmImage();
  void loadFile(const QString& filename);
  void showDcmHeader();
  void loadMinDcmImage(MinDcmImage *midi);
  void nextFrame();
  void previousFrame();
  void play();
  void playReverse();
  void stop();
  void setSpeed(int);
  void setZoomFactor(int);
  void setCurrentImageWindowWidth(int percentage);
  void setCurrentImageWindowCenter(int percentage);
  void imageWidthOrCenterReleased();
  
signals:
  void zoomValueChanged(int value);
  void currentFrame(int value);
  void frameRate(float value);
  void averageFrameRate(double average);
  void averageFrameRate(int average);
  void multiframe(bool);
  void dcmImageChanged();
  void setSeriesWindowing(double width, double center);
  
private slots:
  void handleTimer();
  
protected:
  virtual void resizeEvent (QResizeEvent*);
  virtual void wheelEvent (QWheelEvent* e);
  virtual void mousePressEvent (QMouseEvent* e);
  virtual void mouseMoveEvent (QMouseEvent* e);
  
private:
  typedef enum playmode { NONE = 1, FORWARD, REVERSE };
  std::stringbuf buffer;
  /**
    * When a multi frame dicom file is to be played, the timer
    * is started with the playmode direction. The timer handler will advance
    * to the next or previous frame. The timer stops when stop-timer
    * is issued, or when the end of the multi frame is reached and
    * cine loop is set to 'false'.
    * @param p_playmode forward or reverse is indicated by
    * this parameter.
    */
  void startTimer(playmode p_playmode);
  /**
   * Shows the current frame in the widget.
   */
  void showFrame();
  /**
   * When the widget is created, or the widget is resized,
   * this function calculates the appropriate setting of the
   * zoomfactor, such that the content of the widget
   * appears without sliders for panning. This will only happen
   * if the zoomfactor is in between its minimum and maximum value.
   * @param size the size of the (new) widget area.
   */
  void autoZoom(const QSize& size);
  void gotoNextFrame();
  void gotoPreviousFrame();
  
private:
  MinDcmImage *m_qdicomImage;
  QLabel* m_label;
  
  QTimer m_timer; ///<used for playing loop or runs
  int m_currentFrame; ///<current frame that is showed in the widget
  playmode m_playmode; ///<which cine play are we in, none, forward or reverse
  
  /// indicated speed of playing 100 is normal speed, 200 is double speed and 10 is
  /// 10 percent of the framerate indicated within the dicom file. Maximums are
  /// from 10 to 200.
  unsigned m_speed;
  
  /// indicated zoom factor. 100 is original size, 200 is double size. 30 is 30% of
  /// the original size. Range 30-400 //200
  unsigned m_zoomfactor;
  
  /// True in case the zoomfactor has changed. Used to correct redraw
  /// the contents of the widget.
  bool m_zoomfactorChanged;
  
  /** Autozoom means that the widget calculates the zoomfactor such that
   *  the dicom image fits within the available space of the widget.
   *  this only will happen if the zoomfactor is more than 30% and less
   *  than 200%
   */
  bool m_autozoomAtLoadFile;
  bool m_autozoomAtResize;
  QPoint m_lastMousePosition; ///< for dragging image with mouse
  AverageFrameRate* m_averageFrameRate; ///<class for measuring framerate
  
  /// Window (contrast brightness control)
  bool m_useMinMaxWindow; ///<should the minmax algorithm be used for windowing?
  double m_defaultWindowCenter;
  double m_defaultWindowWidth;
  QPixmap pixmap;
  
  QTextEdit *dcmHeaderToQTextEdit;
};

#endif

