/**************************************************************************
 *  Author      : Johannes A. Koeppen, koeppen@ieee.org                   *
 *  Year(s)     : 2011 - 2015                                             *
 *  File        : saveVolume.cpp                                            *
 *  Context     : Sdr2 (Not approved for clinical use)                    *
 *  License     : GPL 2 or later                                          *
 *  Description : save DICOM series as Nifti or Analyze file             *
 *                                                                        *
 **************************************************************************/


#include <iostream>
#include <math.h>

#include "./coin3d_math/SbVec3f.h"
#include "./coin3d_math/SbPlane.h"
#include "./coin3d_math/SbMatrix.h"
#include "./coin3d_math/SbRotation.h"

#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>

#ifdef MinGW
#include <basetsd.h>
#endif

#include "globals.h"
#include "saveVolume.h"

extern int SDR_VERBOSE;

class SbPlane;

SaveVolume::SaveVolume(QWidget *parent)
  : QDockWidget(parent)
{
  setupUi(this);
  
  obliquityThresholdCosineValue = 0.8;
  textEditSorting->clear();
  m_avrg_dcmImageThickness = 0.0;
  m_ref_mr_echo_time = 0.0;
  m_numNiftiSlices = 0;
  m_minInstanceNum = 100000;
  m_maxInstanceNum = 0;
  QDir currentDir = QDir::current();
  currentDir.cdUp();
  this->m_filedir = currentDir.absolutePath();
  this->m_filename = "";
  
  sorted = false;
  pushButtonOK->setEnabled(true);
  m_have_image_orientation_patient_array = false;
  m_have_image_position_patient_array = false;
  
  connect(pushButtonCancel, SIGNAL(clicked()), this, SLOT(slot_cancel()));
  connect(pushButtonOK, SIGNAL(clicked()), this, SLOT(slotFileSaveAs()));
}


SaveVolume::~SaveVolume()
{
  // ?? DJDecoderRegistration::cleanup();
}


void SaveVolume::setReferenceDcmImage(MinDcmImage *selectedDcmImage)
{
  QByteArray ba = selectedDcmImage->m_series_label.toLatin1();
  const char *ref_label = ba.data();
  if (SDR_VERBOSE > 0)
    fprintf(stdout, "SaveVolume::setReferenceDcmImage with label %s\n",
            ref_label);
            
  pushButtonOK->setEnabled(true);
  success = true;
  QString msg;
  m_fittingDcmImageVector.clear();
  m_avrg_dcmImageThickness = 0.0;
  m_ref_mr_echo_time = 0.0;
  m_dcmImageHeightInt = 0;
  m_dcmImageWidthInt = 0;
  m_ref_mr_echo_time = 0.0;
  
  textEditSorting->clear();
  m_referenceDcmImage = selectedDcmImage;
  m_dcmImageHeightInt = m_referenceDcmImage->m_dicomImage->getHeight();
  m_dcmImageWidthInt = m_referenceDcmImage->m_dicomImage->getWidth();
  m_patient_position = m_referenceDcmImage->m_patient_position.trimmed();
  
  m_ref_mr_echo_time = m_referenceDcmImage->m_mr_echo_time;
  if (m_ref_mr_echo_time == 0.0)
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "setReferenceDcmImage, m_mr_echo_time NOT found\n");
  }
  else
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "setReferenceDcmImage, m_mr_echo_time = %.2f.\n",
              m_ref_mr_echo_time);
  }
  
  double image_position_patient[3];
  if (qStringToDoubleArray(m_referenceDcmImage->m_image_position_patient_str,
                           3, image_position_patient) == 3)
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "setReferenceDcmImage, patient orientation found\n");
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "setReferenceDcmImage, patient orientation NOT found\n");
  }
  
  if (SDR_VERBOSE > 0)
  {
    char p_pos_str[1024];
    strcpy(p_pos_str, m_patient_position.trimmed().toLatin1().data());
    fprintf(stdout, "selected image, patient orientation: '%s'\n", p_pos_str);
  }
  
  fix_rot_row_1[0] = 1.0f;
  fix_rot_row_1[1] = 1.0f;
  fix_rot_row_1[2] = 1.0f;
  fix_rot_row_2[0] = 1.0f;
  fix_rot_row_2[1] = 1.0f;
  fix_rot_row_2[2] = 1.0f;
  fix_rot_row_3[0] = 1.0f;
  fix_rot_row_3[1] = 1.0f;
  fix_rot_row_3[2] = 1.0f;
  
  // needed from FIRST image !
  double tst_orientation_patient[6];
  if (qStringToDoubleArray(m_referenceDcmImage->m_image_orientation_patient_str,
                           6, tst_orientation_patient) == 6)
  {
  }
  else
  {
    textEditSorting->append("Could not find image orientation.");
  }
}


// not really necessary,
//calculate Patient Orientation (0020,0020)
//from first row in Image Orientation (Patient) (0020,0037)
//Anatomical direction shall be designated by the capital letters:
//A (anterior), P (posterior), R (right), L (left), H (head), F (foot).
//Each value of the orientation attribute shall contain at least
//one of these characters.
//If refinements in the orientation descriptions are to be specified,
//then they shall be designated by one or two additional letters in each value.
//Within each value, the letters shall be ordered with the
//principal orientation designated in the first character.
//void
QString
SaveVolume::calculatePatientOrientation(double x, double y, double z)
{
  //fprintf(stdout, "SaveVolume::calculatePatientOrientation\n");
  QString p_o_qstr = "n.n.";
  QString msg;
  char *orientation = new char[4];
  char *optr = orientation;
  *optr = '\0';
  
  char orientationX = x < 0 ? 'R' : 'L';
  char orientationY = y < 0 ? 'A' : 'P';
  char orientationZ = z < 0 ? 'F' : 'H';
  
  double absX = fabs(x);
  double absY = fabs(y);
  double absZ = fabs(z);
  
  int i;
  for (i = 0; i < 3; ++i)
  {
    if (absX > .0001 && absX > absY && absX > absZ)
    {
      *optr++ = orientationX;
      absX = 0;
    }
    else if (absY > .0001 && absY > absX && absY > absZ)
    {
      *optr++ = orientationY;
      absY = 0;
    }
    else if (absZ > .0001 && absZ > absX && absZ > absY)
    {
      *optr++ = orientationZ;
      absZ = 0;
    }
    else break;
    *optr = '\0';
  }
  
  msg = QString("Patient orientation is %1.")
        .arg(orientation);
  textEditSorting->append(msg);
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "patient orientation is %s\n", orientation);
    
  p_o_qstr = (QString) orientation;
  return p_o_qstr;
}


void SaveVolume::setOrigDcmImageVector(TMinDcmImageVector *dcmImageVector)
{
  QString msg;
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::setOrigDcmImageVector\n");
    
  m_fittingDcmImageVector.clear();
  m_origDcmImageVector = dcmImageVector;
  setFittingDcmImageVector(m_origDcmImageVector);
}

/* include dcm images only if :
- m_series_instance_uid,
- m_frame_of_reference,
- m_dcmImageHeight, and m_dcmImageWidth
- m_image_orientation_patient_str
match m_referenceDcmImage
*/
void SaveVolume::setFittingDcmImageVector(TMinDcmImageVector *dcmImageVector)
{
  QString msg;
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "void SaveVolume::setFittingDcmImageVector for echo time %.2f\n",
            m_ref_mr_echo_time);
            
  int w, h;
  int count = 0;
  
  m_minInstanceNum = 100000;
  m_maxInstanceNum = 0;
  
  bool all_have_windowing_info = true;
  
  m_fittingDcmImageVector.clear();
  
  TMinDcmImageVector::iterator it;
  for(it = dcmImageVector->begin(); it != dcmImageVector->end(); it++)
  {
    w = (*it)->m_dicomImage->getWidth();
    h = (*it)->m_dicomImage->getHeight();
    if(((*it)->m_study_instance_uid == m_referenceDcmImage->m_study_instance_uid)
        && ((*it)->m_frame_of_reference == m_referenceDcmImage->m_frame_of_reference)
        && (w == m_dcmImageWidthInt) && (h == m_dcmImageHeightInt)
        && ((*it)->m_image_orientation_patient_str == m_referenceDcmImage->m_image_orientation_patient_str)
        && ((*it)->m_mr_echo_time == m_ref_mr_echo_time) )
    {
    
      bool b = FALSE;
      count++;
      QString numqs = ((QString) (*it)->m_instance_number);
      
      int numi = numqs.toInt(&b);
      if (b)
      {
        if (numi < m_minInstanceNum)
          m_minInstanceNum = numi;
        if (numi > m_maxInstanceNum)
          m_maxInstanceNum = numi;
      }
      m_fittingDcmImageVector.append(*it);
      
      if ((*it)->m_dicomImage->getWindowCount() < 1)
      {
        all_have_windowing_info = false;
        if (SDR_VERBOSE > 1)
          fprintf(stdout, "SaveVolume::setFittingDcmImageVector, dicomImage wo number of VOI.\n");
      }
      else
      {
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "SaveVolume::setFittingDcmImageVector, dicomImage with number of VOI.\n");
      }
    }
  }
  if (m_maxInstanceNum > 0)
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "SaveVolume::setFittingDcmImageVector instance_number min %i, max %i \n",
              m_minInstanceNum, m_maxInstanceNum);
  }
  else
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "SaveVolume::setFittingDcmImageVector instance_number <= 0\n");
    m_minInstanceNum = 1;
    m_maxInstanceNum = count;
  }
  msg = QString("%1 image(s) fit to reference image.")
        .arg(m_fittingDcmImageVector.size());
        
        
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "setFitting volume z planes to min %i and max %i\n",
            m_minInstanceNum, m_maxInstanceNum);
            
  this->spinBoxStartAtSlice->setMinimum(m_minInstanceNum);
  this->spinBoxStartAtSlice->setMaximum(m_maxInstanceNum);
  this->spinBoxStartAtSlice->setValue(m_minInstanceNum);
  this->spinBoxStartAtSlice->repaint(); // >update();
  
  this->spinBoxStopAtSlice->setMinimum(m_minInstanceNum);
  this->spinBoxStopAtSlice->setMaximum(m_maxInstanceNum);
  this->spinBoxStopAtSlice->setValue(m_maxInstanceNum);
  this->spinBoxStopAtSlice->repaint(); //>update();
  
  textEditSorting->append(msg);
}


void SaveVolume::slot_cancel()
{
  m_fittingDcmImageVector.clear();
  m_avrg_dcmImageThickness = 0.0;
  m_ref_mr_echo_time = 0.0;
  textEditSorting->clear();
  this->hide();
}


void SaveVolume::slot_sort()
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo SaveVolume::slot_sort()\n");
    
  m_avrg_dcmImageThickness = 0.0;
  float normal[3];
  double image_position_patient_array[3];
  double image_orientation_patient_array[6];
  double distance_to_origin;
  double maxImageThickness = -1.0;
  double minImageThickness = 100000.0;
  double currentImageThickness = 0.0;
  double thickness0018_0050 = 0.0;
  
  m_numNiftiSlices = 0;
  
  QString msg;
  success = true;
  m_have_image_orientation_patient_array = false;
  m_have_image_position_patient_array = false;
  
  QByteArray qba = m_referenceDcmImage->m_image_orientation_patient_str.toLatin1();
  const char *iop = qba.data();
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "slot sort: m_image_orientation_patient_str: %s\n", iop);
    
  if (qStringToDoubleArray(m_referenceDcmImage->m_image_orientation_patient_str,
                           6, image_orientation_patient_array) != 6)
  {
    textEditSorting->append("Could not find reference image orientation.");
    sorted = false;
    return;
  }
  else
  {
    m_have_image_orientation_patient_array = true;
  }
  SbVec3f row_vct(image_orientation_patient_array[0],
                  image_orientation_patient_array[1],
                  image_orientation_patient_array[2]);
                  
  SbVec3f column_vct(image_orientation_patient_array[3],
                     image_orientation_patient_array[4],
                     image_orientation_patient_array[5]);
                     
  row_vct.normalize();
  column_vct.normalize();
  const SbVec3f plane_normal_vct = row_vct.cross(column_vct);
  plane_normal_vct.getValue(normal[0], normal[1], normal[2]);
  
  TMinDcmImageVector::iterator it;
  for(it = m_fittingDcmImageVector.begin(); it != m_fittingDcmImageVector.end(); it++)
  {
    if (qStringToDoubleArray((*it)->m_image_position_patient_str,
                             3, image_position_patient_array) == 3)
    {
      SbVec3f image_position_patient_vct(image_position_patient_array[0],
                                         image_position_patient_array[1], image_position_patient_array[2]);
                                         
      distance_to_origin = plane_normal_vct.dot(image_position_patient_vct);
      (*it)->m_distance_to_origin = distance_to_origin;
      (*it)->m_image_position_patient_array[0]
        = image_position_patient_array[0];
      (*it)->m_image_position_patient_array[1]
        = image_position_patient_array[1];
      (*it)->m_image_position_patient_array[2]
        = image_position_patient_array[2];
    }
    else
    {
      textEditSorting->append("Could not find image position.");
      pushButtonOK->setEnabled(false);
      sorted = false;
      success = false;
      msg = QString("Could not find image position.");
      QMessageBox::warning(this, "Dcmvol Problem", msg);
      break;
      //continue;
    }
  }
  if(success)
  {
    // sort!
    int i, j, flag = 1;    // set flag to 1 to start first pass
    MinDcmImage *tmp;
    int numLength = m_fittingDcmImageVector.size();
    for(i = 1; (i <= numLength) && flag; i++)
    {
      flag = 0;
      for (j = 0; j < (numLength - 1); j++)
      {
        if ((m_fittingDcmImageVector[j + 1]->m_distance_to_origin) >
            (m_fittingDcmImageVector[j]->m_distance_to_origin))
        {
          tmp = m_fittingDcmImageVector[j]; // swap elements
          m_fittingDcmImageVector[j] = m_fittingDcmImageVector[j + 1];
          m_fittingDcmImageVector[j + 1] = tmp;
          flag = 1;               // indicates that a swap occurred.
        }
      }
    }
    msg = QString("%1 image(s) fit after sort.")
          .arg(m_fittingDcmImageVector.size());
    textEditSorting->append(msg);
    
    if (m_fittingDcmImageVector.size() > 1)
    {
      m_numNiftiSlices = 0;
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "slot_sort, m_fittingDcmImageVector.size() == %i\n",
                m_fittingDcmImageVector.size());
      for(int k = 0; k < m_fittingDcmImageVector.size(); k++)
      {
        if (k > 0)
        {
          SbVec3f previous_ul_corner(m_fittingDcmImageVector[k - 1]->m_image_position_patient_array[0],
                                     m_fittingDcmImageVector[k - 1]->m_image_position_patient_array[1],
                                     m_fittingDcmImageVector[k - 1]->m_image_position_patient_array[2]);
          SbVec3f current_ul_corner(m_fittingDcmImageVector[k]->m_image_position_patient_array[0],
                                    m_fittingDcmImageVector[k]->m_image_position_patient_array[1],
                                    m_fittingDcmImageVector[k]->m_image_position_patient_array[2]);
                                    
          SbPlane previous_plane(plane_normal_vct, previous_ul_corner);
          currentImageThickness = fabs(previous_plane.getDistance(current_ul_corner));
          m_fittingDcmImageVector[k]->m_dcmImageThickness =
            currentImageThickness;
          if (k == 1)
          {
            thickness0018_0050 =
              atof(m_fittingDcmImageVector[1]->m_slice_thickness.toAscii());
            m_fittingDcmImageVector[0]->m_dcmImageThickness =
              fabs(m_fittingDcmImageVector[1]->m_dcmImageThickness);
            m_avrg_dcmImageThickness += fabs(m_fittingDcmImageVector[1]->m_dcmImageThickness);
            m_numNiftiSlices++;
          }
          if (currentImageThickness > maxImageThickness)
            maxImageThickness = currentImageThickness;
          if (currentImageThickness < minImageThickness)
            minImageThickness = currentImageThickness;
          m_numNiftiSlices++;
          m_avrg_dcmImageThickness += currentImageThickness;
        } // eo if (k > 0)
      } // eo for(int k = 0; k < m_fittingDcmImageVector.size(); k++)
      
      m_avrg_dcmImageThickness /= m_numNiftiSlices;
    } // eo if (m_fittingDcmImageVector.size() > 1)
    else if (m_fittingDcmImageVector.size() == 1)
    {
      fprintf(stdout, "slot_sort, m_fittingDcmImageVector.size() == 1\n");
      m_numNiftiSlices = 1;
      m_avrg_dcmImageThickness =
        atof(m_fittingDcmImageVector[0]->m_slice_thickness.toAscii());
      minImageThickness = m_avrg_dcmImageThickness;
      maxImageThickness = m_avrg_dcmImageThickness;
    }
    msg = QString("num slices: %1, average slice thickness: %2\n min thickness: %3\n max thickness %4")
          .arg(m_numNiftiSlices)
          .arg(m_avrg_dcmImageThickness)
          .arg(minImageThickness)
          .arg(maxImageThickness);
    textEditSorting->append(msg);
    msg = QString("thickness0018_0050: %1")
          .arg(thickness0018_0050);
    textEditSorting->append(msg);
    if (m_avrg_dcmImageThickness < 0.000001 && thickness0018_0050 > 0.01)
    {
      m_avrg_dcmImageThickness = thickness0018_0050;
      msg = QString("No thickness from image \n position patient found.\n Thickness0018_0050 used.");
      textEditSorting->append(msg);
    }
    if (m_avrg_dcmImageThickness < 0.000001 )
    {
      msg = QString("No valid volume found, slice thickness is zero.");
      QMessageBox::warning(this, "Dcmvol Problem", msg);
      //   return;
      pushButtonOK->setEnabled(false);
      success = false;
      sorted = false;
    }
  }
  else
  {
    msg = QString("no fitting images found\n");
    //    .arg(maxImageThickness);
    textEditSorting->append(msg);
  }// eo if(success)
  
  if (success)
  {
    MinDcmImage *firstImage = m_fittingDcmImageVector.at(0);
    QByteArray ba = firstImage->m_image_orientation_patient_str.toLatin1();
    const char *iop = ba.data();
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "m_image_orientation_patient_str: %s\n", iop);
      
      
    if (qStringToDoubleArray(firstImage->m_image_orientation_patient_str,
                             6, m_first_image_orientation_patient_array) == 6)
    {
      m_dcmImageOrientationLabel = makeImageOrientationLabelFromImageOrientationPatient(
                                     m_first_image_orientation_patient_array[0],
                                     m_first_image_orientation_patient_array[1],
                                     m_first_image_orientation_patient_array[2],
                                     m_first_image_orientation_patient_array[3],
                                     m_first_image_orientation_patient_array[4],
                                     m_first_image_orientation_patient_array[5]);
                                     
      // not really needed
      calculatePatientOrientation(m_first_image_orientation_patient_array[0],
                                  m_first_image_orientation_patient_array[1],
                                  m_first_image_orientation_patient_array[2]);
    }
    else
    {
      msg = QString("Image orientation (patient) not found in first image.");
      QMessageBox::warning(this, "Dcmvol Problem", msg);
      sorted = false;
    }
    
    if (qStringToDoubleArray(firstImage->m_image_position_patient_str,
                             3, m_first_image_position_patient_array) != 3)
    {
      msg = QString("Image position patient not found in first image.");
      QMessageBox::warning(this, "Dcmvol Problem", msg);
      QByteArray ba = firstImage->m_image_position_patient_str.toLatin1();
      const char *ipp = ba.data();
      
      if (SDR_VERBOSE > 2)
        fprintf(stdout, "firstImage->m_image_position_patient_str: %s\n", ipp);
        
      sorted = false;
    }
    else
    {
      m_have_image_position_patient_array = true;
    }
  }
  // find m_avrg_dcmImageThickness for selected images
  if (m_fittingDcmImageVector.size() > 1)
  {
    m_numNiftiSlices = 0;
    double max_dist;
    double min_dist;
    max_dist = m_fittingDcmImageVector[0]->m_distance_to_origin;
    min_dist = max_dist;
    
    // Can't help, calculate  m_avrg_dcmImageThickness from entire series.
    m_numNiftiSlices = m_fittingDcmImageVector.size();
    int count = 0;
    for(int k = 0; k < m_fittingDcmImageVector.size(); k++)
    {
      bool b = FALSE;
      QString numqs = (QString) m_fittingDcmImageVector.at(k)->m_instance_number;
      int numi = numqs.toInt(&b); //tmp_mini->m_instance_number.toInt(&b);
      if ((b)) // && (k >= start_at) && (k < stop_at))
      {
        double dist = m_fittingDcmImageVector[k]->m_distance_to_origin;
        if (dist < min_dist)
          min_dist = dist;
        if (dist > max_dist)
          max_dist = dist;
          
        count++;
        //printf("sorting subselection, dist to origin is %.3f\n", dist);
      } // eo if (b)
      else
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "slot_sort, no valid instance number found for slice %i\n", k);
      }
    } // eo for(int k = start_at; k < stop_at; k++)
    
    if ((m_numNiftiSlices > 0) && (count > 1))
    {
      m_avrg_dcmImageThickness = (max_dist - min_dist) / (double) (count - 1);
      
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "slot_sort, sorting subselection, min_dist %.3f, max_dist %.3f \n"
                "good slices (count) %i m_numNiftiSlices %i, m_avrg_dcmImageThickness %.3f\n",
                min_dist, max_dist, count, m_numNiftiSlices, m_avrg_dcmImageThickness);
    }
    else
    {
      success = false;
      pushButtonOK->setEnabled(false);
      sorted = false;
      textEditSorting->append("Could not find avrg dcmImageThickness.");
    }
  }
  
  if (success)
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "eo slot_sort sort succeeded\n");
    pushButtonOK->setEnabled(true);
    sorted = true;
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "eo slot_sort, sort failed\n");
  }
}


mat44 SaveVolume::mat44_transpose(mat44 in)
{
  mat44 out;
  for(unsigned int i = 0; i < 4; i++)
  {
    for(unsigned int j = 0; j < 4; j++)
    {
      out.m[i][j] = in.m[j][i];
    }
  }
  return out;
}


mat44 SaveVolume::mat44_mul( mat44 A , mat44 B )  /* multiply 2 4x4 matrices */
{
  mat44 C ;
  int i, j ;
  for( i = 0 ; i < 4 ; i++ )
    for( j = 0 ; j < 4 ; j++ )
      C.m[i][j] =  A.m[i][0] * B.m[0][j]
                   + A.m[i][1] * B.m[1][j]
                   + A.m[i][2] * B.m[2][j]
                   + A.m[i][3] * B.m[3][j];
  return C ;
}

//find 'scanner-anatomical coordinates' (qform_code > 0)\n");
// ignore for shear, not account for 0018, 1120: Gantry/Detector Tilt
// get transform from quaterion
// nim->sform_code = NIFTI_XFORM_UNKNOWN; // = 0
// nim->qform_code = NIFTI_XFORM_ALIGNED_ANAT;
void SaveVolume::findNiftiQuaternion(nifti_image *hdr) //nifti_image &hdr, IMAGELIST *imageList)
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo SaveVolume::findNiftiQuaternion\n");
  //hdr->qfac = 1;
  
  if ((! m_have_image_orientation_patient_array)
      || (! m_have_image_position_patient_array))
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "findNiftiQuaternion wo image_orientation or image_position\n"
              "set qto_xyz to indentity matrix\n");
    mat44 identMatrix;
    identMatrix.m[0][0] = 1.0f;
    identMatrix.m[1][0] = 0.0f;
    identMatrix.m[2][0] = 0.0f;
    identMatrix.m[3][0] = 0.0f;
    identMatrix.m[0][1] = 0.0f;
    identMatrix.m[1][1] = 1.0f;
    identMatrix.m[2][1] = 0.0f;
    identMatrix.m[3][1] = 0.0f;
    identMatrix.m[0][2] = 0.0f;
    identMatrix.m[1][2] = 0.0f;
    identMatrix.m[2][2] = 1.0f;
    identMatrix.m[3][2] = 0.0f;
    identMatrix.m[0][3] = 0.0f;
    identMatrix.m[1][3] = 0.0f;
    identMatrix.m[2][3] = 0.0f;
    identMatrix.m[3][3] = 1.0f;
    hdr->qto_xyz.m[0][0] = identMatrix.m[0][0];
    hdr->qto_xyz.m[0][1] = identMatrix.m[0][1];
    hdr->qto_xyz.m[0][2] = identMatrix.m[0][2];
    hdr->qto_xyz.m[0][3] = identMatrix.m[0][3];
    
    hdr->qto_xyz.m[1][0] = identMatrix.m[1][0];
    hdr->qto_xyz.m[1][1] = identMatrix.m[1][1];
    hdr->qto_xyz.m[1][2] = identMatrix.m[1][2];
    hdr->qto_xyz.m[1][3] = identMatrix.m[1][3];
    
    hdr->qto_xyz.m[2][0] = identMatrix.m[2][0];
    hdr->qto_xyz.m[2][1] = identMatrix.m[2][1];
    hdr->qto_xyz.m[2][2] = identMatrix.m[2][2];
    hdr->qto_xyz.m[2][3] = identMatrix.m[2][3];
    
    hdr->qto_xyz.m[3][0] = identMatrix.m[3][0];
    hdr->qto_xyz.m[3][1] = identMatrix.m[3][1];
    hdr->qto_xyz.m[3][2] = identMatrix.m[3][2];
    hdr->qto_xyz.m[3][3] = identMatrix.m[3][3];
    
    nifti_mat44_to_quatern(hdr->qto_xyz, &(hdr->quatern_b), &(hdr->quatern_c), &(hdr->quatern_d),
                           NULL, NULL, NULL, NULL, NULL, NULL, &(hdr->qfac));
                           
    return;
  }
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "findNiftiQuaternion WITH image_orientation or image_position\n"
            "calculate qto_xyz\n");
            
  // Get the lowest slice and call it k = 0
  float RowSagCos = m_first_image_orientation_patient_array[0];
  float RowCorCos = m_first_image_orientation_patient_array[1];
  float RowTraCos = m_first_image_orientation_patient_array[2];
  float ColSagCos = m_first_image_orientation_patient_array[3];
  float ColCorCos = m_first_image_orientation_patient_array[4];
  float ColTraCos = m_first_image_orientation_patient_array[5];
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::findNiftiQuaternion cosinus found\n");
    
  float	SagNorm = m_first_image_orientation_patient_array[1] *
                  m_first_image_orientation_patient_array[5]
                  - m_first_image_orientation_patient_array[2] * m_first_image_orientation_patient_array[4];
  float CorNorm = m_first_image_orientation_patient_array[2] *
                  m_first_image_orientation_patient_array[3]
                  - m_first_image_orientation_patient_array[0] * m_first_image_orientation_patient_array[5];
  float TraNorm = m_first_image_orientation_patient_array[0] *
                  m_first_image_orientation_patient_array[4]
                  - m_first_image_orientation_patient_array[1] * m_first_image_orientation_patient_array[3];
                  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::findNiftiQuaternion normals found\n");
    
  /*
  fprintf(stdout, "findNifiQuaternion with <%.1f, %.1f, %.1f>, <%.1f, %.1f, %.1f>, <%.1f, %.1f, %.1f>\n",
  fix_rot_row_1[0], fix_rot_row_1[1], fix_rot_row_1[2],
  fix_rot_row_2[0], fix_rot_row_2[1], fix_rot_row_2[2],
  fix_rot_row_3[0], fix_rot_row_3[1], fix_rot_row_3[2]);
  */
  mat44 matrix =
    nifti_make_orthog_mat44(RowSagCos * fix_rot_row_1[0], RowCorCos * fix_rot_row_1[1],
                            RowTraCos * fix_rot_row_1[2],
                            ColSagCos * fix_rot_row_2[0], ColCorCos * fix_rot_row_2[1], ColTraCos * fix_rot_row_2[2],
                            SagNorm   * fix_rot_row_3[0], CorNorm   * fix_rot_row_3[1], TraNorm   * fix_rot_row_3[2]);
                            
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::findNiftiQuaternion orthog matrix calculated\n");
    
  //matrix.m[0][3] = -SagPos;
  //matrix.m[1][3] = -CorPos;
  //matrix.m[2][3] =  TraPos;
  matrix.m[0][0] = -RowSagCos;
  matrix.m[0][1] = -ColSagCos;
  matrix.m[0][2] = -SagNorm;
  
  matrix.m[1][0] = -RowCorCos;
  matrix.m[1][1] = -ColCorCos;
  matrix.m[1][2] = -CorNorm;
  
  matrix.m[2][0] =  RowTraCos;
  matrix.m[2][1] =  ColTraCos;
  matrix.m[2][2] =  TraNorm;
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::findNiftiQuaternion matrix set\n");
  //printMat44(matrix);
  
  // transpose or not transpose
  // guess no!
  //matrix = mat44_transpose(matrix);
  
  hdr->qto_xyz.m[0][0] = matrix.m[0][0];
  hdr->qto_xyz.m[0][1] = matrix.m[0][1];
  hdr->qto_xyz.m[0][2] = matrix.m[0][2];
  hdr->qto_xyz.m[0][3] = matrix.m[0][3];
  
  hdr->qto_xyz.m[1][0] = matrix.m[1][0];
  hdr->qto_xyz.m[1][1] = matrix.m[1][1];
  hdr->qto_xyz.m[1][2] = matrix.m[1][2];
  hdr->qto_xyz.m[1][3] = matrix.m[1][3];
  
  hdr->qto_xyz.m[2][0] = matrix.m[2][0];
  hdr->qto_xyz.m[2][1] = matrix.m[2][1];
  hdr->qto_xyz.m[2][2] = matrix.m[2][2];
  hdr->qto_xyz.m[2][3] = matrix.m[2][3];
  
  hdr->qto_xyz.m[3][0] = matrix.m[3][0];
  hdr->qto_xyz.m[3][1] = matrix.m[3][1];
  hdr->qto_xyz.m[3][2] = matrix.m[3][2];
  hdr->qto_xyz.m[3][3] = matrix.m[3][3];
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::findNiftiQuaternion qto matrix calculated\n");
    
  nifti_mat44_to_quatern(matrix,
                         &hdr->quatern_b,
                         &hdr->quatern_c,
                         &hdr->quatern_d,
                         0, 0, 0,
                         0, 0, 0,
                         &hdr->qfac);
                         
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::findNiftiQuaternion quatern matrix set\n");
    
  // ???
  int last = m_fittingDcmImageVector.size() - 1;
  hdr->qoffset_x = -m_fittingDcmImageVector.at(last)->m_image_position_patient_array[0];
  hdr->qoffset_y = -m_fittingDcmImageVector.at(last)->m_image_position_patient_array[1];
  hdr->qoffset_z =  m_fittingDcmImageVector.at(last)->m_image_position_patient_array[2];
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "hdr->qoffset_x = %.4f, hdr->qoffset_y = %.4f, hdr->qoffset_z = %.4f\n",
            hdr->qoffset_x, hdr->qoffset_y, hdr->qoffset_z);
            
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "eo SaveVolume::findNiftiQuaternion\n");
}


// try to find 'standard space coordinates' (sform_code > 0)\n");
// allows for shear, take to account for 0018, 1120: Gantry/Detector Tilt
// ignore quaterion
// nifti will set the sto transformation from srow_*[]
// nim->sform_code = NIFTI_XFORM_SCANNER_ANAT;
// nim->qform_code = NIFTI_XFORM_UNKNOWN; // = 0
// this or getModelMatrixFromDICOM() ????
void SaveVolume::findFullNiftiTransformation(nifti_image *hdr)
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "SaveVolume::findFullNiftiTransformation\n");
    
  mat44 finalMatrix;
  finalMatrix.m[0][0] = 1.0f;
  finalMatrix.m[1][0] = 0.0f;
  finalMatrix.m[2][0] = 0.0f;
  finalMatrix.m[3][0] = 0.0f;
  finalMatrix.m[0][1] = 0.0f;
  finalMatrix.m[1][1] = 1.0f;
  finalMatrix.m[2][1] = 0.0f;
  finalMatrix.m[3][1] = 0.0f;
  finalMatrix.m[0][2] = 0.0f;
  finalMatrix.m[1][2] = 0.0f;
  finalMatrix.m[2][2] = 1.0f;
  finalMatrix.m[3][2] = 0.0f;
  finalMatrix.m[0][3] = 0.0f;
  finalMatrix.m[1][3] = 0.0f;
  finalMatrix.m[2][3] = 0.0f;
  finalMatrix.m[3][3] = 1.0f;
  
  if (SDR_VERBOSE > 0)
  {
    if (! m_have_image_orientation_patient_array)
      fprintf(stdout,
              "findFullNiftiTransformation: m_have_image_orientation_patient_array == false !!\n");
    else
      fprintf(stdout, "findFullNiftiTransformation: m_have_image_orientation_patient_array == true\n");
  }
  
  if (m_have_image_orientation_patient_array)
  {
    float RowSagCos = m_first_image_orientation_patient_array[0];
    float RowCorCos = m_first_image_orientation_patient_array[1];
    float RowTraCos = m_first_image_orientation_patient_array[2];
    float ColSagCos = m_first_image_orientation_patient_array[3];
    float ColCorCos = m_first_image_orientation_patient_array[4];
    float ColTraCos = m_first_image_orientation_patient_array[5];
    if (SDR_VERBOSE > 2)
    {
      fprintf(stdout, "findFullNiftiTransformation, image_orientation_patient row: %.4f, %.4f, %.4f\n",
              RowSagCos, RowCorCos, RowTraCos);
      fprintf(stdout, "findFullNiftiTransformation, image_orientation_patient col: %.4f, %.4f, %.4f\n",
              ColSagCos, ColCorCos, ColTraCos);
    }
    
    float SagNorm = RowCorCos * ColTraCos - RowTraCos * ColCorCos;
    float CorNorm = RowTraCos * ColSagCos - RowSagCos * ColTraCos;
    float TraNorm = RowSagCos * ColCorCos - RowCorCos * ColSagCos;
    
    double YZ = atof(m_referenceDcmImage->m_detector_tilt.toAscii());
    float shearYZ = (float) YZ;
    
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "findFullNiftiTransformation, consider detector_tilt: %f\n", shearYZ);
      
    float shearXY = 0.0f;
    float shearXZ = 0.0f;
    if (shearYZ != 0.0f)
    {
      shearYZ /= 180.0f;
      shearYZ *= acos(-1.0);
      
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "findFullNiftiTransformation, shearYZ * acos(-1) = %f rad\n", shearYZ);
    }
    else
    {
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "findFullNiftiTransformation, with shearYZ == 0.0\n");
    }
    mat44 shearMatrix;
    shearMatrix.m[0][0] = 1.0;
    shearMatrix.m[0][1] = 0.0;
    shearMatrix.m[0][2] = 0.0;
    shearMatrix.m[0][3] = 0.0;
    shearMatrix.m[1][0] = shearXY;
    shearMatrix.m[1][1] = 1.0;
    shearMatrix.m[1][2] = 0.0;
    shearMatrix.m[1][3] = 0.0;
    shearMatrix.m[2][0] = shearXZ;
    shearMatrix.m[2][1] = shearYZ;
    shearMatrix.m[2][2] = 1.0;
    shearMatrix.m[2][3] = 0.0;
    shearMatrix.m[3][0] = 0.0;
    shearMatrix.m[3][1] = 0.0;
    shearMatrix.m[3][2] = 0.0;
    shearMatrix.m[3][3] = 1.0;
    
    float scale_x = m_pixel_spacing[0];
    float scale_y = m_pixel_spacing[1];
    float scale_z = m_pixel_spacing[2];
    
    if (SDR_VERBOSE > 1)
    {
      fprintf(stdout, "findFullNiftTransformation, m_pixel_spacing: <%.4f, %.4f, %.4f>\n",
              scale_x, scale_y , scale_z);
              
      fprintf(stdout, "dims: <%i, %i, %i>\n",
              m_dcmImageWidthInt, m_dcmImageHeightInt, m_numNiftiSlices);
    }
    
    mat44 rotationMatrix;
    
    rotationMatrix.m[0][0] = -RowSagCos;
    rotationMatrix.m[1][0] = -RowCorCos;
    rotationMatrix.m[2][0] =  RowTraCos;
    rotationMatrix.m[3][0] =  0.0;
    
    rotationMatrix.m[0][1] = -ColSagCos;
    rotationMatrix.m[1][1] = -ColCorCos;
    rotationMatrix.m[2][1] =  ColTraCos;
    rotationMatrix.m[3][1] =  0.0;
    
    rotationMatrix.m[0][2] = -SagNorm;
    rotationMatrix.m[1][2] = -CorNorm;
    rotationMatrix.m[2][2] =  TraNorm;
    rotationMatrix.m[3][2] =  0.0;
    
    rotationMatrix.m[0][3] = 0.0;
    rotationMatrix.m[1][3] =  0.0;
    rotationMatrix.m[2][3] = 0.0;
    rotationMatrix.m[3][3] = 1.0;
    
    mat44 scaleMatrix;
    
    scaleMatrix.m[0][0] = scale_x;
    scaleMatrix.m[1][0] = 0.0;
    scaleMatrix.m[2][0] = 0.0;
    scaleMatrix.m[3][0] = 0.0;
    scaleMatrix.m[0][1] = 0.0;
    scaleMatrix.m[1][1] = scale_y;
    scaleMatrix.m[2][1] = 0.0;
    scaleMatrix.m[3][1] = 0.0;
    scaleMatrix.m[0][2] = 0.0;
    scaleMatrix.m[1][2] = 0.0;
    scaleMatrix.m[2][2] = scale_z;
    scaleMatrix.m[3][2] = 0.0;
    scaleMatrix.m[0][3] = 0.0;
    scaleMatrix.m[1][3] = 0.0;
    scaleMatrix.m[2][3] = 0.0;
    scaleMatrix.m[3][3] = 1.0;
    
    finalMatrix = mat44_mul(finalMatrix, shearMatrix);
    finalMatrix = mat44_mul(finalMatrix, rotationMatrix);
    
    finalMatrix = mat44_mul(finalMatrix, scaleMatrix);
    finalMatrix.m[0][3] = -m_fittingDcmImageVector.last()->m_image_position_patient_array[0];
    finalMatrix.m[1][3] = -m_fittingDcmImageVector.last()->m_image_position_patient_array[1];
    finalMatrix.m[2][3] =  m_fittingDcmImageVector.last()->m_image_position_patient_array[2];
  }
  
  hdr->sto_xyz.m[0][0] = finalMatrix.m[0][0];
  hdr->sto_xyz.m[0][1] = finalMatrix.m[0][1];
  hdr->sto_xyz.m[0][2] = finalMatrix.m[0][2];
  hdr->sto_xyz.m[0][3] = finalMatrix.m[0][3];
  hdr->sto_xyz.m[1][0] = finalMatrix.m[1][0];
  hdr->sto_xyz.m[1][1] = finalMatrix.m[1][1];
  hdr->sto_xyz.m[1][2] = finalMatrix.m[1][2];
  hdr->sto_xyz.m[1][3] = finalMatrix.m[1][3];
  
  hdr->sto_xyz.m[2][0] = finalMatrix.m[2][0];
  hdr->sto_xyz.m[2][1] = finalMatrix.m[2][1];
  hdr->sto_xyz.m[2][2] = finalMatrix.m[2][2];
  hdr->sto_xyz.m[2][3] = finalMatrix.m[2][3];
  
  hdr->sto_xyz.m[3][0] = finalMatrix.m[3][0];
  hdr->sto_xyz.m[3][1] = finalMatrix.m[3][1];
  hdr->sto_xyz.m[3][2] = finalMatrix.m[3][2];
  hdr->sto_xyz.m[3][3] = finalMatrix.m[3][3];
}


int SaveVolume::diPixelRepresentationToNiftiDatatype(int rep, bool print)
{
  int ret = DT_UNKNOWN;
  switch (rep) //(pix->getRepresentation())
  {
    case EPR_Uint8:
      if (print)
        fprintf(stdout, "DiPixelRepresentation: EPR_Uint8, i.e. nifti DT_UNSIGNED_CHAR\n");
      ret = DT_UNSIGNED_CHAR;  // same as DT_UINT8
      break;
    case EPR_Uint16:
      if (print)
        fprintf(stdout, "DiPixelRepresentation: EPR_Uint16, i.e. nifti DT_UINT16\n");
      ret = DT_UINT16;
      break;
    case EPR_Uint32:
      if (print)
        fprintf(stdout, "DiPixelRepresentation: EPR_Uint32, i.e. nifti DT_UINT32\n");
      ret = DT_UINT32;
      break;
    case EPR_Sint8:
      if (print)
        fprintf(stdout, "DiPixelRepresentation: EPR_Sint8, i.e. nifti DT_INT8\n");
      ret = DT_INT8;
      break;
    case EPR_Sint16:
      if (print)
        fprintf(stdout, "DiPixelRepresentation: EPR_Sint16, i.e. nifti DT_SIGNED_SHORT\n");
      ret = DT_SIGNED_SHORT;
      break;
    case EPR_Sint32:
      if (print)
        fprintf(stdout, "DiPixelRepresentation: EPR_Sint32, i.e. nifti DT_SIGNED_INT\n");
      ret = DT_SIGNED_INT;
      break;
    default:
      if (print)
        fprintf(stdout, "DiPixelRepresentation: unknown, i.e. nifti DT_UNKNOWN\n");
      ret = DT_UNKNOWN;
      //ODINLOG(odinlog,errorLog) <<  "pixel representation not supportet"<< STD_endl;
  }
  return ret;
}


// gantry tilt:
//slice_depth *= fabs(cos(DEG2RAD * gantry_tilt));
void SaveVolume::addExtensions(nifti_image *nim)
{
  char str[1024];
  QString qs;
  int num_bytes;
  
  if (! this->checkBoxAnonymize->isChecked())
  {
    qs = m_referenceDcmImage->m_patient_name;
    qs.prepend("PatientName: ");
    //QByteArray ba = qs.toLatin1();
    QByteArray ba = qs.toAscii();
    const char *cs = ba.data();
    strcpy(str, cs);
    num_bytes = (int) strlen(str);
    
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "patient_name: %s (%i byte)\n", str, num_bytes);
      
    nifti_add_extension(nim, str, num_bytes, NIFTI_ECODE_COMMENT);
    
    qs = m_referenceDcmImage->m_pat_birth_date;
    qs.prepend("PatientBirthDate: ");
    strcpy(str, qs.toAscii().data());
    num_bytes = (int) strlen(str);
    
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "birth date: %s (%i byte)\n", str, num_bytes);
    nifti_add_extension(nim, str, num_bytes, NIFTI_ECODE_DICOM);
  }
  
  //m_series_date; //0080 0021
  qs = m_referenceDcmImage->m_series_date;
  //qs.prepend("0x0080,0x0021: ");
  qs.prepend("SeriesDate: ");
  //SeriesDate
  strcpy(str, qs.toAscii().data());
  //strcpy(str, qs.toAscii());
  num_bytes = (int) strlen(str);
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "series date: %s (%i byte)\n", str, num_bytes);
    
  nifti_add_extension(nim, str, num_bytes, NIFTI_ECODE_DICOM);
  
  //m_series_time; //0080 0031
  qs = m_referenceDcmImage->m_series_time;
  //qs.prepend("0x0080,0x0031: ");
  qs.prepend("SeriesTime: ");
  //SeriesTime
  strcpy(str, qs.toAscii().data());
  num_bytes = (int) strlen(str);
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "series_time: %s (%i byte)\n", str, num_bytes);
  nifti_add_extension(nim, str, num_bytes, NIFTI_ECODE_DICOM);
  
  //m_modality; // 0008 0060
  qs = m_referenceDcmImage->m_modality;
  //qs.prepend("0x0008,0x0060: ");
  qs.prepend("Modality: ");
  //Modality
  strcpy(str, qs.toAscii().data());
  num_bytes = (int) strlen(str);
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "modality: %s (%i byte)\n", str, num_bytes);
  nifti_add_extension(nim, str, num_bytes, NIFTI_ECODE_DICOM);
  
  //m_series_description; //0008 103E
  qs = m_referenceDcmImage->m_series_description;
  qs.prepend("SeriesDescription: ");
  
  //SeriesDescription
  strcpy(str, qs.toAscii().data());
  num_bytes = (int) strlen(str);
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "series_description: %s (%i byte)\n", str, num_bytes);
  nifti_add_extension(nim, str, num_bytes, NIFTI_ECODE_DICOM);
  
  //m_study_description; //0008 1030
  qs = m_referenceDcmImage->m_study_description;
  qs.prepend("StudyDescription: ");
  //StudyDescription
  strcpy(str, qs.toAscii().data());
  num_bytes = (int) strlen(str);
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout, "study_description: %s (%i byte)\n", str, num_bytes);
  nifti_add_extension(nim, str, num_bytes, NIFTI_ECODE_DICOM);
  sprintf(nim->descrip, "%s", str);
}


bool SaveVolume::saveNiftiDcmInternalData()
{
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "bo SaveVolume::saveNiftiDcmInternalData\n");
    
  int min_img_num, max_img_num;
  int num_dcm_slices = 0;
  int num_iso_slices = 0;
  int num_used_slices = 0;
  int nim_type, inter_d_type;
  int output_d_type;
  
  float isoFillFactor = 1.0f;
  //int numIsoSlices = 0;
  QString errorStr;
  int allNumisArray[5000];
  int allNumisNum = 0;
  
  UCHAR8 *allDcmPixelUChar = NULL;
  UINT16 *allDcmPixelUInt16 = NULL;
  INT16  *allDcmPixelInt16 = NULL;
  UINT32 *allDcmPixelUInt32 = NULL;
  
  if (!sorted)
  {
    slot_sort();
  }
  
  /***********************/
  // bo naming
  char fn_prefix_str[1024];
  char hn_full_str[1024];
  char in_full_str[1024];
  QString fullFileName = this->m_filedir + this->m_filename;
  QFileInfo fi(fullFileName);
  QString nimPrefix = fi.baseName();
  
  QString fullNimPrefix = this->m_filedir + nimPrefix;
  strcpy(fn_prefix_str, fullNimPrefix.toLatin1().data());
  
  // 0==ANALYZE, 1==NIFTI-1 (1 file), 2==NIFTI-1 (2 files), 3==NIFTI-ASCII (1 file)
  if (this->m_filename.endsWith(".nii", Qt::CaseInsensitive))
  {
    nim_type = 1;
    //QString hq =
    strcpy(hn_full_str, fullFileName.toLatin1().data());
    strcpy(in_full_str, fullFileName.toLatin1().data());
    
  }
  else if (this->m_filename.endsWith(".hdr", Qt::CaseInsensitive))
  {
    nim_type = 0;
    QString iq = fullNimPrefix.append(".img");
    strcpy(hn_full_str, fullFileName.toLatin1().data());
    strcpy(in_full_str, iq.toLatin1().data());
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "unsupported file type\n");
    return false;
  }
  // eo naming
  
  // bo dims and spacing
  int numFrames = m_referenceDcmImage->m_dicomImage->getFrameCount();
  if (numFrames > 1)
  {
    errorStr = QString("Multi frame images are not yet considered.\n (Number of frames is %1.)\n")
               .arg(numFrames);
    textEditSorting->append(errorStr);
  }
  
  /*  clip and crop */
  min_img_num = this->spinBoxStartAtSlice->value();
  max_img_num = this->spinBoxStopAtSlice->value();
  if ((min_img_num > max_img_num) || (max_img_num < min_img_num))
  {
    errorStr = QString("Less than one slice selected.\n(min number %1, max number %2).\n")
               .arg(min_img_num) .arg(max_img_num);
    QMessageBox::critical(this, "Critical Error", errorStr);
    return false;
  }
  
  if (SDR_VERBOSE > 1)
    fprintf(stdout,
            "SaveVolume::saveNiftiDcmInternalData start at instance num %i, stop at %i, num slices: %i\n",
            min_img_num, max_img_num, (max_img_num - min_img_num + 1));
            
  int clip_left = 0;
  int clip_top = 0;
  int clip_width = m_referenceDcmImage->m_dicomImage->getWidth();
  int clip_height = m_referenceDcmImage->m_dicomImage->getHeight();
  
  int crop_x_y = spinBoxCropXY->value();
  
  // predefined uses interdata. no sense in clipping here
  if (crop_x_y > 0.0)
  {
    float one_percent_width = (float) m_referenceDcmImage->m_dicomImage->getWidth() / 100.0;
    float one_percent_height = (float) m_referenceDcmImage->m_dicomImage->getHeight() / 100.0;
    clip_left = (crop_x_y / 2) * one_percent_width;
    clip_top = (crop_x_y / 2) * one_percent_height;
    clip_width -= crop_x_y * one_percent_width;
    clip_height -= crop_x_y * one_percent_height;
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "saveNiftiDcmInternalData image number of pixel width %i, height %i\n",
              m_referenceDcmImage->m_dicomImage->getWidth(),
              m_referenceDcmImage->m_dicomImage->getHeight());
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData no x y clipping required.\n");
  }
  
  if (SDR_VERBOSE > 01)
    fprintf(stdout,
            "saveNiftiDcmInternalData clip_left %i, clip_width %i, clip_top %i, clip_height %i\n",
            clip_left, clip_width, clip_top, clip_height);
            
  QByteArray ba = m_referenceDcmImage->m_pixel_spacing.toLatin1();
  const char *spacing = ba.data();
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "saveNiftiDcmInternalData pixel spacing is %s\n",
            spacing);
  if (qStringToDoubleArray(m_referenceDcmImage->m_pixel_spacing,
                           2, m_pixel_spacing) != 2)
  {
    errorStr = QString("Pixel_spacing not found in: '%1'.\n")
               .arg(m_referenceDcmImage->m_pixel_spacing);
    QMessageBox::critical(this, "Critical Error", errorStr);
    return false;
  }
  if (m_pixel_spacing[0] != m_pixel_spacing[0])
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "pixel spacing x != pixel spacing x is not accounted for.\n");
  }
  if (m_pixel_spacing[0] <= 0.0)
  {
    m_pixel_spacing[0] = 1.0; //0.2;
    m_pixel_spacing[1] = 1.0; //0.2;
    errorStr = QString("Pixel spacing for x is <= 0\n (string is %1).\n"
                       "\n Spacing arbitrarily set to \n width %2 and height %3. \n")
               .arg(m_referenceDcmImage->m_pixel_spacing)
               .arg(m_pixel_spacing[0])
               .arg(m_pixel_spacing[1]);
    QMessageBox::critical(this, "Critical Error", errorStr);
  }
  if (m_avrg_dcmImageThickness == 0.0)
  {
    m_avrg_dcmImageThickness = 1.0; //0.5;
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "m_avrg_dcmImageThickness arbitrarily set to %.4f\n",
              m_avrg_dcmImageThickness);
  }
  else
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "m_avrg_dcmImageThickness %.4f is well calculated.\n",
              m_avrg_dcmImageThickness);
  }
  
  if (checkBoxIsometric->isChecked())
  {
    isoFillFactor = m_avrg_dcmImageThickness / m_pixel_spacing[0];
    
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData m_pixel_spacing[0] %.4f\n",
              m_pixel_spacing[0]);
              
    num_iso_slices = ceil((max_img_num - min_img_num + 1) * isoFillFactor);
    
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData isoFillFactor %.4f * pixel spacing %.4f "
              "(=%.4f) \n should result in m_avrg_dcmImageThickness %.4f\n",
              isoFillFactor, m_pixel_spacing[0],
              (isoFillFactor * m_pixel_spacing[0]),
              m_avrg_dcmImageThickness);
              
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData total number of iso slices %i\n",
              num_iso_slices);
              
    num_used_slices = num_iso_slices;
    //numIsoSlices = m_numNiftiSlices * isoFillFactor;
  }
  else
  {
    num_dcm_slices = max_img_num - min_img_num + 1;
    num_used_slices = num_dcm_slices;
  }
  
  /* pixel info */
  int numberOfPixel = m_referenceDcmImage->m_dicomImage->getWidth()
                      * m_referenceDcmImage->m_dicomImage->getHeight();
  int bytePerPixel = m_referenceDcmImage->m_dicomImage->getOutputDataSize()
                     / numberOfPixel;
                     
  int paddedDepth = pad_to_8_16_32_64(m_referenceDcmImage->m_dicomImage->getDepth());
  int samplesPerPixel = bytePerPixel / (paddedDepth / 8);
  int bytePerSample = bytePerPixel / samplesPerPixel;
  
  int dcm_width =  m_referenceDcmImage->m_dicomImage->getWidth();
  int dcm_height = m_referenceDcmImage->m_dicomImage->getHeight();
  
  if (SDR_VERBOSE > 0)
    std::cout << std::endl
              << "SaveVolume::saveNiftiDcmInternalData, m_referenceDcmImage:" << std::endl
              << "columns per slice         : " << dcm_width << std::endl
              << "rows per slice            : " << dcm_height << std::endl
              << "numberOfPixel per slice   : " << numberOfPixel << std::endl
              << "paddedDepth (bit)         : " << paddedDepth << std::endl
              << "bytePerPixel              : " << bytePerPixel << std::endl
              << "samplesPerPixel           : " << samplesPerPixel << std::endl
              << "bytePerSample             : " << bytePerSample << std::endl
              << "number of slices          : " << num_used_slices << std::endl << std::endl;
              
  // eo dims and spacing
  
  // output data is always unsigned !
  output_d_type = DT_UNKNOWN;
  if (samplesPerPixel == 1)
  {
    if (bytePerSample == 1)
      output_d_type = DT_UNSIGNED_CHAR;
    else if (bytePerSample == 2)
      output_d_type = DT_UINT16;
    else if (bytePerSample == 4)
      output_d_type = DT_UINT32;
  }
  else if (samplesPerPixel == 3)
  {
    output_d_type = DT_RGB24;
  }
  else if (samplesPerPixel == 4)
  {
    output_d_type = DT_RGBA32;
  }
  
  if (SDR_VERBOSE > 2)
  {
    fprintf(stdout, "saveNifti output datatype translates to nifti:  ");
    printNiftDatatype(output_d_type);
  }
  
  // bo get referenceInterData
  const DiPixel *referenceInterData;
  referenceInterData = (DiPixel*) m_referenceDcmImage->m_dicomImage->getInterData();
  if (referenceInterData == NULL)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData, InterData == NULL, gone return\n");
    return false;
  }
  
  int internal_pixel_count = referenceInterData->getCount();
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "saveNiftiDcmInternalData, InterData has %i pixel\n",
            internal_pixel_count);
            
  if (SDR_VERBOSE > 1)
    inter_d_type = diPixelRepresentationToNiftiDatatype(referenceInterData->getRepresentation(),
                   true);
  else
    inter_d_type = diPixelRepresentationToNiftiDatatype(referenceInterData->getRepresentation(),
                   false);
  if (inter_d_type == DT_UNKNOWN)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stderr, "saveNiftiDcmInternalData, InterData datatype is unknown, gone return false\n");
      
    return false;
  }
  
  int diPixelPlanes = referenceInterData->getPlanes();
  //number of planes (e.g. 1 for monochrome and 3 for color images)
  if (diPixelPlanes == 1)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData, InterData is monochrome\n");
  }
  else if (diPixelPlanes == 3)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData, InterData is color\n");
    inter_d_type = NIFTI_TYPE_RGB24;
  }
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout,
              "saveNiftiDcmInternalData, InterData is neither monochrome nor color, gone return false\n");
    return false;
  }
  // eo get referenceInterData
  
  // bo make nifti
  const int initDims[] = {3, clip_width, clip_height,
                          num_used_slices,
                          0, 0, 0, 0
                         };
  nifti_image *nim =  nifti_make_new_nim(initDims, inter_d_type, -1);
  if (nim == NULL)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "saveNiftiDcmInternalData, nifti_image struct == NULL, gone return\n");
    return false;
  }
  
  nim->ndim = 3;
  nim->nx = clip_width;
  nim->ny = clip_height;
  nim->nz = num_used_slices; // number of slices per volume
  nim->nt = 1;               // numFrames, number of timepoints
  nim->nu = 1;
  if ( nim->nt == 1 )
  {
    nim->ndim = 3;
  }
  else
  {
    nim->ndim = 4;
  }
  nim->nv = 1;
  nim->nw = 1;
  nim->dim[0] = 3;
  nim->dim[1] = nim->nx;
  nim->dim[2] = nim->ny;
  nim->dim[3] = nim->nz;
  nim->nvox = nim->nx * nim->ny * nim->nz * nim->nt * nim->nu * nim->nv * nim->nw;
  nim->datatype = inter_d_type;
  
  if (SDR_VERBOSE > 0)
    fprintf(stdout, "save nifti datatype %s\n",
            nifti_datatype_to_string(inter_d_type));
            
  // hdr.nbyper should be sizeof(int) for this datatype, but NIFTI does
  // something else
  nifti_datatype_sizes(nim->datatype, &(nim->nbyper), &(nim->swapsize));
  nim->dx = (float) m_pixel_spacing[0];
  nim->dy = (float) m_pixel_spacing[1];
  
  if (checkBoxIsometric->isChecked())
  {
    nim->dz = nim->dx;
  }
  else
  {
    nim->dz = m_avrg_dcmImageThickness;
  }
  m_pixel_spacing[2] = nim->dz;
  
  nim->dt = 0.0;
  nim->du = 1.0;
  nim->dv = 1.0;
  nim->dw = 1.0;
  nim->pixdim[1] = nim->dx;
  nim->pixdim[2] = nim->dy;
  nim->pixdim[3] = nim->dz;
  
  float slope = 1.0;
  float intercept = 0.0f;
  slope = atof(m_referenceDcmImage->m_rescale_slope_string.toAscii());
  intercept = atof(m_referenceDcmImage->m_rescale_intercept_string.toAscii());
  nim->scl_slope = 1.0;
  nim->scl_inter = 0.0;
  
  if (SDR_VERBOSE > 1)
  {
    fprintf(stdout, "header slope %.2f and intercept %.2f\n",
            slope, intercept);
    fprintf(stdout, "save slope %.2f and intercept %.2f\n",
            nim->scl_slope, nim->scl_inter);
  }
  
  double c = 0.0;
  double w = 0.0;
  double min = 0.0;
  double max = 0.0;
  
  bool bc, bw;
  c = (double) m_referenceDcmImage->m_win_center_string.toInt(&bc);
  w = (double) m_referenceDcmImage->m_win_width_string.toInt(&bw);
  if ((!bc) || (!bw) || (w < 1.0))
  {
    c = m_referenceDcmImage->m_dcmImageWinCenterAndWidth[0];
    w = m_referenceDcmImage->m_dcmImageWinCenterAndWidth[1];
  }
  
  if (SDR_VERBOSE > 2)
    fprintf(stdout, "try to get cal_min and cal_max from dcm header center %.2f and window %.2f\n",
            c, w);
            
  if (min != max)
  {
    w = max - min;
    c = w / 2.0;
    // bug in dcmtk ?
    w = w * slope + intercept;
    c = c * slope + intercept;
  }
  
  nim->cal_min = c - fabs(w / 2.0);               /*!< calibration parameter, minimum   */
  nim->cal_max = c + fabs(w / 2.0);   ;           /*!< calibration parameter, maximum   */
  
  if (SDR_VERBOSE > 0)
    fprintf(stdout, "save cal_min %.2f and cal_max %.2f\n",
            nim->cal_min, nim->cal_max);
  nim->qform_code = NIFTI_XFORM_SCANNER_ANAT;
  nim->sform_code = NIFTI_XFORM_UNKNOWN;
  //mat44 qto_xyz ; used here      /*!< qform: transform (i,j,k) to (x,y,z) */
  //mat44 qto_ijk ; not used here  /*!< qform: transform (x,y,z) to (i,j,k) */
  //mat44 sto_xyz ; used here      /*!< sform: transform (i,j,k) to (x,y,z) */
  //mat44 sto_ijk ; not used here  /*!< sform: transform (x,y,z) to (i,j,k) */
  nim->xyz_units = NIFTI_UNITS_MM;
  nim->toffset = 0.0;    						// Time is since 0:00 of the day
  nim->time_units = NIFTI_UNITS_MSEC;
  
  findFullNiftiTransformation(nim);
  findNiftiQuaternion(nim);
  
  nim->nifti_type = nim_type;     // 1==NIFTI-1 (1 file)
  // - should set the magic in nhdr in Write()
  nifti_set_filenames(nim, fn_prefix_str, 0, -1);
  
  addExtensions(nim);
  int allPixelCount = 0;
  float allReminders = 0.0f;
  if ((inter_d_type == DT_UNSIGNED_CHAR) || (inter_d_type == NIFTI_TYPE_UINT8))
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "inter_d_type == NIFTI_TYPE_UINT8 not yet supported\n");
      
    return false;
  }
  else if (inter_d_type == NIFTI_TYPE_INT8)
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "inter_d_type == NIFTI_TYPE_INT8 not yet supported\n");
    return false;
  }
  else if ((inter_d_type == DT_RGB) || (inter_d_type == NIFTI_TYPE_RGB24))
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "saveNifti allocate %i byte for allDcmPixelUChar\n",
              (numberOfPixel * num_used_slices * 3));
              
    allDcmPixelUChar = new UCHAR8[numberOfPixel * num_used_slices * 3];
    for (int i = m_fittingDcmImageVector.size() - 1; i > -1; i--)
    {
      bool b = false;
      int numi = m_fittingDcmImageVector.at(i)->m_instance_number.toInt(&b);
      if (! b)
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "saveNifti image number not found\n");
        //break;
        continue;
      }
      if ((numi >= min_img_num) && (numi <= max_img_num))
      {
        if (0 == uniqueIntInIntArray(numi, allNumisArray, allNumisNum))
        {
          allNumisArray[allNumisNum] = numi;
          allNumisNum++;
        }
        else
        {
          if (SDR_VERBOSE > 0)
            fprintf(stdout, "saveDcmAsNifti duplicate image number (%i) ignored.\n",
                    numi);
          continue;
        }
        if (SDR_VERBOSE > 1)
          fprintf(stdout, "saveDcmAsNifti do it for image number %i\n",
                  numi);
        DicomImage *di = m_fittingDcmImageVector.at(i)->m_dicomImage;
        if (di == NULL)
        {
          if (SDR_VERBOSE > 0)
            fprintf(stdout, "saveDcmAsNifti, di == NULL\n");
        }
        const UCHAR8 *pData = (UCHAR8*) di->getOutputData(8);
        // bo for each image loop
        for (int y = 0; y < dcm_height; y++)
        {
          for (int x = 0; x < (dcm_width * 3); x += 3)
          {
            allDcmPixelUChar[allPixelCount] = pData[y * (dcm_width * 3) + x];
            allPixelCount++;
            allDcmPixelUChar[allPixelCount] = pData[y * (dcm_width * 3) + x + 1];
            allPixelCount++;
            allDcmPixelUChar[allPixelCount] = pData[y * (dcm_width * 3) + x + 2];
            allPixelCount++;
          }
        }
      }
    } // eo for each image
    
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "saveDcmAsNifti, %i r, g, b elements found.\n",
              allPixelCount);
  }
  
  else if ((inter_d_type == DT_INT16) || (inter_d_type == NIFTI_TYPE_INT16))
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "save data type int16\n");
    allDcmPixelInt16 = new INT16[numberOfPixel * (num_used_slices + (int)floor(isoFillFactor))];
    for (int i = m_fittingDcmImageVector.size() - 1; i > -1; i--)
    {
      bool b = false;
      int numi = m_fittingDcmImageVector.at(i)->m_instance_number.toInt(&b);
      if (! b)
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "saveNiftiDcmInternalData image number not found\n");
        break;
      }
      
      int numRepeats = ceil(isoFillFactor);
      float currentReminder = isoFillFactor - (float) numRepeats;
      allReminders += currentReminder;
      if (allReminders > 1.0)
      {
        numRepeats++;
        allReminders -= 1.0;
      }
      
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "saveNiftiDcmInternalData check image number %i\n",
                numi);
      if ((numi >= min_img_num) && (numi <= max_img_num))
      {
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "saveNiftiDcmInternalData do it for image number %i\n",
                  numi);
        //DicomImage *di = m_fittingDcmImageVector.at(i)->m_dicomImage;
        
        const DiPixel *inter;
        inter = (DiPixel*) m_fittingDcmImageVector.at(i)->m_dicomImage->getInterData();
        const INT16 *pData = (INT16*) inter->getData();
        for (int r = 0; r < numRepeats; r++)
        {
          for (int y = 0; y < dcm_height; y++)
          {
            for (int x = 0; x < dcm_width; x++)
            {
              if ((x >= clip_left) && (x < clip_left + clip_width)
                  && (y >= clip_top) && (y < clip_top + clip_height))
              {
                allDcmPixelInt16[allPixelCount] = pData[y * dcm_width + x];
                allPixelCount++;
              }
            }
          }
        } // eo for each image loop
      }
    } // eo for (int i = m_fittingDcmImageVector.size() -1; i > 0; i--)
    
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "%i pixel saved for Int16\n", allPixelCount);
      
  }  // eo else if (inter_d_type == DT_INT16)
  
  else if ((inter_d_type == DT_UINT16 || inter_d_type == NIFTI_TYPE_UINT16))
  {
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "save data type uint16\n");
    allDcmPixelUInt16 = new UINT16[numberOfPixel * (num_used_slices + (int)floor(isoFillFactor))];
    for (int i = m_fittingDcmImageVector.size() - 1; i > -1; i--)
    {
      bool b = false;
      int numi = m_fittingDcmImageVector.at(i)->m_instance_number.toInt(&b);
      if (! b)
      {
        if (SDR_VERBOSE > 0)
          fprintf(stdout, "saveNiftiDcmInternalData image number not found\n");
        break;
      }
      int numRepeats = ceil(isoFillFactor);
      float currentReminder = isoFillFactor - (float) numRepeats;
      allReminders += currentReminder;
      if (allReminders > 1.0)
      {
        numRepeats++;
        allReminders -= 1.0;
      }
      
      if (SDR_VERBOSE > 1)
        fprintf(stdout, "saveNiftiDcmInternalData check image number %i\n",
                numi);
      if ((numi >= min_img_num) && (numi <= max_img_num))
      {
        if (SDR_VERBOSE > 2)
          fprintf(stdout, "saveNiftiDcmInternalData do it for image number %i\n",
                  numi);
        //DicomImage *di = m_fittingDcmImageVector.at(i)->m_dicomImage;
        
        const DiPixel *inter;
        inter = (DiPixel*) m_fittingDcmImageVector.at(i)->m_dicomImage->getInterData();
        const UINT16 *pData = (UINT16*) inter->getData();
        for (int r = 0; r < numRepeats; r++)
        {
          for (int y = 0; y < dcm_height; y++)
          {
            for (int x = 0; x < dcm_width; x++)
            {
            
              if ((x >= clip_left) && (x < clip_left + clip_width)
                  && (y >= clip_top) && (y < clip_top + clip_height))
              {
                allDcmPixelUInt16[allPixelCount] = pData[y * dcm_width + x];
                allPixelCount++;
              }
            }
          }
        } // eo for each image loop
      }
    } // eo for (int i = m_fittingDcmImageVector.size() -1; i > 0; i--)
    if (SDR_VERBOSE > 1)
      fprintf(stdout, "%i pixel saved for UInt16\n", allPixelCount);
  }  // // eo else if (inter_d_type == DT_UINT16)
  else
  {
    if (SDR_VERBOSE > 0)
      fprintf(stdout, "data type unknown, gone return false\n");
    return false;
  }
  
  if ((inter_d_type == DT_INT16) || (inter_d_type == NIFTI_TYPE_INT16))
  {
    nim->data = (void*)(allDcmPixelInt16);
    nifti_image_write(nim);
  }
  else if ((inter_d_type == DT_UINT16) || (inter_d_type == NIFTI_TYPE_UINT16))
  {
    nim->data = (void*)(allDcmPixelUInt16);
    nifti_image_write(nim);
  }
  else if ((inter_d_type == DT_RGB) || (inter_d_type == NIFTI_TYPE_RGB24))
  {
    if (SDR_VERBOSE > 2)
      fprintf(stdout, "saveNifti should write nifti file now!\n");
    nim->data = (void*)(allDcmPixelUChar);
    if (nim->data == NULL)
    {
      if (SDR_VERBOSE > 0)
        fprintf(stdout, "saveNifti save RGB, nim->data == NULL, return false.\n");
      return false;
    }
    nifti_image_write(nim);
  }
  /*
  if (allDcmPixelUChar != NULL)
    delete [] allDcmPixelUChar;
  
  if (allDcmPixelUInt16 != NULL)
    delete[] allDcmPixelUInt16;
  
  if (allDcmPixelUInt32  != NULL)
    delete [] allDcmPixelUInt32;
  */
  nifti_image_free(nim);
  
  // eo make nifti
  /***********************/
  /*
  Window Center (0028,1050) and Window Width (0028,1051) specify a linear conversion
  from stored pixel values (after any Modality LUT or Rescale Slope and
  Intercept specified in the IOD have been applied) to values to be displayed.
  Window Center contains the input value that is the center of the window.
  Window Width contains the width of the window.
  */
  return true;
}


bool SaveVolume::slotFileSave()
{
  QString msg;
  
  if (this->m_filename.isEmpty())
    return slotFileSaveAs();
    
  QString fullFileName = this->m_filedir + this->m_filename;
  
  bool written = false;
  if ((this->m_filename.endsWith(".nii", Qt::CaseInsensitive))
      || (this->m_filename.endsWith(".hdr", Qt::CaseInsensitive)))
  {
    written = saveNiftiDcmInternalData(); //Crop();
  }
  else
  {
    QMessageBox::information(this, "debug", "Unknown file type.");
  }
  
  if (written)
  {
    msg = QString("File '%1' written to disk.")
          .arg(this->m_filename);
    QMessageBox::information(this, "SDR2 Information", msg);
  }
  else
  {
    msg = QString("Something went wrong while writing file '%1' to disk.")
          .arg(this->m_filename);
    QMessageBox::information(this, "SDR2 Information", msg);
  }
  return true;
}


bool SaveVolume::slotFileSaveAs()
{
  QString msg;
  
  MinDcmImage *tmpMinDcmImage = m_fittingDcmImageVector.at(0);
  
  int c = tmpMinDcmImage->m_dicomImage->getFrameCount();
  if (c > 1)
  {
    msg = QString("Dicom image is multiframe (%1 frames.\n Only first frame will be saved.")
          .arg(c);
    QMessageBox::warning(this, "SDR2 Issue", msg);
    this->m_filename = "";
    this->sorted = false;
  }
  
  if(! tmpMinDcmImage->m_dicomImage->isMonochrome())
  {
    msg = QString("Saving of colored Dicom images is experimental.\n");
    
    QMessageBox::warning(this, "Critical Error", msg);
    this->m_filename = "";
    this->sorted = false;
  }
  
  QString dir = m_filedir;
  //QString fname = m_filename;
  QString fn = QFileDialog::getSaveFileName(this,
               tr("Save as..."),
               dir,
               tr("Nifti Files (*.nii);;Analyse Files (*.hdr);;All Files (*)"));
               
  if (fn.isEmpty())
    return false;
  if (! (fn.endsWith(".nii", Qt::CaseInsensitive)
         || fn.endsWith(".hdr", Qt::CaseInsensitive)))
    fn += ".nii"; // default
    
  //setCurrentFileName(fn);
  QFileInfo fi(fn);
  QString absFilePath = fi.absoluteFilePath();
  m_filename = fi.fileName();
  m_filedir = absFilePath.left(absFilePath.length() - m_filename.length());
  this->sorted = false;
  this->hide();
  return slotFileSave();
}


void SaveVolume::bubbleSortFittingDcmImageVector()
{
  int i, j, flag = 1;    // set flag to 1 to start first pass
  MinDcmImage *tmp;
  int numLength = m_fittingDcmImageVector.size();
  for(i = 1; (i <= numLength) && flag; i++)
  {
    flag = 0;
    for (j = 0; j < (numLength - 1); j++)
    {
      if (m_fittingDcmImageVector[j + 1]->m_distance_to_origin > m_fittingDcmImageVector[j +
          1]->m_distance_to_origin)
      {
        tmp = m_fittingDcmImageVector[j]; // swap elements
        m_fittingDcmImageVector[j] = m_fittingDcmImageVector[j + 1];
        m_fittingDcmImageVector[j + 1] = tmp;
        flag = 1;               // indicates that a swap occurred.
      }
    }
  }
  return;   //arrays are passed to functions by address; nothing is returned
}


QString SaveVolume::getMajorAxisFromPatientRelativeDirectionCosine(double x, double y, double z)
{
  QString axis = NULL;
  
  QString orientationX = x < 0 ? "R" : "L";
  QString orientationY = y < 0 ? "A" : "P";
  QString orientationZ = z < 0 ? "F" : "H";
  
  double absX = ABS(x); //abs(x);
  double absY = ABS(y);
  double absZ = ABS(z);
  
  // The tests here really don't need to check the other dimensions,
  // just the threshold, since the sum of the squares should be == 1.0
  // but just in case ...
  
  if (absX > obliquityThresholdCosineValue && absX > absY && absX > absZ)
  {
    axis = orientationX;
  }
  else if (absY > obliquityThresholdCosineValue && absY > absX && absY > absZ)
  {
    axis = orientationY;
  }
  else if (absZ > obliquityThresholdCosineValue && absZ > absX && absZ > absY)
  {
    axis = orientationZ;
  }
  return axis;
}


QString SaveVolume::makeImageOrientationLabelFromImageOrientationPatient(
  double rowX, double rowY, double rowZ,
  double colX, double colY, double colZ)
{
  QString label = NULL;
  QString rowAxis = getMajorAxisFromPatientRelativeDirectionCosine(rowX, rowY, rowZ);
  QString colAxis = getMajorAxisFromPatientRelativeDirectionCosine(colX, colY, colZ);
  if (rowAxis != NULL && colAxis != NULL)
  {
    if (((rowAxis == "R") || (rowAxis == "L"))
        && ((colAxis == "A") || (colAxis == "P")))
      label = "AXIAL";
    else if (((colAxis == "R") || (colAxis == "L"))
             && ((rowAxis == "A") || (rowAxis == "P")))
      label = "AXIAL";
    else if (((rowAxis == "R") || (rowAxis == "L"))
             && ((colAxis == "H") || (colAxis == "F")))
      label = "CORONAL";
    else if (((colAxis == "R") || (colAxis == "L"))
             && ((rowAxis == "H") || (rowAxis == "F")))
      label = "CORONAL";
    else if (((rowAxis == "A") || (rowAxis == "P"))
             && ((colAxis == "H") || (colAxis == "F")))
      label = "SAGITTAL";
    else if (((colAxis == "A") || (colAxis == "P"))
             && ((rowAxis == "H") || (rowAxis == "F")))
      label = "SAGITTAL";
  }
  else
  {
    label = "OBLIQUE";
  }
  
  char tmp_str[1024];
  strcpy(tmp_str, label.toLatin1().data());
  if (SDR_VERBOSE > 0)
    fprintf(stdout, "nifti prefix name / image orientation: '%s'\n", tmp_str);
  return label;
}

